"use strict";
const fs = require("fs");
const fetch_1 = require("../../shared/fetch");
const i18n = require("../../shared/i18n");
const index_1 = require("./index");
const { superpowers: { appApiVersion: appApiVersion } } = JSON.parse(fs.readFileSync(`${__dirname}/../../package.json`, { encoding: "utf8" }));
function openServer(serverEntry) {
    index_1.clearActiveTab();
    let tabElt = index_1.tabStrip.tabsRoot.querySelector(`li[data-server-id="${serverEntry.id}"]`);
    let paneElt = index_1.panesElt.querySelector(`div[data-server-id="${serverEntry.id}"]`);
    if (tabElt == null) {
        tabElt = makeServerTab(serverEntry);
        index_1.tabStrip.tabsRoot.appendChild(tabElt);
        paneElt = makeServerPane(serverEntry);
        index_1.panesElt.appendChild(paneElt);
    }
    tabElt.classList.add("active");
    paneElt.hidden = false;
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = openServer;
function makeServerTab(serverEntry) {
    const tabElt = document.createElement("li");
    tabElt.dataset["serverId"] = serverEntry.id;
    const iconElt = document.createElement("img");
    iconElt.className = "icon";
    iconElt.src = "images/tabs/server.svg";
    tabElt.appendChild(iconElt);
    const labelElt = document.createElement("div");
    labelElt.className = "label";
    tabElt.appendChild(labelElt);
    const locationElt = document.createElement("div");
    locationElt.className = "location";
    locationElt.textContent = `${serverEntry.hostname}:${serverEntry.port}`;
    labelElt.appendChild(locationElt);
    const nameElt = document.createElement("div");
    nameElt.className = "name";
    nameElt.textContent = serverEntry.label;
    labelElt.appendChild(nameElt);
    const closeButton = document.createElement("button");
    closeButton.className = "close";
    tabElt.appendChild(closeButton);
    return tabElt;
}
function makeServerPane(serverEntry) {
    const paneElt = document.createElement("div");
    paneElt.dataset["serverId"] = serverEntry.id;
    const connectingElt = document.createElement("div");
    connectingElt.className = "connecting";
    paneElt.appendChild(connectingElt);
    const statusElt = document.createElement("div");
    connectingElt.appendChild(statusElt);
    const retryButton = document.createElement("button");
    retryButton.textContent = i18n.t("common:server.tryAgain");
    connectingElt.appendChild(retryButton);
    function onRetryButtonClick(event) {
        event.preventDefault();
        tryConnecting();
    }
    retryButton.addEventListener("click", onRetryButtonClick);
    const host = serverEntry.hostname + (serverEntry.port != null ? `:${serverEntry.port}` : "");
    function tryConnecting() {
        statusElt.textContent = i18n.t("common:server.connecting", { host: host });
        retryButton.hidden = true;
        fetch_1.default(`http://${host}/superpowers.json`, "json", onFetchJSON);
    }
    function onFetchJSON(err, serverInfo) {
        if (err != null) {
            statusElt.textContent = i18n.t("common:server.errors.superpowersJSON", { host: host });
            retryButton.hidden = false;
            return;
        }
        if (serverInfo == null || typeof serverInfo !== "object") {
            statusElt.textContent = i18n.t("common:server.errors.notSuperpowers", { host: host });
            retryButton.hidden = false;
            return;
        }
        if (serverInfo.appApiVersion !== appApiVersion) {
            statusElt.textContent = i18n.t("common:server.errors.incompatibleVersion", { host: host, serverVersion: serverInfo.appApiVersion, appVersion: appApiVersion });
            retryButton.hidden = false;
            return;
        }
        const webviewElt = document.createElement("webview");
        webviewElt.preload = `${__dirname}/../../SupApp/index.js`;
        function clearEventListeners() {
            webviewElt.removeEventListener("did-finish-load", onLoad);
            webviewElt.removeEventListener("did-fail-load", onError);
        }
        function onLoad() {
            clearEventListeners();
            paneElt.removeChild(connectingElt);
        }
        function onError() {
            clearEventListeners();
            paneElt.removeChild(webviewElt);
            statusElt.textContent = "Failed to load iframe";
        }
        webviewElt.addEventListener("did-finish-load", onLoad);
        webviewElt.addEventListener("did-fail-load", onError);
        webviewElt.src = `http://${host}`;
        paneElt.appendChild(webviewElt);
        webviewElt.focus();
    }
    tryConnecting();
    return paneElt;
}
