/// <reference path="../typings/fuzzy.d.ts" />
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var fuzzy = require("fuzzy");
var TreeView = require("dnd-tree-view");
var Dialogs = require("simple-dialogs");
var i18n = require("./i18n");
var html_1 = require("./html");
var FindAssetDialog = (function (_super) {
    __extends(FindAssetDialog, _super);
    function FindAssetDialog(entries, editorsByAssetType, callback) {
        var _this = this;
        _super.call(this, callback);
        this.entries = entries;
        this.editorsByAssetType = editorsByAssetType;
        this.entriesByPath = {};
        this.pathsList = [];
        this.pathsWithoutSlashesList = [];
        this.entryElts = [];
        this.onSearchInput = function (event) {
            var results = fuzzy.filter(_this.searchElt.value, _this.pathsList);
            var resultsWithoutSlashes = fuzzy.filter(_this.searchElt.value, _this.pathsWithoutSlashesList);
            results = results.concat(resultsWithoutSlashes);
            _this.treeView.clearSelection();
            _this.treeView.treeRoot.innerHTML = "";
            if (results.length === 0)
                return;
            results.sort(function (a, b) { return b.score - a.score; });
            var index = results.length - 1;
            for (var i = 0; i < results.length; i++) {
                var result = results[index];
                if (result.original.search(new RegExp(_this.searchElt.value, "i")) !== -1) {
                    results.splice(index, 1);
                    results.unshift(result);
                }
                else {
                    index -= 1;
                }
            }
            for (var _i = 0, results_1 = results; _i < results_1.length; _i++) {
                var result = results_1[_i];
                var liElt = _this.entryElts[result.index];
                _this.treeView.append(liElt, "item");
            }
            _this.treeView.addToSelection(_this.treeView.treeRoot.firstChild);
        };
        this.onSearchKeyDown = function (event) {
            if (event.keyCode === 38 /* Up */) {
                event.stopPropagation();
                event.preventDefault();
                _this.treeView.moveVertically(-1);
            }
            else if (event.keyCode === 40 /* Down */) {
                event.stopPropagation();
                event.preventDefault();
                _this.treeView.moveVertically(1);
            }
        };
        this.dialogElt.classList.add("find-asset-dialog");
        var searchGroup = html_1.default("div", "group", { parent: this.formElt, style: { display: "flex" } });
        this.searchElt = html_1.default("input", {
            parent: searchGroup, type: "search",
            placeholder: i18n.t("common:searchPlaceholder"),
            style: { flex: "1 1 0" }
        });
        this.searchElt.addEventListener("input", this.onSearchInput);
        this.searchElt.addEventListener("keydown", this.onSearchKeyDown);
        this.searchElt.focus();
        var treeViewContainer = html_1.default("div", "assets-tree-view", { parent: this.formElt });
        this.treeView = new TreeView(treeViewContainer, { multipleSelection: false });
        this.treeView.on("activate", function () { _this.submit(); });
        this.entries.walk(function (node) {
            if (node.type == null || editorsByAssetType[node.type] == null)
                return;
            var path = _this.entries.getPathFromId(node.id);
            _this.entriesByPath[path] = node;
            _this.pathsList.push(path);
            _this.pathsWithoutSlashesList.push(path.replace(/\//g, " "));
            var liElt = _this.createEntryElement(node);
            _this.entryElts.push(liElt);
            _this.treeView.append(liElt, "item");
        });
        this.treeView.addToSelection(this.treeView.treeRoot.firstChild);
    }
    FindAssetDialog.prototype.createEntryElement = function (entry) {
        var liElt = document.createElement("li");
        liElt.dataset["id"] = entry.id;
        var iconElt = document.createElement("img");
        iconElt.draggable = false;
        iconElt.src = "/systems/" + SupCore.system.id + "/plugins/" + this.editorsByAssetType[entry.type].pluginPath + "/editors/" + entry.type + "/icon.svg";
        liElt.appendChild(iconElt);
        var nameSpan = document.createElement("span");
        nameSpan.className = "name";
        nameSpan.textContent = this.entries.getPathFromId(entry.id);
        liElt.appendChild(nameSpan);
        return liElt;
    };
    FindAssetDialog.prototype.submit = function () {
        if (this.treeView.selectedNodes.length === 0)
            return;
        _super.prototype.submit.call(this, this.treeView.selectedNodes[0].dataset["id"]);
    };
    return FindAssetDialog;
}(Dialogs.BaseDialog));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = FindAssetDialog;
