"use strict";
var ProjectClient = (function () {
    function ProjectClient(socket, options) {
        var _this = this;
        this.entriesSubscribers = [];
        this.assetsById = {};
        this.subscribersByAssetId = {};
        this.resourcesById = {};
        this.subscribersByResourceId = {};
        this.onWelcome = function (clientId) {
            _this.id = clientId;
        };
        this.onAssetReceived = function (assetId, assetType, err, assetData) {
            // FIXME: The asset was probably trashed in the meantime, handle that
            if (err != null) {
                console.warn("Got an error in ProjectClient.onAssetReceived: " + err);
                return;
            }
            var subscribers = _this.subscribersByAssetId[assetId];
            if (subscribers == null)
                return;
            var asset = null;
            if (assetData != null) {
                asset = _this.assetsById[assetId] = new SupCore.system.data.assetClasses[assetType](assetId, assetData);
                asset.client_load();
            }
            for (var _i = 0, subscribers_1 = subscribers; _i < subscribers_1.length; _i++) {
                var subscriber = subscribers_1[_i];
                if (subscriber.onAssetReceived != null)
                    subscriber.onAssetReceived(assetId, asset);
            }
        };
        this.onAssetEdited = function (assetId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var subscribers = _this.subscribersByAssetId[assetId];
            if (subscribers == null)
                return;
            var asset = _this.assetsById[assetId];
            Object.getPrototypeOf(asset)[("client_" + command)].apply(asset, args);
            for (var _a = 0, subscribers_2 = subscribers; _a < subscribers_2.length; _a++) {
                var subscriber = subscribers_2[_a];
                if (subscriber.onAssetEdited != null)
                    subscriber.onAssetEdited.apply(subscriber, [assetId, command].concat(args));
            }
        };
        this.onAssetTrashed = function (assetId) {
            var subscribers = _this.subscribersByAssetId[assetId];
            if (subscribers == null)
                return;
            for (var _i = 0, subscribers_3 = subscribers; _i < subscribers_3.length; _i++) {
                var subscriber = subscribers_3[_i];
                if (subscriber.onAssetTrashed != null)
                    subscriber.onAssetTrashed(assetId);
            }
            _this.assetsById[assetId].client_unload();
            delete _this.assetsById[assetId];
            delete _this.subscribersByAssetId[assetId];
        };
        this.onAssetRestored = function (assetId, assetType, assetData) {
            var subscribers = _this.subscribersByAssetId[assetId];
            if (subscribers == null)
                return;
            _this.assetsById[assetId].client_unload();
            var asset = _this.assetsById[assetId] = new SupCore.system.data.assetClasses[assetType](assetId, assetData);
            asset.client_load();
            for (var _i = 0, subscribers_4 = subscribers; _i < subscribers_4.length; _i++) {
                var subscriber = subscribers_4[_i];
                if (subscriber.onAssetRestored != null)
                    subscriber.onAssetRestored(assetId, asset);
            }
        };
        this.onResourceReceived = function (resourceId, err, resourceData) {
            if (err != null) {
                console.warn("Got an error in ProjectClient.onResourceReceived: " + err);
                return;
            }
            var subscribers = _this.subscribersByResourceId[resourceId];
            if (subscribers == null)
                return;
            var resource = null;
            if (resourceData != null)
                resource = _this.resourcesById[resourceId] = new SupCore.system.data.resourceClasses[resourceId](resourceId, resourceData);
            for (var _i = 0, subscribers_5 = subscribers; _i < subscribers_5.length; _i++) {
                var subscriber = subscribers_5[_i];
                if (subscriber.onResourceReceived != null)
                    subscriber.onResourceReceived(resourceId, resource);
            }
        };
        this.onResourceEdited = function (resourceId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var subscribers = _this.subscribersByResourceId[resourceId];
            if (subscribers == null)
                return;
            var resource = _this.resourcesById[resourceId];
            Object.getPrototypeOf(resource)[("client_" + command)].apply(resource, args);
            for (var _a = 0, subscribers_6 = subscribers; _a < subscribers_6.length; _a++) {
                var subscriber = subscribers_6[_a];
                if (subscriber.onResourceEdited != null)
                    subscriber.onResourceEdited.apply(subscriber, [resourceId, command].concat(args));
            }
        };
        this.onEntriesReceived = function (err, entries, nextEntryId) {
            _this.entries = new SupCore.Data.Entries(entries, nextEntryId);
            _this.socket.on("add:entries", _this.onEntryAdded);
            _this.socket.on("move:entries", _this.onEntryMoved);
            _this.socket.on("setProperty:entries", _this.onSetEntryProperty);
            _this.socket.on("save:entries", _this.onEntrySaved);
            _this.socket.on("trash:entries", _this.onEntryTrashed);
            for (var _i = 0, _a = _this.entriesSubscribers; _i < _a.length; _i++) {
                var subscriber = _a[_i];
                if (subscriber.onEntriesReceived != null)
                    subscriber.onEntriesReceived(_this.entries);
            }
        };
        this.onEntryAdded = function (entry, parentId, index) {
            _this.entries.client_add(entry, parentId, index);
            for (var _i = 0, _a = _this.entriesSubscribers; _i < _a.length; _i++) {
                var subscriber = _a[_i];
                if (subscriber.onEntryAdded != null)
                    subscriber.onEntryAdded(entry, parentId, index);
            }
        };
        this.onEntryMoved = function (id, parentId, index) {
            _this.entries.client_move(id, parentId, index);
            for (var _i = 0, _a = _this.entriesSubscribers; _i < _a.length; _i++) {
                var subscriber = _a[_i];
                if (subscriber.onEntryMoved != null)
                    subscriber.onEntryMoved(id, parentId, index);
            }
        };
        this.onSetEntryProperty = function (id, key, value) {
            _this.entries.client_setProperty(id, key, value);
            for (var _i = 0, _a = _this.entriesSubscribers; _i < _a.length; _i++) {
                var subscriber = _a[_i];
                if (subscriber.onSetEntryProperty != null)
                    subscriber.onSetEntryProperty(id, key, value);
            }
        };
        this.onEntrySaved = function (id, revisionId, revisionName) {
            _this.entries.client_save(id, revisionId, revisionName);
            for (var _i = 0, _a = _this.entriesSubscribers; _i < _a.length; _i++) {
                var subscriber = _a[_i];
                if (subscriber.onEntrySaved != null)
                    subscriber.onEntrySaved(id, revisionId, revisionName);
            }
        };
        this.onEntryTrashed = function (id) {
            _this.entries.client_remove(id);
            for (var _i = 0, _a = _this.entriesSubscribers; _i < _a.length; _i++) {
                var subscriber = _a[_i];
                if (subscriber.onEntryTrashed != null)
                    subscriber.onEntryTrashed(id);
            }
        };
        this.socket = socket;
        this.socket.on("welcome", this.onWelcome);
        this.socket.on("edit:assets", this.onAssetEdited);
        this.socket.on("trash:assets", this.onAssetTrashed);
        this.socket.on("restore:assets", this.onAssetRestored);
        this.socket.on("edit:resources", this.onResourceEdited);
        // Allow keeping an entries subscription alive at all times
        // Used in the scene editor to avoid constantly unsub'ing & resub'ing
        this.keepEntriesSubscription = options != null && options.subEntries;
        if (this.keepEntriesSubscription)
            this.socket.emit("sub", "entries", null, this.onEntriesReceived);
    }
    ProjectClient.prototype.subEntries = function (subscriber) {
        this.entriesSubscribers.push(subscriber);
        if (this.entriesSubscribers.length === 1 && !this.keepEntriesSubscription) {
            this.socket.emit("sub", "entries", null, this.onEntriesReceived);
        }
        else if (this.entries != null)
            subscriber.onEntriesReceived(this.entries);
    };
    ProjectClient.prototype.unsubEntries = function (subscriber) {
        this.entriesSubscribers.splice(this.entriesSubscribers.indexOf(subscriber), 1);
        if (this.entriesSubscribers.length === 0 && !this.keepEntriesSubscription) {
            this.socket.emit("unsub", "entries");
            this.socket.off("add:entries", this.onEntryAdded);
            this.socket.off("move:entries", this.onEntryMoved);
            this.socket.off("setProperty:entries", this.onSetEntryProperty);
            this.socket.off("save:entries", this.onEntrySaved);
            this.socket.off("trash:entries", this.onEntryTrashed);
            this.entries = null;
        }
    };
    ProjectClient.prototype.subAsset = function (assetId, assetType, subscriber) {
        var subscribers = this.subscribersByAssetId[assetId];
        if (subscribers == null) {
            subscribers = this.subscribersByAssetId[assetId] = [];
            this.socket.emit("sub", "assets", assetId, this.onAssetReceived.bind(this, assetId, assetType));
        }
        else {
            var asset = this.assetsById[assetId];
            if (asset != null && subscriber.onAssetReceived != null)
                subscriber.onAssetReceived(assetId, asset);
        }
        subscribers.push(subscriber);
    };
    ProjectClient.prototype.unsubAsset = function (assetId, subscriber) {
        var subscribers = this.subscribersByAssetId[assetId];
        if (subscribers == null)
            return;
        var index = subscribers.indexOf(subscriber);
        if (index === -1)
            return;
        subscribers.splice(index, 1);
        if (subscribers.length === 0) {
            delete this.subscribersByAssetId[assetId];
            if (this.assetsById[assetId] != null) {
                this.assetsById[assetId].client_unload();
                delete this.assetsById[assetId];
            }
            this.socket.emit("unsub", "assets", assetId);
        }
    };
    ProjectClient.prototype.editAsset = function (assetId, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var callback;
        if (typeof args[args.length - 1] === "function")
            callback = args.pop();
        args.push(function (err, ack) {
            if (err != null) {
                new SupClient.Dialogs.InfoDialog(err);
                return;
            }
            if (callback != null)
                callback(ack);
        });
        (_a = this.socket).emit.apply(_a, ["edit:assets", assetId, command].concat(args));
        var _a;
    };
    ProjectClient.prototype.editAssetNoErrorHandling = function (assetId, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        (_a = this.socket).emit.apply(_a, ["edit:assets", assetId, command].concat(args));
        var _a;
    };
    ProjectClient.prototype.getAssetRevision = function (assetId, assetType, revisionId, onRevisionReceivedCallback) {
        this.socket.emit("getRevision:assets", assetId, revisionId, function (err, assetData) {
            if (err != null) {
                /* tslint:disable:no-unused-expression */
                new SupClient.Dialogs.InfoDialog(err);
                /* tslint:enable:no-unused-expression */
                return;
            }
            var asset = new SupCore.system.data.assetClasses[assetType](assetId, assetData);
            asset.client_load();
            onRevisionReceivedCallback(assetId, asset);
        });
    };
    ProjectClient.prototype.subResource = function (resourceId, subscriber) {
        var subscribers = this.subscribersByResourceId[resourceId];
        if (subscribers == null) {
            subscribers = this.subscribersByResourceId[resourceId] = [];
            this.socket.emit("sub", "resources", resourceId, this.onResourceReceived.bind(this, resourceId));
        }
        else {
            var resource = this.resourcesById[resourceId];
            if (resource != null && subscriber.onResourceReceived != null)
                subscriber.onResourceReceived(resourceId, resource);
        }
        subscribers.push(subscriber);
    };
    ProjectClient.prototype.unsubResource = function (resourceId, subscriber) {
        var subscribers = this.subscribersByResourceId[resourceId];
        if (subscribers == null)
            return;
        var index = subscribers.indexOf(subscriber);
        if (index === -1)
            return;
        subscribers.splice(index, 1);
        if (subscribers.length === 0) {
            delete this.subscribersByResourceId[resourceId];
            if (this.resourcesById[resourceId] != null) {
                delete this.resourcesById[resourceId];
            }
            this.socket.emit("unsub", "resources", resourceId);
        }
    };
    ProjectClient.prototype.editResource = function (resourceId, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var callback;
        if (typeof args[args.length - 1] === "function")
            callback = args.pop();
        args.push(function (err, id) {
            if (err != null) {
                new SupClient.Dialogs.InfoDialog(err);
                return;
            }
            if (callback != null)
                callback(id);
        });
        (_a = this.socket).emit.apply(_a, ["edit:resources", resourceId, command].concat(args));
        var _a;
    };
    return ProjectClient;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = ProjectClient;
