"use strict";
var fs = require("fs");
var cookies = require("js-cookie");
var path = require("path");
var _ = require("lodash");
var SupClient = require("./index");
exports.languageIds = fs.readdirSync(__dirname + "/../../public/locales");
var languageCode = cookies.get("supLanguage");
var i18nFallbackContexts = {};
var i18nContexts = {};
var commonLocalesLoaded = false;
function load(files, callback) {
    var firstLoad = !commonLocalesLoaded;
    function onLoadFinished() {
        if (firstLoad) {
            setupDefaultDialogLabels();
            setupHotkeyTitles();
        }
        callback();
    }
    if (languageCode === "none") {
        onLoadFinished();
        return;
    }
    if (!commonLocalesLoaded) {
        files.unshift({ root: "/", name: "common" });
        commonLocalesLoaded = true;
    }
    var filesToLoad = files.length;
    if (filesToLoad === 0) {
        onLoadFinished();
        return;
    }
    if (languageCode !== "en")
        filesToLoad *= 2;
    var loadFile = function (languageCode, file, root) {
        var filePath = path.join(file.root, "locales/" + languageCode, file.name + ".json");
        SupClient.fetch(filePath, "json", function (err, response) {
            if (err != null) {
                filesToLoad -= 1;
                if (filesToLoad === 0)
                    onLoadFinished();
            }
            else {
                var context = file.context != null ? file.context : file.name;
                if (root[context] == null)
                    root[context] = response;
                else
                    root[context] = _.merge(root[context], response);
                filesToLoad -= 1;
                if (filesToLoad === 0)
                    onLoadFinished();
            }
        });
    };
    for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
        var file = files_1[_i];
        loadFile(languageCode, file, i18nContexts);
        if (languageCode !== "en" && languageCode !== "none")
            loadFile("en", file, i18nFallbackContexts);
    }
}
exports.load = load;
function t(key, variables) {
    if (variables === void 0) { variables = {}; }
    if (languageCode === "none")
        return key;
    var _a = key.split(":"), context = _a[0], keys = _a[1];
    var keyParts = keys.split(".");
    var value = i18nContexts[context];
    if (value == null)
        return fallbackT(key, variables);
    for (var _i = 0, keyParts_1 = keyParts; _i < keyParts_1.length; _i++) {
        var keyPart = keyParts_1[_i];
        value = value[keyPart];
        if (value == null)
            return fallbackT(key, variables);
    }
    if (typeof value === "string")
        return insertVariables(value, variables);
    else
        return key;
}
exports.t = t;
function fallbackT(key, variables) {
    if (variables === void 0) { variables = {}; }
    var _a = key.split(":"), context = _a[0], keys = _a[1];
    var keyParts = keys.split(".");
    var valueOrText = i18nFallbackContexts[context];
    if (valueOrText == null)
        return key;
    for (var _i = 0, keyParts_2 = keyParts; _i < keyParts_2.length; _i++) {
        var keyPart = keyParts_2[_i];
        valueOrText = valueOrText[keyPart];
        if (valueOrText == null)
            return key;
    }
    if (typeof valueOrText === "string")
        return insertVariables(valueOrText, variables);
    else
        return key;
}
function insertVariables(text, variables) {
    var index = 0;
    do {
        index = text.indexOf("${", index);
        if (index !== -1) {
            var endIndex = text.indexOf("}", index);
            var key = text.slice(index + 2, endIndex);
            var value = variables[key] != null ? variables[key] : "\"" + key + "\" is missing";
            text = text.slice(0, index) + value + text.slice(endIndex + 1);
            index += 1;
        }
    } while (index !== -1);
    return text;
}
function setupDefaultDialogLabels() {
    for (var label in SupClient.Dialogs.BaseDialog.defaultLabels) {
        SupClient.Dialogs.BaseDialog.defaultLabels[label] = t("common:actions." + label);
    }
}
function setupHotkeyTitles() {
    var hotkeyButtons = document.querySelectorAll("[data-hotkey]");
    for (var _i = 0, hotkeyButtons_1 = hotkeyButtons; _i < hotkeyButtons_1.length; _i++) {
        var hotkeyButton = hotkeyButtons_1[_i];
        var hotkeys = hotkeyButton.dataset["hotkey"].split("+");
        var hotkeyComplete = "";
        for (var _a = 0, hotkeys_1 = hotkeys; _a < hotkeys_1.length; _a++) {
            var hotkey = hotkeys_1[_a];
            var hotkeyPartKey = void 0;
            if (hotkey === "control" && window.navigator.platform === "MacIntel")
                hotkeyPartKey = "common:hotkeys.command";
            else
                hotkeyPartKey = "common:hotkeys." + hotkey;
            var hotkeyPartString = t(hotkeyPartKey);
            if (hotkeyComplete !== "")
                hotkeyComplete += "+";
            if (hotkeyPartString === hotkeyPartKey)
                hotkeyComplete += hotkey;
            else
                hotkeyComplete += hotkeyPartString;
        }
        hotkeyButton.title += " (" + hotkeyComplete + ")";
    }
}
