"use strict";
var io = require("socket.io-client");
var url = require("url");
var querystring = require("querystring");
var cookies = require("js-cookie");
exports.cookies = cookies;
var fetch_1 = require("./fetch");
exports.fetch = fetch_1.default;
var loadScript_1 = require("./loadScript");
exports.loadScript = loadScript_1.default;
var readFile_1 = require("./readFile");
exports.readFile = readFile_1.default;
var ProjectClient_1 = require("./ProjectClient");
exports.ProjectClient = ProjectClient_1.default;
var events_1 = require("./events");
exports.setupHotkeys = events_1.setupHotkeys;
exports.setupHelpCallback = events_1.setupHelpCallback;
var table = require("./table");
exports.table = table;
var Dialogs = require("simple-dialogs");
exports.Dialogs = Dialogs;
var FindAssetDialog_1 = require("./FindAssetDialog");
var i18n = require("./i18n");
exports.i18n = i18n;
var html_1 = require("./html");
exports.html = html_1.default;
require("./events");
var ResizeHandle = require("resize-handle");
exports.query = querystring.parse(window.location.search.slice(1));
Dialogs.FindAssetDialog = FindAssetDialog_1.default;
// Refuses filesystem-unsafe characters
// See http://superuser.com/q/358855
exports.namePattern = "[^\\\\/:*?\"<>|\\[\\]]+";
// Expose SupApp to iframes
if (global.SupApp == null) {
    global.SupApp = (top.SupApp != null) ? top.SupApp : null;
}
// Initialize empty system
SupCore.system = new SupCore.System("", "");
var plugins = {};
var scriptPathRegex = /^\/systems\/([^\/])+\/plugins\/([^\/])+\/([^\/])+/;
function registerPlugin(contextName, pluginName, plugin) {
    if (plugins[contextName] == null)
        plugins[contextName] = {};
    if (plugins[contextName][pluginName] != null) {
        console.error("SupClient.registerPlugin: Tried to register two or more plugins " +
            ("named \"" + pluginName + "\" in context \"" + contextName + "\""));
        return;
    }
    var scriptURL = url.parse(document.currentScript.src);
    var pluginPath = scriptPathRegex.exec(scriptURL.pathname)[0];
    plugins[contextName][pluginName] = { path: pluginPath, content: plugin };
}
exports.registerPlugin = registerPlugin;
function getPlugins(contextName) {
    return plugins[contextName];
}
exports.getPlugins = getPlugins;
// Plugins list
function connect(projectId, options) {
    if (options == null)
        options = {};
    if (options.reconnection == null)
        options.reconnection = false;
    var namespace = (projectId != null) ? "project:" + projectId : "hub";
    var socket = io.connect(window.location.protocol + "//" + window.location.host + "/" + namespace, { transports: ["websocket"], reconnection: options.reconnection });
    socket.on("welcome", function (clientId, config) {
        SupCore.system.id = config.systemId;
    });
    return socket;
}
exports.connect = connect;
function onAssetTrashed() {
    document.body.innerHTML = "";
    var h1 = document.createElement("h1");
    h1.textContent = "This asset has been trashed.";
    var div = document.createElement("div");
    div.className = "superpowers-error";
    div.appendChild(h1);
    document.body.appendChild(div);
}
exports.onAssetTrashed = onAssetTrashed;
function onDisconnected() {
    document.body.innerHTML = "";
    var h1 = document.createElement("h1");
    h1.textContent = "You were disconnected.";
    var button = document.createElement("button");
    button.textContent = "Reconnect";
    button.addEventListener("click", function () { location.reload(); });
    var div = document.createElement("div");
    div.className = "superpowers-error";
    div.appendChild(h1);
    div.appendChild(button);
    document.body.appendChild(div);
}
exports.onDisconnected = onDisconnected;
function getTreeViewInsertionPoint(treeView) {
    var selectedElt = treeView.selectedNodes[0];
    var parentId;
    var index;
    if (selectedElt != null) {
        if (selectedElt.classList.contains("group")) {
            parentId = selectedElt.dataset["id"];
        }
        else {
            if (selectedElt.parentElement.classList.contains("children")) {
                parentId = selectedElt.parentElement.previousElementSibling != null ? selectedElt.parentElement.previousElementSibling.dataset["id"] : null;
            }
            index = 1;
            while (selectedElt.previousElementSibling != null) {
                selectedElt = selectedElt.previousElementSibling;
                if (selectedElt.tagName === "LI")
                    index++;
            }
        }
    }
    return { parentId: parentId, index: index };
}
exports.getTreeViewInsertionPoint = getTreeViewInsertionPoint;
function getTreeViewSiblingInsertionPoint(treeView) {
    var selectedElt = treeView.selectedNodes[0];
    var parentId = selectedElt.parentElement.previousElementSibling != null ? selectedElt.parentElement.previousElementSibling.dataset["id"] : null;
    var index = 1;
    while (selectedElt.previousElementSibling != null) {
        selectedElt = selectedElt.previousElementSibling;
        if (selectedElt.tagName === "LI")
            index++;
    }
    return { parentId: parentId, index: index };
}
exports.getTreeViewSiblingInsertionPoint = getTreeViewSiblingInsertionPoint;
function getTreeViewDropPoint(dropLocation, treeById) {
    var parentId;
    var index;
    var parentNode;
    var targetEntryId = dropLocation.target.dataset["id"];
    switch (dropLocation.where) {
        case "inside":
            {
                if (targetEntryId != null) {
                    parentNode = treeById.byId[targetEntryId];
                    index = parentNode.children.length;
                }
                else {
                    index = 0;
                }
            }
            break;
        case "above":
        case "below":
            {
                var targetNode = treeById.byId[targetEntryId];
                parentNode = treeById.parentNodesById[targetNode.id];
                index = (parentNode != null) ? parentNode.children.indexOf(targetNode) : treeById.pub.indexOf(targetNode);
                if (dropLocation.where === "below")
                    index++;
            }
            break;
    }
    if (parentNode != null)
        parentId = parentNode.id;
    return { parentId: parentId, index: index };
}
exports.getTreeViewDropPoint = getTreeViewDropPoint;
function getListViewDropIndex(dropLocation, listById, reversed) {
    if (reversed === void 0) { reversed = false; }
    var targetEntryId = dropLocation.target.dataset["id"];
    var targetNode = listById.byId[targetEntryId];
    var index = listById.pub.indexOf(targetNode);
    if (!reversed && dropLocation.where === "below")
        index++;
    if (reversed && dropLocation.where === "above")
        index++;
    return index;
}
exports.getListViewDropIndex = getListViewDropIndex;
function findEntryByPath(entries, path) {
    var parts = (typeof path === "string") ? path.split("/") : path;
    var foundEntry;
    entries.every(function (entry) {
        if (entry.name === parts[0]) {
            if (parts.length === 1) {
                foundEntry = entry;
                return false;
            }
            if (entry.children == null)
                return true;
            foundEntry = findEntryByPath(entry.children, parts.slice(1));
            return false;
        }
        else
            return true;
    });
    return foundEntry;
}
exports.findEntryByPath = findEntryByPath;
function openEntry(entryId, state) {
    window.parent.postMessage({ type: "openEntry", id: entryId, state: state }, window.location.origin);
}
exports.openEntry = openEntry;
function setEntryRevisionDisabled(disabled) {
    window.parent.postMessage({ type: "setEntryRevisionDisabled", id: exports.query.asset, disabled: disabled }, window.location.origin);
}
exports.setEntryRevisionDisabled = setEntryRevisionDisabled;
function setupCollapsablePane(paneElt, refreshCallback) {
    var handle = new ResizeHandle(paneElt, "bottom");
    if (refreshCallback != null)
        handle.on("drag", function () { refreshCallback(); });
    var statusElt = paneElt.querySelector(".header");
    var buttonElt = document.createElement("button");
    buttonElt.classList.add("toggle");
    statusElt.appendChild(buttonElt);
    var contentElt = paneElt.querySelector(".content");
    var collaspe = function (collapsed) {
        contentElt.hidden = collapsed;
        buttonElt.textContent = collapsed ? "+" : "–";
        if (refreshCallback != null)
            refreshCallback();
    };
    collaspe(paneElt.classList.contains("collapsed"));
    statusElt.addEventListener("click", function (event) { collaspe(paneElt.classList.toggle("collapsed")); });
}
exports.setupCollapsablePane = setupCollapsablePane;
