"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var events_1 = require("events");
var html_1 = require("./html");
var FindAssetDialog_1 = require("./FindAssetDialog");
function createTable(parent) {
    var table = html_1.default("table", { parent: parent });
    var tbody = html_1.default("tbody", { parent: table });
    return { table: table, tbody: tbody };
}
exports.createTable = createTable;
function appendRow(parentTableBody, name, options) {
    var row = html_1.default("tr", { parent: parentTableBody });
    var labelCell = html_1.default("th", { parent: row });
    var checkbox;
    if (options != null && options.checkbox) {
        var container = html_1.default("div", { parent: labelCell });
        html_1.default("div", { parent: container, textContent: name, title: options.title });
        checkbox = html_1.default("input", { parent: container, type: "checkbox" });
    }
    else {
        labelCell.textContent = name;
        if (options != null && options.title != null)
            labelCell.title = options.title;
    }
    var valueCell = html_1.default("td", { parent: row });
    return { row: row, labelCell: labelCell, valueCell: valueCell, checkbox: checkbox };
}
exports.appendRow = appendRow;
function appendHeader(parentTableBody, text) {
    var headerRow = html_1.default("tr", { parent: parentTableBody });
    html_1.default("th", { parent: headerRow, textContent: text, colSpan: 2 });
    return headerRow;
}
exports.appendHeader = appendHeader;
function appendTextField(parent, value) {
    return html_1.default("input", { parent: parent, type: "text", value: value });
}
exports.appendTextField = appendTextField;
function appendTextAreaField(parent, value) {
    return html_1.default("textarea", { parent: parent, value: value });
}
exports.appendTextAreaField = appendTextAreaField;
;
function appendNumberField(parent, value, options) {
    var input = html_1.default("input", { parent: parent, type: "number", value: value.toString() });
    if (options != null) {
        if (options.min != null)
            input.min = options.min.toString();
        if (options.max != null)
            input.max = options.max.toString();
        if (options.step != null)
            input.step = options.step.toString();
    }
    return input;
}
exports.appendNumberField = appendNumberField;
function appendNumberFields(parent, values, options) {
    var inputsParent = html_1.default("div", "inputs", { parent: parent });
    var inputs = [];
    for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
        var value = values_1[_i];
        inputs.push(appendNumberField(inputsParent, value, options));
    }
    return inputs;
}
exports.appendNumberFields = appendNumberFields;
function appendBooleanField(parent, checked) {
    return html_1.default("input", { parent: parent, type: "checkbox", checked: checked });
}
exports.appendBooleanField = appendBooleanField;
function appendSelectBox(parent, options, initialValue) {
    if (initialValue === void 0) { initialValue = ""; }
    var selectInput = html_1.default("select", { parent: parent });
    for (var value in options)
        appendSelectOption(selectInput, value, options[value]);
    selectInput.value = initialValue;
    return selectInput;
}
exports.appendSelectBox = appendSelectBox;
function appendSelectOption(parent, value, label) {
    return html_1.default("option", { parent: parent, value: value, textContent: label });
}
exports.appendSelectOption = appendSelectOption;
function appendSelectOptionGroup(parent, label) {
    return html_1.default("optgroup", { parent: parent, label: label, textContent: label });
}
exports.appendSelectOptionGroup = appendSelectOptionGroup;
function appendColorField(parent, value) {
    var colorParent = html_1.default("div", "inputs", { parent: parent });
    var textField = appendTextField(colorParent, value);
    textField.classList.add("color");
    var pickerField = html_1.default("input", { parent: colorParent, type: "color", value: "#" + value });
    return { textField: textField, pickerField: pickerField };
}
exports.appendColorField = appendColorField;
function appendSliderField(parent, value, options) {
    var sliderParent = html_1.default("div", "inputs", { parent: parent });
    var sliderField = html_1.default("input", { parent: sliderParent, type: "range", value: value.toString(), style: { flex: "2" } });
    if (options != null) {
        if (options.min != null)
            sliderField.min = options.min.toString();
        if (options.max != null)
            sliderField.max = options.max.toString();
        if (options.sliderStep != null)
            sliderField.step = options.sliderStep.toString();
    }
    var numberField = appendNumberField(sliderParent, value, options);
    return { sliderField: sliderField, numberField: numberField };
}
exports.appendSliderField = appendSliderField;
function appendAssetField(parent, assetId, assetType, projectClient) {
    var assetParent = html_1.default("div", "inputs", { parent: parent });
    var textField = html_1.default("input", { parent: assetParent, type: "text", readOnly: true, style: { cursor: "pointer" } });
    var buttonElt = html_1.default("button", { parent: assetParent, disabled: true, textContent: SupClient.i18n.t("common:actions.select") });
    var pluginPath;
    var assetSubscriber = new AssetFieldSubscriber(assetId, projectClient);
    assetSubscriber.on("change", function (assetId) {
        textField.value = assetId == null ? "" : assetSubscriber.entries.byId[assetId] == null ? "???" : assetSubscriber.entries.getPathFromId(assetId);
        buttonElt.textContent = SupClient.i18n.t("common:actions." + (assetId == null ? "select" : "clear"));
        buttonElt.disabled = pluginPath == null;
    });
    SupClient.fetch("/systems/" + SupCore.system.id + "/plugins.json", "json", function (err, pluginsInfo) {
        pluginPath = pluginsInfo.paths.editors[assetType];
        if (assetSubscriber.entries != null)
            buttonElt.disabled = false;
    });
    textField.addEventListener("click", function (event) {
        if (assetSubscriber.assetId != null) {
            SupClient.openEntry(assetSubscriber.assetId);
        }
        else {
            new FindAssetDialog_1.default(projectClient.entries, (_a = {}, _a[assetType] = { pluginPath: pluginPath }, _a), function (assetId) { if (assetId != null)
                assetSubscriber.selectAssetId(assetId); });
        }
        var _a;
    });
    textField.addEventListener("dragover", function (event) {
        if (!buttonElt.disabled)
            event.preventDefault();
    });
    textField.addEventListener("drop", function (event) {
        var entryId = event.dataTransfer.getData("application/vnd.superpowers.entry").split(",")[0];
        if (typeof entryId !== "string")
            return;
        var entry = assetSubscriber.entries.byId[entryId];
        if (entry == null || entry.type !== assetType)
            return;
        assetSubscriber.selectAssetId(entryId);
    });
    buttonElt.addEventListener("click", function (event) {
        if (assetSubscriber.assetId != null) {
            assetSubscriber.selectAssetId(null);
            return;
        }
        new FindAssetDialog_1.default(projectClient.entries, (_a = {}, _a[assetType] = { pluginPath: pluginPath }, _a), function (assetId) { if (assetId != null)
            assetSubscriber.selectAssetId(assetId); });
        var _a;
    });
    return assetSubscriber;
}
exports.appendAssetField = appendAssetField;
var AssetFieldSubscriber = (function (_super) {
    __extends(AssetFieldSubscriber, _super);
    function AssetFieldSubscriber(assetId, projectClient) {
        _super.call(this);
        this.assetId = assetId;
        this.projectClient = projectClient;
        this.projectClient.subEntries(this);
    }
    AssetFieldSubscriber.prototype.destroy = function () {
        this.projectClient.unsubEntries(this);
    };
    AssetFieldSubscriber.prototype.onEntriesReceived = function (entries) {
        var _this = this;
        this.entries = entries;
        setTimeout(function () { _this.emit("change", _this.assetId); }, 1);
    };
    AssetFieldSubscriber.prototype.onEntryAdded = function (entry, parentId, index) { };
    AssetFieldSubscriber.prototype.onEntryMoved = function (id, parentId, index) {
        this.emit("change", this.assetId);
    };
    AssetFieldSubscriber.prototype.onSetEntryProperty = function (id, key, value) {
        if (key === "name")
            this.emit("change", this.assetId);
    };
    AssetFieldSubscriber.prototype.onEntryTrashed = function (id) {
        if (id === this.assetId)
            this.emit("change", this.assetId);
    };
    AssetFieldSubscriber.prototype.selectAssetId = function (assetId) {
        this.onChangeAssetId(assetId);
        this.emit("select", assetId);
    };
    AssetFieldSubscriber.prototype.onChangeAssetId = function (assetId) {
        this.assetId = assetId;
        this.emit("change", this.assetId);
    };
    return AssetFieldSubscriber;
}(events_1.EventEmitter));
