"use strict";
var fs = require("fs");
var path = require("path");
exports.systems = {};
function shouldIgnoreFolder(pluginName) { return pluginName.indexOf(".") !== -1 || pluginName === "node_modules"; }
var System = (function () {
    function System(id, folderName) {
        this.id = id;
        this.folderName = folderName;
        this.plugins = {};
        this.data = new SystemData(this);
    }
    System.prototype.requireForAllPlugins = function (filePath) {
        var pluginsPath = path.resolve(SupCore.systemsPath + "/" + this.folderName + "/plugins");
        for (var _i = 0, _a = fs.readdirSync(pluginsPath); _i < _a.length; _i++) {
            var pluginAuthor = _a[_i];
            var pluginAuthorPath = pluginsPath + "/" + pluginAuthor;
            if (shouldIgnoreFolder(pluginAuthor))
                continue;
            for (var _b = 0, _c = fs.readdirSync(pluginAuthorPath); _b < _c.length; _b++) {
                var pluginName = _c[_b];
                if (shouldIgnoreFolder(pluginName))
                    continue;
                var completeFilePath = pluginAuthorPath + "/" + pluginName + "/" + filePath;
                if (fs.existsSync(completeFilePath)) {
                    /* tslint:disable */
                    require(completeFilePath);
                }
            }
        }
    };
    System.prototype.registerPlugin = function (contextName, pluginName, plugin) {
        if (this.plugins[contextName] == null)
            this.plugins[contextName] = {};
        if (this.plugins[contextName][pluginName] != null) {
            console.error("SupCore.system.registerPlugin: Tried to register two or more plugins " +
                ("named \"" + pluginName + "\" in context \"" + contextName + "\", system \"" + this.id + "\""));
        }
        this.plugins[contextName][pluginName] = plugin;
    };
    System.prototype.getPlugins = function (contextName) {
        return this.plugins[contextName];
    };
    return System;
}());
exports.System = System;
var SystemData = (function () {
    function SystemData(system) {
        this.system = system;
        this.assetClasses = {};
        this.resourceClasses = {};
    }
    SystemData.prototype.registerAssetClass = function (name, assetClass) {
        if (this.assetClasses[name] != null) {
            console.log("SystemData.registerAssetClass: Tried to register two or more asset classes named \"" + name + "\" in system \"" + this.system.id + "\"");
            return;
        }
        this.assetClasses[name] = assetClass;
        return;
    };
    SystemData.prototype.registerResource = function (id, resourceClass) {
        if (this.resourceClasses[id] != null) {
            console.log("SystemData.registerResource: Tried to register two or more plugin resources named \"" + id + "\" in system \"" + this.system.id + "\"");
            return;
        }
        this.resourceClasses[id] = resourceClass;
    };
    return SystemData;
}());
