"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var CreateOrEditProjectDialog = (function (_super) {
    __extends(CreateOrEditProjectDialog, _super);
    function CreateOrEditProjectDialog(systemsById, options, callback) {
        var _this = this;
        _super.call(this, callback);
        this.iconFile = null;
        this.onIconChange = function (event) {
            if (_this.iconInputElt.files.length === 0) {
                _this.iconFile = null;
                if (_this.existingProject == null)
                    _this.iconElt.src = "/images/default-project-icon.png";
                else
                    _this.iconElt.src = "/projects/" + _this.existingProject.id + "/icon.png";
            }
            else {
                _this.iconFile = _this.iconInputElt.files[0];
                var reader = new FileReader();
                reader.addEventListener("load", function (event) { _this.iconElt.src = event.target.result; });
                reader.readAsDataURL(_this.iconFile);
            }
        };
        this.onFieldKeyDown = function (event) {
            if (event.keyCode === 13 /* Return */) {
                event.preventDefault();
                _this.submit();
            }
        };
        this.onProjectTypeChange = function () {
            if (_this.projectType != null) {
                var path = _this.projectType.systemId + "." + _this.projectType.templateName;
                var oldOptionElt = _this.projectTypeSelectElt.querySelector("option[value=\"" + path + "\"]");
                var oldTemplate = _this.getTemplate(_this.projectType.systemId, _this.projectType.templateName);
                oldOptionElt.textContent = oldTemplate.title;
            }
            var _a = _this.projectTypeSelectElt.value.split("."), systemId = _a[0], templateName = _a[1];
            _this.projectType = { systemId: systemId, templateName: templateName };
            var template = _this.getTemplate(systemId, templateName);
            var systemTitle = SupClient.i18n.t("system-" + systemId + ":title");
            _this.projectTypeSelectElt.querySelector("option:checked").textContent = systemTitle + " \u2014 " + template.title;
            _this.templateDescriptionElt.textContent = template.description;
            var systemDescription = SupClient.i18n.t("system-" + systemId + ":description");
            _this.systemDescriptionElt.textContent = systemDescription;
            if (systemDescription.length === 0 && template.description.length === 0) {
                _this.systemDescriptionElt.textContent = "(No description provided)";
            }
        };
        this.systemsById = systemsById;
        if (options == null)
            options = {};
        if (options.autoOpen == null)
            options.autoOpen = true;
        this.existingProject = options.existingProject;
        // Prompt
        SupClient.html("div", "group", {
            parent: this.formElt,
            textContent: SupClient.i18n.t(this.existingProject == null ? "hub:newProject.prompt" : "hub:editDetails.prompt")
        });
        var containerElt = SupClient.html("div", "group", { parent: this.formElt, style: { display: "flex" } });
        // Icon
        this.iconInputElt = SupClient.html("input", { type: "file", hidden: true, accept: "image/png", parent: containerElt });
        var iconButtonElt = SupClient.html("button", {
            type: "button", parent: containerElt,
            style: {
                cursor: "pointer",
                background: "transparent",
                width: "72px", height: "72px",
                margin: "0", padding: "0", border: "0",
                fontSize: "0"
            }
        });
        iconButtonElt.addEventListener("click", function () { return _this.iconInputElt.click(); });
        var iconSrc = options.existingProject == null ? "/images/default-project-icon.png" : ("/projects/" + options.existingProject.id + "/icon.png");
        this.iconElt = SupClient.html("img", {
            src: iconSrc, draggable: false, parent: iconButtonElt,
            style: {
                width: "72px", height: "72px",
                border: "1px solid rgba(0,0,0,0.2)",
                borderRadius: "4px",
                background: "#eee"
            }
        });
        this.iconInputElt.addEventListener("change", this.onIconChange);
        var textContainerElt = SupClient.html("div", { parent: containerElt });
        textContainerElt.style.flex = "1";
        textContainerElt.style.display = "flex";
        textContainerElt.style.flexFlow = "column";
        textContainerElt.style.marginLeft = "0.5em";
        // Name
        this.nameInputElt = SupClient.html("input", {
            parent: textContainerElt,
            required: true,
            placeholder: SupClient.i18n.t("hub:newProject.namePlaceholder"),
            pattern: SupClient.namePattern,
            title: SupClient.i18n.t("common:namePatternDescription")
        });
        this.nameInputElt.style.marginBottom = "0.5em";
        // Description
        this.descriptionInputElt = SupClient.html("textarea", {
            parent: textContainerElt, placeholder: SupClient.i18n.t("hub:newProject.descriptionPlaceholder"),
            style: { flex: "1", resize: "none" }
        });
        this.descriptionInputElt.addEventListener("keypress", this.onFieldKeyDown);
        // Down
        var downElt = SupClient.html("div", { style: { display: "flex", alignItems: "center" } });
        if (options.existingProject == null) {
            // Project type
            this.projectTypeSelectElt = SupClient.html("select", { parent: this.formElt, style: { marginBottom: "0.5em" } });
            for (var systemId in systemsById) {
                var systemInfo = systemsById[systemId];
                var optGroupElt = SupClient.html("optgroup", {
                    parent: this.projectTypeSelectElt,
                    label: SupClient.i18n.t("system-" + systemId + ":title")
                });
                SupClient.html("option", {
                    parent: optGroupElt, value: systemId + ".empty",
                    textContent: SupClient.i18n.t("hub:newProject.emptyProject.title")
                });
                for (var _i = 0, systemInfo_1 = systemInfo; _i < systemInfo_1.length; _i++) {
                    var templateName = systemInfo_1[_i];
                    SupClient.html("option", {
                        parent: optGroupElt, value: systemId + "." + templateName,
                        textContent: SupClient.i18n.t(systemId + "-" + templateName + ":title")
                    });
                }
            }
            // Template description
            var descriptionContainerElt = SupClient.html("div", {
                parent: this.formElt,
                style: {
                    backgroundColor: "#eee",
                    border: "1px solid #ccc",
                    padding: "0.5em",
                    color: "#444",
                    marginBottom: "0.5em"
                }
            });
            this.templateDescriptionElt = SupClient.html("div", "template-description", { parent: descriptionContainerElt });
            this.systemDescriptionElt = SupClient.html("div", "system-description", { parent: descriptionContainerElt });
            this.onProjectTypeChange();
            // Auto-open checkbox
            this.openCheckboxElt = SupClient.html("input", {
                parent: downElt,
                type: "checkbox", id: "auto-open-checkbox",
                checked: options.autoOpen,
                style: { margin: "0 0.5em 0 0" }
            });
            SupClient.html("label", {
                parent: downElt,
                textContent: SupClient.i18n.t("hub:newProject.autoOpen"),
                htmlFor: "auto-open-checkbox",
                style: { flex: "1", margin: "0" }
            });
        }
        this.formElt.appendChild(downElt);
        // Buttons
        var buttonsElt = SupClient.html("div", "buttons", { parent: downElt });
        if (options.existingProject != null)
            buttonsElt.style.flex = "1";
        buttonsElt.className = "buttons";
        var cancelButtonElt = SupClient.html("button", "cancel-button", {
            parent: buttonsElt,
            type: "button", textContent: SupClient.i18n.t("common:actions.cancel")
        });
        cancelButtonElt.addEventListener("click", function (event) { event.preventDefault(); _this.cancel(); });
        this.validateButtonElt = SupClient.html("button", "validate-button", {
            textContent: SupClient.i18n.t(options.existingProject == null ? "common:actions.create" : "common:actions.update"),
        });
        if (navigator.platform === "Win32")
            buttonsElt.insertBefore(this.validateButtonElt, cancelButtonElt);
        else
            buttonsElt.appendChild(this.validateButtonElt);
        // Existing project
        if (options.existingProject != null) {
            this.nameInputElt.value = options.existingProject.name;
            this.descriptionInputElt.value = options.existingProject.description;
        }
        else {
            this.projectTypeSelectElt.addEventListener("change", this.onProjectTypeChange);
            this.projectTypeSelectElt.addEventListener("keydown", this.onFieldKeyDown);
        }
        this.nameInputElt.focus();
    }
    CreateOrEditProjectDialog.prototype.submit = function () {
        var systemId = null;
        var templateName = null;
        if (this.projectTypeSelectElt != null)
            _a = this.projectTypeSelectElt.value.split("."), systemId = _a[0], templateName = _a[1];
        var project = {
            name: this.nameInputElt.value,
            description: this.descriptionInputElt.value,
            systemId: systemId,
            template: templateName !== "empty" ? templateName : null,
            icon: this.iconFile
        };
        _super.prototype.submit.call(this, { project: project, open: (this.openCheckboxElt != null) ? this.openCheckboxElt.checked : null });
        var _a;
    };
    CreateOrEditProjectDialog.prototype.getTemplate = function (systemId, templateName) {
        if (templateName !== "empty") {
            return {
                title: SupClient.i18n.t(systemId + "-" + templateName + ":title"),
                description: SupClient.i18n.t(systemId + "-" + templateName + ":description")
            };
        }
        return {
            title: SupClient.i18n.t("hub:newProject.emptyProject.title"),
            description: SupClient.i18n.t("hub:newProject.emptyProject.description")
        };
    };
    return CreateOrEditProjectDialog;
}(SupClient.Dialogs.BaseDialog));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = CreateOrEditProjectDialog;
