"use strict";
var CreateOrEditProjectDialog_1 = require("./CreateOrEditProjectDialog");
var async = require("async");
var TreeView = require("dnd-tree-view");
var data = {};
var ui = {};
var socket;
var port = (window.location.port.length === 0) ? "80" : window.location.port;
var languageNamesById = {};
if (localStorage.getItem("superpowers-dev-mode") != null)
    languageNamesById["none"] = "None";
function start() {
    document.querySelector(".server-name").textContent = SupClient.i18n.t("hub:serverAddress", { hostname: window.location.hostname, port: port });
    ui.projectsTreeView = new TreeView(document.querySelector(".projects-tree-view"), { multipleSelection: false });
    ui.projectsTreeView.on("selectionChange", onProjectSelectionChange);
    ui.projectsTreeView.on("activate", onProjectActivate);
    document.querySelector(".projects-buttons .new-project").addEventListener("click", onNewProjectClick);
    document.querySelector(".projects-buttons .open-project").addEventListener("click", onProjectActivate);
    document.querySelector(".projects-buttons .edit-project").addEventListener("click", onEditProjectClick);
    var selectLanguageElt = document.querySelector("select.language");
    var languageIds = Object.keys(languageNamesById);
    languageIds.sort(function (a, b) {
        // Always sort "None" at the end
        if (a === "none")
            return 1;
        if (b === "none")
            return -1;
        return languageNamesById[a].localeCompare(languageNamesById[b]);
    });
    for (var _i = 0, languageIds_1 = languageIds; _i < languageIds_1.length; _i++) {
        var languageId = languageIds_1[_i];
        SupClient.html("option", { parent: selectLanguageElt, value: languageId, textContent: languageNamesById[languageId] });
    }
    selectLanguageElt.value = SupClient.cookies.get("supLanguage");
    document.querySelector("select.language").addEventListener("change", function (event) {
        SupClient.cookies.set("supLanguage", event.target.value, { expires: 7 });
        window.location.reload();
    });
    socket = SupClient.connect(null, { reconnection: true });
    socket.on("error", onConnectionError);
    socket.on("connect", onConnected);
    socket.on("disconnect", onDisconnected);
    socket.on("add:projects", onProjectAdded);
    socket.on("setProperty:projects", onSetProjectProperty);
    socket.on("updateIcon:projects", onUpdateProjectIcon);
}
var i18nFiles = [{ root: "/", name: "hub" }];
loadSystemsInfo(function () {
    async.each(SupClient.i18n.languageIds, function (languageId, cb) {
        SupClient.fetch("/locales/" + languageId + "/common.json", "json", function (err, data) {
            languageNamesById[languageId] = data.activeLanguage;
            cb();
        });
    }, function () {
        SupClient.i18n.load(i18nFiles, start);
    });
});
function loadSystemsInfo(callback) {
    data.systemsById = {};
    SupClient.fetch("/systems.json", "json", function (err, systemsInfo) {
        async.each(systemsInfo.list, function (systemId, cb) {
            i18nFiles.push({ root: "/systems/" + systemId, name: "system", context: "system-" + systemId });
            SupClient.fetch("/systems/" + systemId + "/templates.json", "json", function (err, templatesList) {
                for (var _i = 0, templatesList_1 = templatesList; _i < templatesList_1.length; _i++) {
                    var templateName = templatesList_1[_i];
                    i18nFiles.push({
                        root: "/systems/" + systemId + "/templates/" + templateName,
                        name: "template",
                        context: systemId + "-" + templateName
                    });
                }
                data.systemsById[systemId] = templatesList;
                cb();
            });
        }, function () { callback(); });
    });
}
// Network callbacks
function onConnectionError() {
    window.location.replace("/login");
}
function onConnected() {
    socket.emit("sub", "projects", null, onProjectsReceived);
    var buttons = document.querySelectorAll(".projects-buttons button");
    buttons[0].disabled = false;
    var noSelection = ui.projectsTreeView.selectedNodes.length === 0;
    for (var i = 1; i < buttons.length; i++)
        buttons[i].disabled = noSelection;
}
function onDisconnected() {
    SupClient.Dialogs.cancelDialogIfAny();
    data.projects = null;
    ui.projectsTreeView.clearSelection();
    ui.projectsTreeView.treeRoot.innerHTML = "";
    var buttons = document.querySelectorAll(".projects-buttons button");
    for (var i = 0; i < buttons.length; i++)
        buttons[i].disabled = true;
    document.querySelector(".tree-loading").hidden = false;
}
function onProjectsReceived(err, projects) {
    data.projects = new SupCore.Data.Projects(projects);
    ui.projectsTreeView.clearSelection();
    ui.projectsTreeView.treeRoot.innerHTML = "";
    for (var _i = 0, projects_1 = projects; _i < projects_1.length; _i++) {
        var manifest = projects_1[_i];
        var liElt = createProjectElement(manifest);
        ui.projectsTreeView.append(liElt, "item");
    }
    document.querySelector(".tree-loading").hidden = true;
}
function onProjectAdded(manifest, index) {
    data.projects.client_add(manifest, index);
    var liElt = createProjectElement(manifest);
    ui.projectsTreeView.insertAt(liElt, "item", index);
}
function onSetProjectProperty(id, key, value) {
    data.projects.client_setProperty(id, key, value);
    var projectElt = ui.projectsTreeView.treeRoot.querySelector("[data-id='" + id + "']");
    switch (key) {
        case "name":
            projectElt.querySelector(".name").textContent = value;
            break;
        case "description":
            projectElt.querySelector(".description").textContent = value;
            break;
    }
}
function onUpdateProjectIcon(id) {
    var projectElt = ui.projectsTreeView.treeRoot.querySelector("[data-id='" + id + "']");
    var iconElt = projectElt.querySelector("img");
    iconElt.src = "/projects/" + id + "/icon.png?" + Date.now();
}
// User interface
function createProjectElement(manifest) {
    var liElt = SupClient.html("li", { dataset: { id: manifest.id } });
    SupClient.html("img", { parent: liElt, src: "/projects/" + manifest.id + "/icon.png" });
    var infoElt = SupClient.html("div", "info", { parent: liElt });
    SupClient.html("div", "name", { parent: infoElt, textContent: manifest.name });
    var detailsElt = SupClient.html("div", "details", { parent: infoElt });
    SupClient.html("span", "description", { parent: detailsElt, textContent: manifest.description });
    SupClient.html("span", "project-type", { parent: detailsElt, textContent: SupClient.i18n.t("system-" + manifest.systemId + ":title") });
    return liElt;
}
function onProjectSelectionChange() {
    var buttons = document.querySelectorAll(".projects-buttons button");
    buttons[0].disabled = false;
    var noSelection = ui.projectsTreeView.selectedNodes.length === 0;
    for (var i = 1; i < buttons.length; i++)
        buttons[i].disabled = noSelection;
}
function onProjectActivate() {
    var projectId = ui.projectsTreeView.selectedNodes[0].dataset["id"];
    var href = "/project/?project=" + projectId;
    if (SupApp != null)
        SupApp.openWindow("" + window.location.origin + href);
    else
        window.location.href = href;
}
var autoOpenProject = true;
function onNewProjectClick() {
    new CreateOrEditProjectDialog_1.default(data.systemsById, { autoOpen: autoOpenProject }, function (result) {
        if (result == null)
            return;
        autoOpenProject = result.open;
        socket.emit("add:projects", result.project, onProjectAddedAck);
    });
}
function onProjectAddedAck(err, id) {
    if (err != null) {
        new SupClient.Dialogs.InfoDialog(err);
        return;
    }
    ui.projectsTreeView.clearSelection();
    var node = ui.projectsTreeView.treeRoot.querySelector("li[data-id='" + id + "']");
    ui.projectsTreeView.addToSelection(node);
    ui.projectsTreeView.scrollIntoView(node);
    if (autoOpenProject)
        onProjectActivate();
}
function onEditProjectClick() {
    if (ui.projectsTreeView.selectedNodes.length !== 1)
        return;
    var selectedNode = ui.projectsTreeView.selectedNodes[0];
    var existingProject = data.projects.byId[selectedNode.dataset["id"]];
    new CreateOrEditProjectDialog_1.default(data.systemsById, { existingProject: existingProject }, function (result) {
        if (result == null)
            return;
        autoOpenProject = result.open;
        delete result.project.systemId;
        if (result.project.icon == null)
            delete result.project.icon;
        socket.emit("edit:projects", existingProject.id, result.project, function (err) {
            if (err != null) {
                new SupClient.Dialogs.InfoDialog(err);
                return;
            }
        });
    });
}
