"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var async = require("async");
var TreeView = require("dnd-tree-view");
var network_1 = require("../network");
var buildPluginsLoaded = false;
var previousSelectedPluginName;
var StartBuildDialog = (function (_super) {
    __extends(StartBuildDialog, _super);
    function StartBuildDialog(entries, entriesTreeView, callback) {
        var _this = this;
        _super.call(this, callback);
        this.entries = entries;
        this.entriesTreeView = entriesTreeView;
        this.selectedPluginName = previousSelectedPluginName;
        this.settingsEditorsByName = {};
        this.onBuildPluginsLoaded = function () {
            _this.loadingContainer.parentElement.removeChild(_this.loadingContainer);
            var plugins = SupClient.getPlugins("build");
            var pluginNames = Object.keys(plugins);
            if (plugins == null && pluginNames.length === 0)
                return;
            for (var _i = 0, pluginNames_1 = pluginNames; _i < pluginNames_1.length; _i++) {
                var pluginName = pluginNames_1[_i];
                var plugin = plugins[pluginName].content;
                var elt = SupClient.html("li", { dataset: { buildPlugin: pluginName } });
                _this.treeView.append(elt, "item");
                SupClient.html("div", "label", { parent: elt, textContent: SupClient.i18n.t("buildSettingsEditors:" + pluginName + ".label") });
                SupClient.html("div", "description", { parent: elt, textContent: SupClient.i18n.t("buildSettingsEditors:" + pluginName + ".description") });
                _this.settingsEditorsByName[pluginName] = new plugin.settingsEditor(_this.settingsContainer, _this.entries, _this.entriesTreeView);
            }
            if (_this.selectedPluginName == null)
                _this.selectedPluginName = pluginNames[0];
            _this.treeView.addToSelection(_this.treeView.treeRoot.querySelector("li[data-build-plugin=\"" + _this.selectedPluginName + "\"]"));
            _this.settingsEditorsByName[_this.selectedPluginName].setVisible(true);
            _this.validateButtonElt.disabled = false;
        };
        this.onTreeViewSelectionChange = function () {
            if (_this.treeView.selectedNodes.length === 0) {
                _this.treeView.addToSelection(_this.treeView.treeRoot.querySelector("li[data-build-plugin=\"" + _this.selectedPluginName + "\"]"));
            }
            else {
                _this.settingsEditorsByName[_this.selectedPluginName].setVisible(false);
                _this.selectedPluginName = _this.treeView.selectedNodes[0].dataset["buildPlugin"];
                previousSelectedPluginName = _this.selectedPluginName;
                _this.settingsEditorsByName[_this.selectedPluginName].setVisible(true);
            }
        };
        this.formElt.style.width = "700px";
        SupClient.html("header", { parent: this.formElt, textContent: SupClient.i18n.t("project:build.title") });
        var styleElt = SupClient.html("style", { parent: this.formElt });
        styleElt.textContent = "\n    .build-plugins-tree-view ol.tree { position: absolute; }\n    .build-plugins-tree-view ol.tree li {\n      display: block;\n      height: auto;\n      padding: .5em 1em;\n      white-space: nowrap;\n      overflow-x: hidden;\n      text-overflow: ellipsis;\n    }\n    .build-plugins-tree-view .label { font-size: 1.5em; }\n    ";
        var mainContainer = SupClient.html("div", "group", {
            parent: this.formElt,
            style: { display: "flex", height: "300px" }
        });
        // Build plugins
        var treeViewContainer = SupClient.html("div", ["build-plugins-tree-view"], {
            parent: mainContainer,
            style: {
                position: "relative",
                border: "1px solid #aaa",
                overflowY: "auto",
                width: "200px",
                marginRight: "0.5em"
            }
        });
        this.loadingContainer = SupClient.html("div", "tree-loading", { parent: treeViewContainer });
        SupClient.html("div", { parent: this.loadingContainer, textContent: SupClient.i18n.t("common:states.loading") });
        this.treeView = new TreeView(treeViewContainer, { multipleSelection: false });
        this.treeView.treeRoot.style.paddingBottom = "0";
        this.treeView.on("selectionChange", this.onTreeViewSelectionChange);
        // Build settings
        this.settingsContainer = SupClient.html("div", {
            parent: mainContainer,
            style: { flex: "1 1 0" }
        });
        // Buttons
        var buttonsElt = SupClient.html("div", "buttons", { parent: this.formElt });
        var cancelButtonElt = SupClient.html("button", "cancel-button", {
            parent: buttonsElt, type: "button",
            textContent: SupClient.i18n.t("common:actions.cancel")
        });
        cancelButtonElt.addEventListener("click", function (event) { event.preventDefault(); _this.cancel(); });
        this.validateButtonElt = SupClient.html("button", "validate-button", {
            textContent: SupClient.i18n.t("project:build.build"),
            disabled: true
        });
        if (navigator.platform === "Win32")
            buttonsElt.insertBefore(this.validateButtonElt, cancelButtonElt);
        else
            buttonsElt.appendChild(this.validateButtonElt);
        this.treeView.treeRoot.focus();
        if (!buildPluginsLoaded)
            loadBuildPlugins(this.onBuildPluginsLoaded);
        else
            this.onBuildPluginsLoaded();
    }
    StartBuildDialog.prototype.submit = function () {
        var _this = this;
        if (!buildPluginsLoaded)
            return;
        this.settingsEditorsByName[this.selectedPluginName].getSettings(function (settings) {
            if (settings == null)
                return;
            var plugin = SupClient.getPlugins("build")[_this.selectedPluginName];
            _super.prototype.submit.call(_this, {
                pluginPath: plugin.path,
                buildPluginName: _this.selectedPluginName,
                settings: settings
            });
        });
    };
    return StartBuildDialog;
}(SupClient.Dialogs.BaseDialog));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = StartBuildDialog;
function loadBuildPlugins(callback) {
    var i18nFiles = [];
    for (var _i = 0, _a = network_1.pluginsInfo.list; _i < _a.length; _i++) {
        var pluginName = _a[_i];
        var root = "/systems/" + SupCore.system.id + "/plugins/" + pluginName;
        i18nFiles.push({ root: root, name: "buildSettingsEditors" });
    }
    async.parallel([
        function (cb) {
            SupClient.i18n.load(i18nFiles, cb);
        }, function (cb) {
            async.each(network_1.pluginsInfo.list, function (pluginName, cb) {
                var pluginPath = "/systems/" + SupCore.system.id + "/plugins/" + pluginName;
                SupClient.loadScript(pluginPath + "/bundles/build.js", cb);
            }, cb);
        }
    ], function () {
        buildPluginsLoaded = true;
        callback();
    });
}
