"use strict";
var network_1 = require("../../network");
var tabsAssets = require("../../tabs/assets");
var entriesTreeView = require("./");
var CreateAssetDialog_1 = require("./CreateAssetDialog");
var autoOpenAsset = true;
function onEntryAddedAck(err, id) {
    if (err != null) {
        new SupClient.Dialogs.InfoDialog(err);
        return;
    }
    entriesTreeView.widget.clearSelection();
    var entry = entriesTreeView.widget.treeRoot.querySelector("li[data-id='" + id + "']");
    entriesTreeView.widget.addToSelection(entry);
    entriesTreeView.updateSelectedEntry();
    if (autoOpenAsset)
        tabsAssets.open(id);
    if (network_1.entries.byId[id].type == null)
        entry.classList.remove("collapsed");
}
function onNewAssetClick() {
    new CreateAssetDialog_1.default(autoOpenAsset, function (result) {
        if (result == null)
            return;
        if (result.name === "")
            result.name = SupClient.i18n.t(tabsAssets.editorsByAssetType[result.type].pluginPath + ":editors." + result.type + ".title");
        autoOpenAsset = result.open;
        network_1.socket.emit("add:entries", result.name, result.type, SupClient.getTreeViewInsertionPoint(entriesTreeView.widget), onEntryAddedAck);
    });
}
exports.onNewAssetClick = onNewAssetClick;
function onNewFolderClick() {
    var options = {
        placeholder: SupClient.i18n.t("project:treeView.newFolder.placeholder"),
        initialValue: SupClient.i18n.t("project:treeView.newFolder.initialValue"),
        validationLabel: SupClient.i18n.t("common:actions.create"),
        pattern: SupClient.namePattern,
        title: SupClient.i18n.t("common:namePatternDescription")
    };
    new SupClient.Dialogs.PromptDialog(SupClient.i18n.t("project:treeView.newFolder.prompt"), options, function (name) {
        if (name == null)
            return;
        network_1.socket.emit("add:entries", name, null, SupClient.getTreeViewInsertionPoint(entriesTreeView.widget), onEntryAddedAck);
    });
}
exports.onNewFolderClick = onNewFolderClick;
function onRenameEntryClick() {
    if (entriesTreeView.widget.selectedNodes.length !== 1)
        return;
    var selectedNode = entriesTreeView.widget.selectedNodes[0];
    var entry = network_1.entries.byId[selectedNode.dataset["id"]];
    var options = {
        initialValue: entry.name,
        validationLabel: SupClient.i18n.t("common:actions.rename"),
        pattern: SupClient.namePattern,
        title: SupClient.i18n.t("common:namePatternDescription")
    };
    new SupClient.Dialogs.PromptDialog(SupClient.i18n.t("project:treeView.renamePrompt"), options, function (newName) {
        if (newName == null || newName === entry.name)
            return;
        network_1.socket.emit("setProperty:entries", entry.id, "name", newName, function (err) {
            if (err != null) {
                new SupClient.Dialogs.InfoDialog(err);
                return;
            }
        });
    });
}
exports.onRenameEntryClick = onRenameEntryClick;
function onDuplicateEntryClick() {
    if (entriesTreeView.widget.selectedNodes.length !== 1)
        return;
    var selectedNode = entriesTreeView.widget.selectedNodes[0];
    var entry = network_1.entries.byId[selectedNode.dataset["id"]];
    var options = {
        initialValue: entry.name,
        validationLabel: SupClient.i18n.t("common:actions.duplicate"),
        pattern: SupClient.namePattern,
        title: SupClient.i18n.t("common:namePatternDescription")
    };
    new SupClient.Dialogs.PromptDialog(SupClient.i18n.t("project:treeView.duplicatePrompt"), options, function (newName) {
        if (newName == null)
            return;
        var insertionPoint = entry.type == null ? SupClient.getTreeViewSiblingInsertionPoint(entriesTreeView.widget) : SupClient.getTreeViewInsertionPoint(entriesTreeView.widget);
        network_1.socket.emit("duplicate:entries", newName, entry.id, insertionPoint, onEntryAddedAck);
    });
}
exports.onDuplicateEntryClick = onDuplicateEntryClick;
function onTrashEntryClick() {
    if (entriesTreeView.widget.selectedNodes.length === 0)
        return;
    var selectedEntries = [];
    function checkNextEntry() {
        selectedEntries.splice(0, 1);
        if (selectedEntries.length === 0) {
            var confirmLabel = SupClient.i18n.t("project:treeView.trash.prompt");
            var validationLabel = SupClient.i18n.t("project:treeView.trash.title");
            new SupClient.Dialogs.ConfirmDialog(confirmLabel, { validationLabel: validationLabel }, function (confirm) {
                if (!confirm)
                    return;
                for (var _i = 0, _a = entriesTreeView.widget.selectedNodes; _i < _a.length; _i++) {
                    var selectedNode = _a[_i];
                    var entry = network_1.entries.byId[selectedNode.dataset["id"]];
                    network_1.socket.emit("trash:entries", entry.id, function (err) {
                        if (err != null) {
                            new SupClient.Dialogs.InfoDialog(err);
                            return;
                        }
                    });
                }
                entriesTreeView.widget.clearSelection();
            });
        }
        else
            warnBrokenDependency(selectedEntries[0]);
    }
    function warnBrokenDependency(entry) {
        if (entry.type == null)
            for (var _i = 0, _a = entry.children; _i < _a.length; _i++) {
                var entryChild = _a[_i];
                selectedEntries.push(entryChild);
            }
        if (entry.dependentAssetIds != null && entry.dependentAssetIds.length > 0) {
            var dependentAssetNames = [];
            for (var _b = 0, _c = entry.dependentAssetIds; _b < _c.length; _b++) {
                var usingId = _c[_b];
                dependentAssetNames.push(network_1.entries.getPathFromId(usingId));
            }
            var infoLabel = SupClient.i18n.t("project:treeView.trash.warnBrokenDependency", {
                entryName: network_1.entries.getPathFromId(entry.id), dependentEntryNames: dependentAssetNames.join(", ")
            });
            new SupClient.Dialogs.InfoDialog(infoLabel, null, function () { checkNextEntry(); });
        }
        else
            checkNextEntry();
    }
    for (var _i = 0, _a = entriesTreeView.widget.selectedNodes; _i < _a.length; _i++) {
        var selectedNode = _a[_i];
        selectedEntries.push(network_1.entries.byId[selectedNode.dataset["id"]]);
    }
    warnBrokenDependency(selectedEntries[0]);
}
exports.onTrashEntryClick = onTrashEntryClick;
var entriesFilterStrip = document.querySelector(".filter-buttons");
function setupFilterStrip() {
    var filterElt = entriesFilterStrip;
    filterElt.innerHTML = "";
    var toggleAllElt = SupClient.html("img", "toggle-all", { parent: filterElt, draggable: false });
    toggleAllElt.addEventListener("click", onToggleAllFilterClick);
    for (var _i = 0, _a = tabsAssets.assetTypes; _i < _a.length; _i++) {
        var assetType = _a[_i];
        var iconElt = SupClient.html("img", { parent: filterElt, dataset: { assetType: assetType }, draggable: false });
        iconElt.src = "/systems/" + SupCore.system.id + "/plugins/" + tabsAssets.editorsByAssetType[assetType].pluginPath + "/editors/" + assetType + "/icon.svg";
        iconElt.addEventListener("click", onToggleAssetTypeFilterClick);
    }
}
exports.setupFilterStrip = setupFilterStrip;
function onToggleAssetTypeFilterClick(event) {
    var filterElt = event.target;
    var filtered = filterElt.classList.toggle("filtered");
    var assetType = filterElt.dataset["assetType"];
    var entryElts = entriesTreeView.widget.treeRoot.querySelectorAll("[data-asset-type='" + assetType + "']");
    for (var _i = 0, entryElts_1 = entryElts; _i < entryElts_1.length; _i++) {
        var entryElt = entryElts_1[_i];
        entryElt.hidden = filtered;
    }
    var allAssetTypesFiltered = true;
    for (var _a = 0, _b = tabsAssets.assetTypes; _a < _b.length; _a++) {
        var assetType_1 = _b[_a];
        var filtered_1 = entriesFilterStrip.querySelector("[data-asset-type='" + assetType_1 + "']").classList.contains("filtered");
        if (!filtered_1) {
            allAssetTypesFiltered = false;
            break;
        }
    }
    entriesFilterStrip.querySelector(".toggle-all").classList.toggle("filtered", allAssetTypesFiltered);
}
function onToggleAllFilterClick() {
    var enableAllFilters = !entriesFilterStrip.querySelector(".toggle-all").classList.contains("filtered");
    var filterElts = entriesFilterStrip.querySelectorAll("img");
    for (var _i = 0, filterElts_1 = filterElts; _i < filterElts_1.length; _i++) {
        var filterElt = filterElts_1[_i];
        filterElt.classList.toggle("filtered", enableAllFilters);
        var assetType = filterElt.dataset["assetType"];
        var entryElts = entriesTreeView.widget.treeRoot.querySelectorAll("[data-asset-type='" + assetType + "']");
        for (var _a = 0, entryElts_2 = entryElts; _a < entryElts_2.length; _a++) {
            var entryElt = entryElts_2[_a];
            entryElt.hidden = enableAllFilters;
        }
    }
}
function onToggleFilterStripClick() {
    entriesFilterStrip.hidden = !entriesFilterStrip.hidden;
    if (entriesFilterStrip.hidden) {
        var hiddenEntryElts = entriesTreeView.widget.treeRoot.querySelectorAll("li.item[hidden]");
        for (var _i = 0, hiddenEntryElts_1 = hiddenEntryElts; _i < hiddenEntryElts_1.length; _i++) {
            var hiddenEntryElt = hiddenEntryElts_1[_i];
            hiddenEntryElt.hidden = false;
        }
    }
    else {
        for (var _a = 0, _b = tabsAssets.assetTypes; _a < _b.length; _a++) {
            var assetType = _b[_a];
            var filtered = entriesFilterStrip.querySelector("[data-asset-type='" + assetType + "']").classList.contains("filtered");
            var entryElts = entriesTreeView.widget.treeRoot.querySelectorAll("[data-asset-type='" + assetType + "']");
            for (var _c = 0, entryElts_3 = entryElts; _c < entryElts_3.length; _c++) {
                var entryElt = entryElts_3[_c];
                entryElt.hidden = filtered;
            }
        }
    }
}
exports.onToggleFilterStripClick = onToggleFilterStripClick;
function onSearchEntryDialog() {
    if (network_1.entries == null)
        return;
    new SupClient.Dialogs.FindAssetDialog(network_1.entries, tabsAssets.editorsByAssetType, function (entryId) {
        if (entryId == null)
            return;
        tabsAssets.open(entryId);
        entriesTreeView.widget.clearSelection();
        var entryElt = entriesTreeView.widget.treeRoot.querySelector("[data-id='" + entryId + "']");
        entriesTreeView.widget.addToSelection(entryElt);
        entriesTreeView.widget.scrollIntoView(entryElt);
    });
}
exports.onSearchEntryDialog = onSearchEntryDialog;
