"use strict";
var TreeView = require("dnd-tree-view");
var network_1 = require("../../network");
var tabsAssets = require("../../tabs/assets");
var sidebar = require("../");
var buttonCallbacks_1 = require("./buttonCallbacks");
function start() {
    exports.widget = new TreeView(document.querySelector(".entries-tree-view"), { dragStartCallback: onEntryDragStart, dropCallback: onTreeViewDrop });
    exports.widget.on("selectionChange", updateSelectedEntry);
    exports.widget.on("activate", onEntryActivate);
    document.querySelector(".entries-buttons .new-asset").addEventListener("click", buttonCallbacks_1.onNewAssetClick);
    document.querySelector(".entries-buttons .new-folder").addEventListener("click", buttonCallbacks_1.onNewFolderClick);
    document.querySelector(".entries-buttons .search").addEventListener("click", buttonCallbacks_1.onSearchEntryDialog);
    document.querySelector(".entries-buttons .rename-entry").addEventListener("click", buttonCallbacks_1.onRenameEntryClick);
    document.querySelector(".entries-buttons .duplicate-entry").addEventListener("click", buttonCallbacks_1.onDuplicateEntryClick);
    document.querySelector(".entries-buttons .trash-entry").addEventListener("click", buttonCallbacks_1.onTrashEntryClick);
    document.querySelector(".entries-buttons .filter").addEventListener("click", buttonCallbacks_1.onToggleFilterStripClick);
    sidebar.openInNewWindowButton.addEventListener("click", onOpenInNewWindowClick);
    // Hot-keys
    document.addEventListener("keydown", function (event) {
        if (document.querySelector(".dialog") != null)
            return;
        if (event.keyCode === 113) {
            event.preventDefault();
            buttonCallbacks_1.onRenameEntryClick();
        }
        if (event.keyCode === 68 && (event.ctrlKey || event.metaKey)) {
            event.preventDefault();
            buttonCallbacks_1.onDuplicateEntryClick();
        }
        if (event.keyCode === 46) {
            event.preventDefault();
            buttonCallbacks_1.onTrashEntryClick();
        }
    });
}
exports.start = start;
function enable() {
    document.querySelector(".entries-buttons .new-asset").disabled = false;
    document.querySelector(".entries-buttons .new-folder").disabled = false;
    document.querySelector(".entries-buttons .search").disabled = false;
    document.querySelector(".entries-buttons .filter").disabled = false;
    document.querySelector(".filter-buttons").hidden = true;
    document.querySelector(".entries-tree-view .tree-loading").hidden = true;
    function walk(entry, parentEntry, parentElt) {
        var liElt = createEntryElement(entry);
        liElt.classList.add("collapsed");
        var nodeType = (entry.children != null) ? "group" : "item";
        exports.widget.append(liElt, nodeType, parentElt);
        if (entry.children != null)
            for (var _i = 0, _a = entry.children; _i < _a.length; _i++) {
                var child = _a[_i];
                walk(child, entry, liElt);
            }
    }
    for (var _i = 0, _a = network_1.entries.pub; _i < _a.length; _i++) {
        var entry = _a[_i];
        walk(entry, null, null);
    }
    buttonCallbacks_1.setupFilterStrip();
}
exports.enable = enable;
function disable() {
    exports.widget.clear();
    document.querySelector(".entries-buttons .new-asset").disabled = true;
    document.querySelector(".entries-buttons .new-folder").disabled = true;
    document.querySelector(".entries-buttons .search").disabled = true;
    document.querySelector(".entries-buttons .rename-entry").disabled = true;
    document.querySelector(".entries-buttons .duplicate-entry").disabled = true;
    document.querySelector(".entries-buttons .trash-entry").disabled = true;
    document.querySelector(".entries-buttons .filter").disabled = true;
    document.querySelector(".filter-buttons").hidden = true;
    document.querySelector(".entries-tree-view .tree-loading").hidden = false;
}
exports.disable = disable;
function createEntryElement(entry) {
    var liElt = SupClient.html("li", { dataset: { id: entry.id } });
    var parentEntry = network_1.entries.parentNodesById[entry.id];
    if (parentEntry != null)
        liElt.dataset["parentId"] = parentEntry.id;
    if (entry.type != null) {
        liElt.dataset["assetType"] = entry.type;
        var iconElt = SupClient.html("img", { parent: liElt, draggable: false });
        iconElt.src = "/systems/" + SupCore.system.id + "/plugins/" + tabsAssets.editorsByAssetType[entry.type].pluginPath + "/editors/" + entry.type + "/icon.svg";
    }
    SupClient.html("span", "name", { parent: liElt, textContent: entry.name });
    if (entry.type != null) {
        var badgesSpan = SupClient.html("span", "badges", { parent: liElt });
        for (var _i = 0, _a = entry.badges; _i < _a.length; _i++) {
            var badge = _a[_i];
            SupClient.html("span", badge.id, { parent: badgesSpan, textContent: SupClient.i18n.t("badges:" + badge.id) });
        }
        liElt.addEventListener("mouseenter", function (event) { liElt.appendChild(sidebar.openInNewWindowButton); });
        liElt.addEventListener("mouseleave", function (event) {
            if (sidebar.openInNewWindowButton.parentElement != null)
                sidebar.openInNewWindowButton.parentElement.removeChild(sidebar.openInNewWindowButton);
        });
    }
    else {
        var childrenElt_1 = SupClient.html("span", "children", { parent: liElt, textContent: "(" + entry.children.length + ")", style: { display: "none" } });
        liElt.addEventListener("mouseenter", function (event) { childrenElt_1.style.display = ""; });
        liElt.addEventListener("mouseleave", function (event) { childrenElt_1.style.display = "none"; });
    }
    return liElt;
}
exports.createEntryElement = createEntryElement;
function onEntryDragStart(event, entryElt) {
    var id = entryElt.dataset["id"];
    event.dataTransfer.setData("text/plain", network_1.entries.getPathFromId(id));
    var entryIds = [id];
    for (var _i = 0, _a = exports.widget.selectedNodes; _i < _a.length; _i++) {
        var node = _a[_i];
        if (node.dataset["id"] !== id)
            entryIds.push(node.dataset["id"]);
    }
    event.dataTransfer.setData("application/vnd.superpowers.entry", entryIds.join(","));
    return true;
}
function onTreeViewDrop(event, dropLocation, orderedNodes) {
    if (orderedNodes == null) {
        // TODO: Support creating assets by importing some files
        return false;
    }
    var dropPoint = SupClient.getTreeViewDropPoint(dropLocation, network_1.entries);
    var entryIds = [];
    for (var _i = 0, orderedNodes_1 = orderedNodes; _i < orderedNodes_1.length; _i++) {
        var entry = orderedNodes_1[_i];
        entryIds.push(entry.dataset["id"]);
    }
    var sourceParentNode = network_1.entries.parentNodesById[entryIds[0]];
    var sourceChildren = (sourceParentNode != null && sourceParentNode.children != null) ? sourceParentNode.children : network_1.entries.pub;
    var sameParent = (sourceParentNode != null && dropPoint.parentId === sourceParentNode.id);
    var i = 0;
    for (var _a = 0, entryIds_1 = entryIds; _a < entryIds_1.length; _a++) {
        var id = entryIds_1[_a];
        network_1.socket.emit("move:entries", id, dropPoint.parentId, dropPoint.index + i, function (err) {
            if (err != null) {
                new SupClient.Dialogs.InfoDialog(err);
                return;
            }
        });
        if (!sameParent || sourceChildren.indexOf(network_1.entries.byId[id]) >= dropPoint.index)
            i++;
    }
    return false;
}
function updateSelectedEntry() {
    var allButtons = document.querySelectorAll(".entries-buttons button.edit");
    for (var index = 0; index < allButtons.length; index++) {
        var button = allButtons.item(index);
        button.disabled = exports.widget.selectedNodes.length === 0 || (button.classList.contains("single") && exports.widget.selectedNodes.length !== 1);
    }
}
exports.updateSelectedEntry = updateSelectedEntry;
function scrollEntryIntoView(entryId) {
    var entryElt = exports.widget.treeRoot.querySelector("[data-id='" + entryId + "']");
    exports.widget.clearSelection();
    exports.widget.scrollIntoView(entryElt);
    exports.widget.addToSelection(entryElt);
}
exports.scrollEntryIntoView = scrollEntryIntoView;
function onEntryActivate() {
    var activatedEntry = exports.widget.selectedNodes[0];
    tabsAssets.open(activatedEntry.dataset["id"]);
}
function onOpenInNewWindowClick(event) {
    var id = event.target.parentElement.dataset["id"];
    var name = event.target.parentElement.dataset["name"];
    var url;
    if (id != null)
        url = window.location.origin + "/project/?project=" + SupClient.query.project + "&asset=" + id;
    else
        url = window.location.origin + "/project/?project=" + SupClient.query.project + "&tool=" + name;
    if (SupApp != null)
        SupApp.openWindow(url);
    else
        window.open(url);
}
