"use strict";
var async = require("async");
var network_1 = require("../network");
var entriesTreeView = require("../sidebar/entriesTreeView");
var tabs = require("./");
function setup(editorPaths, callback) {
    exports.editorsByAssetType = {};
    var pluginsRoot = "/systems/" + SupCore.system.id + "/plugins";
    async.each(Object.keys(editorPaths), function (assetType, cb) {
        var toolTitle = SupClient.i18n.t(editorPaths[assetType] + ":editors." + assetType + ".title");
        var pluginPath = editorPaths[assetType];
        exports.editorsByAssetType[assetType] = { title: toolTitle, pluginPath: pluginPath, revision: false };
        SupClient.fetch(pluginsRoot + "/" + pluginPath + "/editors/" + assetType + "/manifest.json", "json", function (err, editorManifest) {
            if (err != null) {
                cb(err);
                return;
            }
            exports.editorsByAssetType[assetType].revision = editorManifest.revision;
            cb();
        });
    }, function (err) {
        if (err != null) {
            callback(err);
            return;
        }
        exports.assetTypes = Object.keys(editorPaths).sort(function (a, b) { return exports.editorsByAssetType[a].title.localeCompare(exports.editorsByAssetType[b].title); });
        callback();
    });
}
exports.setup = setup;
function open(id, state) {
    var entry = network_1.entries.byId[id];
    if (entry == null)
        return;
    // Just toggle folders
    if (entry.type == null) {
        entriesTreeView.widget.selectedNodes[0].classList.toggle("collapsed");
        return;
    }
    entriesTreeView.scrollEntryIntoView(id);
    var tab = tabs.tabStrip.tabsRoot.querySelector("li[data-asset-id='" + id + "']");
    if (tab == null) {
        tab = createTabElement(entry);
        tabs.tabStrip.tabsRoot.appendChild(tab);
        var paneElt = SupClient.html("div", "pane-container", { parent: tabs.panesElt, dataset: { assetId: id } });
        var revisionOuterContainer = SupClient.html("div", "revision-outer-container", { parent: paneElt, hidden: !exports.editorsByAssetType[entry.type].revision });
        var revisionInnerContainer = SupClient.html("div", "revision-inner-container", { parent: revisionOuterContainer });
        SupClient.html("span", { parent: revisionInnerContainer, textContent: SupClient.i18n.t("project:revision.title") });
        var selectElt_1 = SupClient.html("select", { parent: revisionInnerContainer, disabled: true });
        SupClient.html("option", { parent: selectElt_1, textContent: SupClient.i18n.t("project:revision.current"), value: "current" });
        for (var revisionIndex = entry.revisions.length - 1; revisionIndex >= 0; revisionIndex--) {
            var revision = entry.revisions[revisionIndex];
            SupClient.html("option", { parent: selectElt_1, textContent: revision.name, value: revision.id });
        }
        var saveOrRestoreButtonElt_1 = SupClient.html("button", { parent: revisionInnerContainer, textContent: SupClient.i18n.t("common:actions.save"), disabled: true });
        var src = "/systems/" + SupCore.system.id + "/plugins/" + exports.editorsByAssetType[entry.type].pluginPath + "/editors/" + entry.type + "/?project=" + SupClient.query.project + "&asset=" + id;
        var iframe_1 = SupClient.html("iframe", { parent: paneElt, src: src });
        if (state != null)
            iframe_1.addEventListener("load", function () { iframe_1.contentWindow.postMessage({ type: "setState", state: state }, window.location.origin); });
        selectElt_1.addEventListener("change", function () {
            saveOrRestoreButtonElt_1.textContent = SupClient.i18n.t("common:actions." + (selectElt_1.value === "current" ? "save" : "restore"));
            iframe_1.contentWindow.postMessage({ type: "setRevision", revisionId: selectElt_1.value }, window.location.origin);
        });
        saveOrRestoreButtonElt_1.addEventListener("click", function () {
            if (selectElt_1.value === "current") {
                var date = new Date();
                var options = {
                    header: SupClient.i18n.t("project:revision.title"),
                    validationLabel: SupClient.i18n.t("common:actions.save"),
                    initialValue: date.getUTCFullYear() + "-" + date.getUTCMonth() + "-" + date.getUTCDay() + " " + date.getUTCHours() + "h" + date.getUTCMinutes(),
                    title: SupClient.i18n.t("common:namePatternDescription"),
                    pattern: SupClient.namePattern
                };
                /* tslint:disable:no-unused-expression */
                new SupClient.Dialogs.PromptDialog(SupClient.i18n.t("project:revision.prompt"), options, function (revisionName) {
                    /* tslint:enable:no-unused-expression */
                    if (revisionName == null)
                        return;
                    network_1.socket.emit("save:entries", id, revisionName, function (err) {
                        if (err != null) {
                            /* tslint:disable:no-unused-expression */
                            new SupClient.Dialogs.InfoDialog(err);
                            /* tslint:enable:no-unused-expression */
                            return;
                        }
                    });
                });
            }
            else {
                selectElt_1.disabled = true;
                saveOrRestoreButtonElt_1.disabled = true;
                saveOrRestoreButtonElt_1.textContent = SupClient.i18n.t("project:revision.restoring");
                network_1.socket.emit("restore:assets", id, selectElt_1.value, function (err) {
                    if (err != null) {
                        /* tslint:disable:no-unused-expression */
                        new SupClient.Dialogs.InfoDialog(err);
                        /* tslint:enable:no-unused-expression */
                        return;
                    }
                    selectElt_1.disabled = false;
                    selectElt_1.value = "current";
                    saveOrRestoreButtonElt_1.disabled = false;
                    saveOrRestoreButtonElt_1.textContent = SupClient.i18n.t("common:actions.save");
                    iframe_1.contentWindow.postMessage({ type: "setRevision", revisionId: "restored" }, window.location.origin);
                });
            }
        });
    }
    else if (state != null) {
        var iframe = tabs.panesElt.querySelector("[data-asset-id='" + id + "'] iframe");
        iframe.contentWindow.postMessage({ type: "setState", state: state }, window.location.origin);
    }
    tabs.onActivate(tab);
    return tab;
}
exports.open = open;
function createTabElement(entry) {
    var tabElt = SupClient.html("li", { dataset: { assetId: entry.id } });
    if (entry.type != null) {
        var iconElt = SupClient.html("img", "icon", { parent: tabElt });
        iconElt.src = "/systems/" + SupCore.system.id + "/plugins/" + exports.editorsByAssetType[entry.type].pluginPath + "/editors/" + entry.type + "/icon.svg";
        // FIXME: This event isn't emitted on the last opened tab in Chrome. Works fine in Firefox though
        tabElt.addEventListener("dblclick", function () { entriesTreeView.scrollEntryIntoView(entry.id); });
    }
    var tabLabel = SupClient.html("div", "label", { parent: tabElt });
    SupClient.html("div", "location", { parent: tabLabel });
    SupClient.html("div", "name", { parent: tabLabel });
    var closeButton = SupClient.html("button", "close", { parent: tabElt });
    closeButton.addEventListener("click", function () { tabs.onClose(tabElt); });
    refreshTabElement(entry, tabElt);
    return tabElt;
}
function refreshTabElement(entry, tabElt) {
    if (tabElt == null)
        tabElt = tabs.tabStrip.tabsRoot.querySelector("li[data-asset-id='" + entry.id + "']");
    if (tabElt == null)
        return;
    var entryPath = network_1.entries.getPathFromId(entry.id);
    var entryName = entry.name;
    var lastSlash = entryPath.lastIndexOf("/");
    var entryLocation = (lastSlash !== -1) ? entryPath.slice(0, lastSlash) : "";
    var maxEntryLocationLength = 20;
    while (entryLocation.length > maxEntryLocationLength) {
        var slashIndex = entryLocation.indexOf("/", 2);
        if (slashIndex === -1)
            break;
        entryLocation = "\u2026/" + entryLocation.slice(slashIndex + 1);
    }
    tabElt.querySelector(".label .location").textContent = entryLocation;
    tabElt.querySelector(".label .name").textContent = entryName;
    tabElt.title = entryPath;
}
exports.refreshTabElement = refreshTabElement;
function setRevisionDisabled(id, disabled) {
    var revisionContainer = tabs.panesElt.querySelector("[data-asset-id='" + id + "'] .revision-inner-container");
    revisionContainer.querySelector("select").disabled = disabled;
    revisionContainer.querySelector("button").disabled = disabled;
}
exports.setRevisionDisabled = setRevisionDisabled;
