"use strict";
var TabStrip = require("tab-strip");
var tabsBarElt = document.querySelector(".tabs-bar");
exports.tabStrip = new TabStrip(tabsBarElt);
exports.panesElt = document.querySelector(".main .panes");
function start() {
    exports.tabStrip.on("activateTab", onActivate);
    exports.tabStrip.on("closeTab", onClose);
    // Prevent <iframe> panes from getting mouse event while dragging tabs
    function restorePanesMouseEvent(event) {
        exports.panesElt.style.pointerEvents = "";
        document.removeEventListener("mouseup", restorePanesMouseEvent);
    }
    tabsBarElt.addEventListener("mousedown", function (event) {
        exports.panesElt.style.pointerEvents = "none";
        document.addEventListener("mouseup", restorePanesMouseEvent);
    });
}
exports.start = start;
function onActivate(tabElement) {
    var activeTab = exports.tabStrip.tabsRoot.querySelector(".active");
    if (activeTab != null) {
        activeTab.classList.remove("active");
        var activePaneElt = exports.panesElt.querySelector(".pane-container.active");
        activePaneElt.classList.remove("active");
        var activeIframe = activePaneElt.querySelector("iframe");
        activeIframe.contentWindow.postMessage({ type: "deactivate" }, window.location.origin);
    }
    tabElement.classList.add("active");
    tabElement.classList.remove("unread");
    var assetId = tabElement.dataset["assetId"];
    var paneElt;
    if (assetId != null)
        paneElt = exports.panesElt.querySelector(".pane-container[data-asset-id='" + assetId + "']");
    else
        paneElt = exports.panesElt.querySelector(".pane-container[data-name='" + tabElement.dataset["pane"] + "']");
    paneElt.classList.add("active");
    var iframe = paneElt.querySelector("iframe");
    iframe.contentWindow.focus();
    iframe.contentWindow.postMessage({ type: "activate" }, window.location.origin);
}
exports.onActivate = onActivate;
function onClose(tabElement) {
    if (tabElement == null)
        tabElement = exports.tabStrip.tabsRoot.querySelector(".active");
    var assetId = tabElement.dataset["assetId"];
    var paneElt;
    if (assetId != null)
        paneElt = exports.panesElt.querySelector(".pane-container[data-asset-id='" + assetId + "']");
    else {
        if (tabElement.classList.contains("pinned"))
            return;
        paneElt = exports.panesElt.querySelector(".pane-container[data-name='" + tabElement.dataset["pane"] + "']");
    }
    if (tabElement.classList.contains("active")) {
        var activeTabElement = (tabElement.nextElementSibling != null) ? tabElement.nextElementSibling : tabElement.previousElementSibling;
        if (activeTabElement != null)
            onActivate(activeTabElement);
    }
    tabElement.parentElement.removeChild(tabElement);
    paneElt.parentElement.removeChild(paneElt);
}
exports.onClose = onClose;
function onActivatePrevious() {
    var activeTabElt = exports.tabStrip.tabsRoot.querySelector(".active");
    for (var tabIndex = 0; exports.tabStrip.tabsRoot.children.length; tabIndex++) {
        var tabElt = exports.tabStrip.tabsRoot.children[tabIndex];
        if (tabElt === activeTabElt) {
            var newTabIndex = (tabIndex === 0) ? exports.tabStrip.tabsRoot.children.length - 1 : tabIndex - 1;
            onActivate(exports.tabStrip.tabsRoot.children[newTabIndex]);
            return;
        }
    }
}
exports.onActivatePrevious = onActivatePrevious;
function onActivateNext() {
    var activeTabElt = exports.tabStrip.tabsRoot.querySelector(".active");
    for (var tabIndex = 0; exports.tabStrip.tabsRoot.children.length; tabIndex++) {
        var tabElt = exports.tabStrip.tabsRoot.children[tabIndex];
        if (tabElt === activeTabElt) {
            var newTabIndex = (tabIndex === exports.tabStrip.tabsRoot.children.length - 1) ? 0 : tabIndex + 1;
            onActivate(exports.tabStrip.tabsRoot.children[newTabIndex]);
            return;
        }
    }
}
exports.onActivateNext = onActivateNext;
