"use strict";
var async = require("async");
var sidebar = require("../sidebar");
var tabs = require("./");
var homeTab = require("./homeTab");
var toolsElt = document.querySelector(".sidebar .tools ul");
function setup(toolPaths, callback) {
    exports.toolsByName = {};
    var pluginsRoot = "/systems/" + SupCore.system.id + "/plugins";
    async.each(Object.keys(toolPaths), function (toolName, cb) {
        var toolTitle = SupClient.i18n.t(toolPaths[toolName] + ":editors." + toolName + ".title");
        var pluginPath = toolPaths[toolName];
        exports.toolsByName[toolName] = { title: toolTitle, pluginPath: pluginPath, pinned: false };
        SupClient.fetch(pluginsRoot + "/" + pluginPath + "/editors/" + toolName + "/manifest.json", "json", function (err, toolManifest) {
            if (err != null) {
                cb(err);
                return;
            }
            exports.toolsByName[toolName].pinned = toolManifest.pinned;
            cb();
        });
    }, function (err) {
        if (err != null) {
            callback(err);
            return;
        }
        toolsElt.innerHTML = "";
        var toolNames = Object.keys(exports.toolsByName);
        toolNames.sort(function (a, b) { return exports.toolsByName[a].title.localeCompare(exports.toolsByName[b].title); });
        for (var _i = 0, toolNames_1 = toolNames; _i < toolNames_1.length; _i++) {
            var toolName = toolNames_1[_i];
            setupTool(toolName);
        }
        callback();
    });
}
exports.setup = setup;
function setupTool(toolName) {
    var tool = exports.toolsByName[toolName];
    if (tool.pinned && SupClient.query.asset == null && SupClient.query["tool"] == null) {
        open(toolName);
        return;
    }
    var toolElt = SupClient.html("li", { parent: toolsElt, dataset: { name: toolName } });
    toolElt.addEventListener("mouseenter", function (event) { event.target.appendChild(sidebar.openInNewWindowButton); });
    toolElt.addEventListener("mouseleave", function (event) {
        if (sidebar.openInNewWindowButton.parentElement != null)
            sidebar.openInNewWindowButton.parentElement.removeChild(sidebar.openInNewWindowButton);
    });
    var containerElt = SupClient.html("div", { parent: toolElt });
    SupClient.html("img", { parent: containerElt, src: "/systems/" + SupCore.system.id + "/plugins/" + tool.pluginPath + "/editors/" + toolName + "/icon.svg" });
    var nameSpanElt = SupClient.html("span", "name", { parent: containerElt, textContent: SupClient.i18n.t(tool.pluginPath + ":editors." + toolName + ".title") });
    nameSpanElt.addEventListener("click", function (event) { open(event.target.parentElement.parentElement.dataset["name"]); });
}
function open(name, state) {
    var tab = tabs.tabStrip.tabsRoot.querySelector("li[data-pane='" + name + "']");
    if (tab == null) {
        var tool = exports.toolsByName[name];
        tab = createTabElement(name, tool);
        if (exports.toolsByName[name].pinned) {
            var toolElt = toolsElt.querySelector("li[data-name=\"" + name + "\"]");
            if (toolElt != null)
                toolElt.parentElement.removeChild(toolElt);
            var firstUnpinnedTab = tabs.tabStrip.tabsRoot.querySelector("li:not(.pinned)");
            tabs.tabStrip.tabsRoot.insertBefore(tab, firstUnpinnedTab);
        }
        else {
            tabs.tabStrip.tabsRoot.appendChild(tab);
        }
        var paneElt = SupClient.html("div", "pane-container", { parent: tabs.panesElt, dataset: { name: name } });
        var src = "/systems/" + SupCore.system.id + "/plugins/" + tool.pluginPath + "/editors/" + name + "/?project=" + SupClient.query.project;
        var iframe_1 = SupClient.html("iframe", { parent: paneElt, src: src });
        if (state != null)
            iframe_1.addEventListener("load", function () { iframe_1.contentWindow.postMessage({ type: "setState", state: state }, window.location.origin); });
    }
    else if (state != null) {
        var iframe = tabs.panesElt.querySelector("iframe[data-name='" + name + "']");
        iframe.contentWindow.postMessage({ type: "setState", state: state }, window.location.origin);
    }
    tabs.onActivate(tab);
    if (name === "main")
        homeTab.setup(tab);
}
exports.open = open;
function createTabElement(toolName, tool) {
    var tabElt = SupClient.html("li", { dataset: { pane: toolName } });
    tabElt.classList.toggle("pinned", tool.pinned);
    var iconElt = SupClient.html("img", "icon", { parent: tabElt });
    iconElt.src = "/systems/" + SupCore.system.id + "/plugins/" + tool.pluginPath + "/editors/" + toolName + "/icon.svg";
    if (!tool.pinned) {
        var tabLabel = SupClient.html("div", "label", { parent: tabElt });
        SupClient.html("div", "name", { parent: tabLabel, textContent: SupClient.i18n.t(tool.pluginPath + ":editors." + toolName + ".title") });
        var closeButton = SupClient.html("button", "close", { parent: tabElt });
        closeButton.addEventListener("click", function () { tabs.onClose(tabElt); });
    }
    return tabElt;
}
