"use strict";
var fs = require("fs");
var path = require("path");
var async = require("async");
var ProjectServer_1 = require("./ProjectServer");
var RemoteHubClient_1 = require("./RemoteHubClient");
var ProjectHub = (function () {
    function ProjectHub(globalIO, dataPath, callback) {
        var _this = this;
        this.data = {
            projects: null
        };
        this.serversById = {};
        this.onAddSocket = function (socket) {
            /* tslint:disable:no-unused-variable */
            var client = new RemoteHubClient_1.default(_this, socket);
            // this.clients.push(client);
            /* tslint:enable:no-unused-variable */
        };
        this.globalIO = globalIO;
        this.projectsPath = path.join(dataPath, "projects");
        this.buildsPath = path.join(dataPath, "builds");
        var serveProjects = function (callback) {
            async.eachSeries(fs.readdirSync(_this.projectsPath), function (folderName, cb) {
                if (folderName.indexOf(".") !== -1) {
                    cb(null);
                    return;
                }
                _this.loadingProjectFolderName = folderName;
                _this.loadProject(folderName, cb);
            }, function (err) {
                if (err != null)
                    throw err;
                _this.loadingProjectFolderName = null;
                callback();
            });
        };
        var setupProjectsList = function (callback) {
            var data = [];
            for (var id in _this.serversById)
                data.push(_this.serversById[id].data.manifest.pub);
            data.sort(SupCore.Data.Projects.sort);
            _this.data.projects = new SupCore.Data.Projects(data);
            callback();
        };
        var serve = function (callback) {
            _this.io = _this.globalIO.of("/hub");
            _this.io.on("connection", _this.onAddSocket);
            callback();
        };
        async.waterfall([serveProjects, setupProjectsList, serve], callback);
    }
    ProjectHub.prototype.saveAll = function (callback) {
        var _this = this;
        async.each(Object.keys(this.serversById), function (id, cb) {
            _this.serversById[id].save(cb);
        }, callback);
    };
    ProjectHub.prototype.loadProject = function (folderName, callback) {
        var _this = this;
        var server = new ProjectServer_1.default(this.globalIO, this.projectsPath + "/" + folderName, this.buildsPath, function (err) {
            if (err != null) {
                callback(err);
                return;
            }
            if (_this.serversById[server.data.manifest.pub.id] != null) {
                callback(new Error(("There's already a project with this ID: " + server.data.manifest.pub.id + " ") +
                    ("(" + server.projectPath + " and " + _this.serversById[server.data.manifest.pub.id].projectPath + ")")));
                return;
            }
            _this.serversById[server.data.manifest.pub.id] = server;
            callback(null);
        });
    };
    ProjectHub.prototype.removeRemoteClient = function (socketId) {
        // this.clients.splice ...
    };
    return ProjectHub;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = ProjectHub;
