"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var BaseRemoteClient_1 = require("./BaseRemoteClient");
var config_1 = require("./config");
var path = require("path");
var fs = require("fs");
var mkdirp = require("mkdirp");
var rimraf = require("rimraf");
var async = require("async");
var recursiveReaddir = require("recursive-readdir");
var RemoteProjectClient = (function (_super) {
    __extends(RemoteProjectClient, _super);
    function RemoteProjectClient(server, id, socket) {
        var _this = this;
        _super.call(this, server, socket);
        // Manifest
        this.onSetManifestProperty = function (key, value, callback) {
            _this.server.data.manifest.setProperty(key, value, function (err, actualValue) {
                if (err != null) {
                    callback(err, null);
                    return;
                }
                _this.server.io.in("sub:manifest").emit("setProperty:manifest", key, actualValue);
                callback(null, actualValue);
            });
        };
        // Entries
        this.onAddEntry = function (name, type, options, callback) {
            if (!_this.errorIfCant("editAssets", callback))
                return;
            name = name.trim();
            if (name.length === 0) {
                callback("Entry name cannot be empty");
                return;
            }
            if (name.indexOf("/") !== -1) {
                callback("Entry name cannot contain slashes");
                return;
            }
            var entry = { id: null, name: name, type: type, badges: [], dependentAssetIds: [] };
            if (options == null)
                options = {};
            _this.server.data.entries.add(entry, options.parentId, options.index, function (err, actualIndex) {
                if (err != null) {
                    callback(err, null);
                    return;
                }
                var onEntryCreated = function () {
                    _this.server.io.in("sub:entries").emit("add:entries", entry, options.parentId, actualIndex);
                    callback(null, entry.id);
                };
                if (entry.type != null) {
                    var assetClass = _this.server.system.data.assetClasses[entry.type];
                    var asset_1 = new assetClass(entry.id, null, _this.server);
                    asset_1.init({ name: entry.name }, function () {
                        var assetPath = path.join(_this.server.projectPath, "assets/" + _this.server.data.entries.getStoragePathFromId(entry.id));
                        mkdirp(assetPath, function () { asset_1.save(assetPath, onEntryCreated); });
                    });
                }
                else {
                    onEntryCreated();
                }
            });
        };
        this.onDuplicateEntry = function (newName, id, options, callback) {
            if (!_this.errorIfCant("editAssets", callback))
                return;
            var entryToDuplicate = _this.server.data.entries.byId[id];
            if (entryToDuplicate == null) {
                callback("Entry " + id + " doesn't exist");
                return;
            }
            var entry = {
                id: null, name: newName, type: entryToDuplicate.type,
                badges: [], dependentAssetIds: []
            };
            if (options == null)
                options = {};
            if (entryToDuplicate.type == null) {
                _this.server.data.entries.add(entry, options.parentId, options.index, function (err, actualIndex) {
                    if (err != null) {
                        callback(err, null);
                        return;
                    }
                    _this.server.io.in("sub:entries").emit("add:entries", entry, options.parentId, actualIndex);
                    async.each(entryToDuplicate.children, function (child, cb) {
                        _this.onDuplicateEntry(child.name, child.id, { parentId: entry.id, index: entryToDuplicate.children.indexOf(child) }, function (err, duplicatedId) {
                            if (err == null)
                                cb(null);
                            else
                                cb(new Error(err));
                        });
                    }, function (err) {
                        if (err != null)
                            callback(err.message, null);
                        else
                            callback(null, entry.id);
                    });
                });
            }
            else {
                _this.server.data.entries.add(entry, options.parentId, options.index, function (err, actualIndex) {
                    if (err != null) {
                        callback(err);
                        return;
                    }
                    var newAssetPath = path.join(_this.server.projectPath, "assets/" + _this.server.data.entries.getStoragePathFromId(entry.id));
                    _this.server.data.assets.acquire(id, null, function (err, referenceAsset) {
                        mkdirp(newAssetPath, function () {
                            referenceAsset.save(newAssetPath, function (err) {
                                _this.server.data.assets.release(id, null);
                                if (err != null) {
                                    _this.server.log("Failed to save duplicated asset at " + newAssetPath + " (duplicating " + id + ")");
                                    _this.server.log(err.toString());
                                    _this.server.data.entries.remove(entry.id, function (err) { if (err != null)
                                        _this.server.log(err); });
                                    callback("Could not save asset");
                                    return;
                                }
                                _this.server.data.assets.acquire(entry.id, null, function (err, newAsset) {
                                    _this.server.data.assets.release(entry.id, null);
                                    _this.server.io.in("sub:entries").emit("add:entries", entry, options.parentId, actualIndex);
                                    newAsset.restore();
                                    callback(null, entry.id);
                                });
                            });
                        });
                    });
                });
            }
        };
        this.onMoveEntry = function (id, parentId, index, callback) {
            if (!_this.errorIfCant("editAssets", callback))
                return;
            var oldFullAssetPath = _this.server.data.entries.getStoragePathFromId(id);
            var oldParent = _this.server.data.entries.parentNodesById[id];
            _this.server.data.entries.move(id, parentId, index, function (err, actualIndex) {
                if (err != null) {
                    callback(err);
                    return;
                }
                _this.onEntryChangeFullPath(id, oldFullAssetPath, function () {
                    if (oldParent == null || oldParent.children.length > 0)
                        return;
                    var oldParentPath = path.join(_this.server.projectPath, "assets/" + _this.server.data.entries.getStoragePathFromId(oldParent.id));
                    fs.readdir(oldParentPath, function (err, files) { if (files.length === 0)
                        fs.rmdir(oldParentPath); });
                });
                _this.server.io.in("sub:entries").emit("move:entries", id, parentId, actualIndex);
                callback(null);
            });
        };
        this.onTrashEntry = function (id, callback) {
            if (!_this.errorIfCant("editAssets", callback))
                return;
            var trashEntryRecursively = function (entry, callback) {
                var finishTrashEntry = function (err) {
                    if (err != null) {
                        callback(err);
                        return;
                    }
                    // Clear all dependencies for this entry
                    var dependentAssetIds = (entry != null) ? entry.dependentAssetIds : null;
                    var dependencies = _this.server.data.entries.dependenciesByAssetId[entry.id];
                    if (dependencies != null) {
                        var removedDependencyEntryIds = [];
                        for (var _i = 0, dependencies_1 = dependencies; _i < dependencies_1.length; _i++) {
                            var depId = dependencies_1[_i];
                            var depEntry = _this.server.data.entries.byId[depId];
                            if (depEntry == null)
                                continue;
                            var dependentAssetIds_1 = depEntry.dependentAssetIds;
                            var index = dependentAssetIds_1.indexOf(entry.id);
                            if (index !== -1) {
                                dependentAssetIds_1.splice(index, 1);
                                removedDependencyEntryIds.push(depId);
                            }
                        }
                        if (removedDependencyEntryIds.length > 0) {
                            _this.server.io.in("sub:entries").emit("remove:dependencies", entry.id, dependencies);
                        }
                        delete _this.server.data.entries.dependenciesByAssetId[entry.id];
                    }
                    var asset;
                    async.series([
                        function (cb) {
                            if (entry.type != null) {
                                _this.server.data.assets.acquire(entry.id, null, function (err, acquiredAsset) {
                                    if (err != null) {
                                        cb(err);
                                        return;
                                    }
                                    asset = acquiredAsset;
                                    cb(null);
                                });
                            }
                            else
                                cb(null);
                        }, function (cb) {
                            // Apply and clear any scheduled saved
                            var scheduledSaveCallback = _this.server.scheduledSaveCallbacks[("assets:" + entry.id)];
                            if (scheduledSaveCallback == null) {
                                cb();
                                return;
                            }
                            if (scheduledSaveCallback.timeoutId != null)
                                clearTimeout(scheduledSaveCallback.timeoutId);
                            delete _this.server.scheduledSaveCallbacks[("assets:" + entry.id)];
                            var assetPath = path.join(_this.server.projectPath, "assets/" + _this.server.data.entries.getStoragePathFromId(entry.id));
                            asset.save(assetPath, cb);
                        }, function (cb) {
                            // Delete the entry
                            _this.server.data.entries.remove(entry.id, function (err) {
                                if (err != null) {
                                    cb(new Error(err));
                                    return;
                                }
                                _this.server.io.in("sub:entries").emit("trash:entries", entry.id);
                                // Notify and clear all asset subscribers
                                var roomName = "sub:assets:" + entry.id;
                                _this.server.io.in(roomName).emit("trash:assets", entry.id);
                                var room = _this.server.io.adapter.rooms[roomName]; // room is null when the asset isn't open in any client
                                if (room != null) {
                                    for (var socketId in room.sockets) {
                                        var remoteClient = _this.server.clientsBySocketId[socketId];
                                        remoteClient.socket.leave(roomName);
                                        remoteClient.subscriptions.splice(remoteClient.subscriptions.indexOf(roomName), 1);
                                    }
                                }
                                // Generate badges for any assets depending on this entry
                                if (dependentAssetIds != null)
                                    _this.server.markMissingDependency(dependentAssetIds, id);
                                // Skip asset destruction & release if trashing a folder
                                if (asset == null) {
                                    cb(null);
                                    return;
                                }
                                // NOTE: It is important that we destroy the asset after having removed its entry
                                // from the tree so that nobody can subscribe to it after it's been destroyed
                                asset.destroy(function () {
                                    _this.server.data.assets.releaseAll(entry.id);
                                    cb(null);
                                });
                            });
                        }
                    ], callback);
                };
                if (entry.type == null) {
                    async.each(entry.children, function (entry, cb) { trashEntryRecursively(entry, cb); }, finishTrashEntry);
                }
                else
                    finishTrashEntry(null);
            };
            var trashedAssetFolder = _this.server.data.entries.getStoragePathFromId(id);
            var entry = _this.server.data.entries.byId[id];
            var gotChildren = entry.type == null && entry.children.length > 0;
            var parentEntry = _this.server.data.entries.parentNodesById[id];
            trashEntryRecursively(entry, function (err) {
                if (err != null) {
                    callback(err.message);
                    return;
                }
                // After the trash on memory, move folder to trashed assets and clean up empty folders
                var deleteFolder = function (folderPath, callback) {
                    fs.readdir(folderPath, function (err, files) {
                        if (err != null) {
                            if (err.code !== "ENOENT")
                                callback(err.message);
                            else
                                callback(null);
                            return;
                        }
                        if (files.length === 0) {
                            fs.rmdir(folderPath, function (err) {
                                if (err != null) {
                                    callback(err.message);
                                    return;
                                }
                                callback(null);
                            });
                        }
                        else
                            callback(null);
                    });
                };
                if (entry.type != null || gotChildren) {
                    _this.server.moveAssetFolderToTrash(trashedAssetFolder, function (err) {
                        if (err != null) {
                            callback(err.message);
                            return;
                        }
                        if (parentEntry != null && parentEntry.children.length === 0)
                            deleteFolder(path.join(_this.server.projectPath, "assets", _this.server.data.entries.getStoragePathFromId(parentEntry.id)), callback);
                        else
                            callback(null);
                    });
                }
                else
                    deleteFolder(path.join(_this.server.projectPath, "assets", trashedAssetFolder), callback);
            });
        };
        this.onSetEntryProperty = function (id, key, value, callback) {
            if (!_this.errorIfCant("editAssets", callback))
                return;
            if (key === "name" && value.indexOf("/") !== -1) {
                callback("Entry name cannot contain slashes");
                return;
            }
            var oldFullAssetPath = _this.server.data.entries.getStoragePathFromId(id);
            _this.server.data.entries.setProperty(id, key, value, function (err, actualValue) {
                if (err != null) {
                    callback(err);
                    return;
                }
                if (key === "name")
                    _this.onEntryChangeFullPath(id, oldFullAssetPath);
                _this.server.io.in("sub:entries").emit("setProperty:entries", id, key, actualValue);
                callback(null);
            });
        };
        this.onEntryChangeFullPath = function (assetId, oldFullAssetPath, callback) {
            var mustScheduleSave = false;
            var scheduledSaveCallback = _this.server.scheduledSaveCallbacks[("assets:" + assetId)];
            if (scheduledSaveCallback != null && scheduledSaveCallback.timeoutId != null) {
                clearTimeout(scheduledSaveCallback.timeoutId);
                scheduledSaveCallback.timeoutId = null;
                scheduledSaveCallback.callback = null;
                mustScheduleSave = true;
            }
            var assetPath = _this.server.data.entries.getStoragePathFromId(assetId);
            async.series([
                function (cb) {
                    var index = assetPath.lastIndexOf("/");
                    if (index !== -1) {
                        var parentPath = assetPath.slice(0, index);
                        mkdirp(path.join(_this.server.projectPath, "assets/" + parentPath), cb);
                    }
                    else
                        cb(null);
                }, function (cb) {
                    var oldDirPath = path.join(_this.server.projectPath, "assets/" + oldFullAssetPath);
                    var dirPath = path.join(_this.server.projectPath, "assets/" + assetPath);
                    fs.rename(oldDirPath, dirPath, function (err) {
                        if (mustScheduleSave)
                            _this.server.scheduleAssetSave(assetId);
                        if (callback != null)
                            callback();
                    });
                }
            ]);
        };
        this.onSaveEntry = function (entryId, revisionName, callback) {
            if (!_this.errorIfCant("editAssets", callback))
                return;
            if (revisionName.length === 0) {
                callback("Revision name can't be empty");
                return;
            }
            _this.server.data.entries.save(entryId, revisionName, function (err, revisionId) {
                if (err != null) {
                    callback(err);
                    return;
                }
                _this.server.data.assets.acquire(entryId, null, function (err, asset) {
                    if (err != null) {
                        callback("Could not acquire asset");
                        return;
                    }
                    _this.server.data.assets.release(entryId, null);
                    var revisionPath = path.join(_this.server.projectPath, "assetRevisions/" + entryId + "/" + revisionId + "-" + revisionName);
                    mkdirp(revisionPath, function (err) {
                        if (err != null) {
                            callback("Could not write the save");
                            return;
                        }
                        asset.save(revisionPath, function (err) {
                            if (err != null) {
                                callback("Could not write the save");
                                console.log(err);
                                return;
                            }
                            _this.server.io.in("sub:entries").emit("save:entries", entryId, revisionId, revisionName);
                            callback(null);
                        });
                    });
                });
            });
        };
        // Assets
        this.onEditAsset = function (id, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var callback = null;
            if (typeof args[args.length - 1] === "function")
                callback = args.pop();
            if (!_this.errorIfCant("editAssets", callback))
                return;
            var entry = _this.server.data.entries.byId[id];
            if (entry == null || entry.type == null) {
                callback("No such asset");
                return;
            }
            if (command == null) {
                callback("Invalid command");
                return;
            }
            var commandMethod = _this.server.system.data.assetClasses[entry.type].prototype[("server_" + command)];
            if (commandMethod == null) {
                callback("Invalid command");
                return;
            }
            // if (callback == null) { this.server.log("Ignoring edit:assets command, missing a callback"); return; }
            _this.server.data.assets.acquire(id, null, function (err, asset) {
                if (err != null) {
                    callback("Could not acquire asset");
                    return;
                }
                commandMethod.call.apply(commandMethod, [asset, _this].concat(args, [function (err, ack) {
                    var callbackArgs = [];
                    for (var _i = 2; _i < arguments.length; _i++) {
                        callbackArgs[_i - 2] = arguments[_i];
                    }
                    _this.server.data.assets.release(id, null);
                    if (err != null) {
                        callback(err);
                        return;
                    }
                    (_a = _this.server.io.in("sub:assets:" + id)).emit.apply(_a, ["edit:assets", id, command].concat(callbackArgs));
                    callback(null, ack);
                    var _a;
                }]));
            });
        };
        this.onRestoreAsset = function (assetId, revisionId, callback) {
            var entry = _this.server.data.entries.byId[assetId];
            if (entry == null || entry.type == null) {
                callback("No such asset");
                return;
            }
            var assetClass = _this.server.system.data.assetClasses[entry.type];
            var newAsset = new assetClass(assetId, null, _this.server);
            var revisionName = _this.server.data.entries.revisionsByEntryId[assetId][revisionId];
            var revisionPath = "assetRevisions/" + assetId + "/" + revisionId + "-" + revisionName;
            newAsset.load(path.join(_this.server.projectPath, revisionPath));
            newAsset.on("load", function () {
                _this.server.data.assets.acquire(assetId, null, function (err, asset) {
                    if (err != null) {
                        callback("Could not acquire asset");
                        return;
                    }
                    _this.server.data.assets.release(assetId, null);
                    for (var _i = 0, _a = entry.badges; _i < _a.length; _i++) {
                        var badge = _a[_i];
                        asset.emit("clearBadge", badge.id);
                    }
                    entry.badges.length = 0;
                    asset.pub = newAsset.pub;
                    asset.setup();
                    asset.restore();
                    asset.emit("change");
                    _this.server.io.in("sub:assets:" + assetId).emit("restore:assets", assetId, entry.type, asset.pub);
                    callback(null);
                });
            });
        };
        this.onGetAssetRevision = function (assetId, revisionId, callback) {
            var entry = _this.server.data.entries.byId[assetId];
            if (entry == null || entry.type == null) {
                callback("No such asset");
                return;
            }
            var assetClass = _this.server.system.data.assetClasses[entry.type];
            var revisionAsset = new assetClass(assetId, null, _this.server);
            var revisionName = _this.server.data.entries.revisionsByEntryId[assetId][revisionId];
            var revisionPath = "assetRevisions/" + assetId + "/" + revisionId + "-" + revisionName;
            revisionAsset.load(path.join(_this.server.projectPath, revisionPath));
            revisionAsset.on("load", function () { callback(null, revisionAsset.pub); });
        };
        // Resources
        this.onEditResource = function (id, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var callback = null;
            if (typeof args[args.length - 1] === "function")
                callback = args.pop();
            if (!_this.errorIfCant("editResources", callback))
                return;
            if (command == null) {
                callback("Invalid command");
                return;
            }
            var commandMethod = _this.server.system.data.resourceClasses[id].prototype[("server_" + command)];
            if (commandMethod == null) {
                callback("Invalid command");
                return;
            }
            // if (callback == null) { this.server.log("Ignoring edit:assets command, missing a callback"); return; }
            _this.server.data.resources.acquire(id, null, function (err, resource) {
                if (err != null) {
                    callback("Could not acquire resource");
                    return;
                }
                commandMethod.call.apply(commandMethod, [resource, _this].concat(args, [function (err, ack) {
                    var callbackArgs = [];
                    for (var _i = 2; _i < arguments.length; _i++) {
                        callbackArgs[_i - 2] = arguments[_i];
                    }
                    _this.server.data.resources.release(id, null);
                    if (err != null) {
                        callback(err);
                        return;
                    }
                    (_a = _this.server.io.in("sub:resources:" + id)).emit.apply(_a, ["edit:resources", id, command].concat(callbackArgs));
                    callback(null, ack);
                    var _a;
                }]));
            });
        };
        // Rooms
        this.onEditRoom = function (id, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var callback = null;
            if (typeof args[args.length - 1] === "function")
                callback = args.pop();
            if (!_this.errorIfCant("editRooms", callback))
                return;
            if (command == null) {
                callback("Invalid command");
                return;
            }
            var commandMethod = SupCore.Data.Room.prototype[("server_" + command)];
            if (commandMethod == null) {
                callback("Invalid command");
                return;
            }
            // if (callback == null) { this.server.log("Ignoring edit:rooms command, missing a callback"); return; }
            _this.server.data.rooms.acquire(id, null, function (err, room) {
                if (err != null) {
                    callback("Could not acquire room");
                    return;
                }
                commandMethod.call.apply(commandMethod, [room, _this].concat(args, [function (err) {
                    var callbackArgs = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        callbackArgs[_i - 1] = arguments[_i];
                    }
                    _this.server.data.rooms.release(id, null);
                    if (err != null) {
                        callback(err);
                        return;
                    }
                    (_a = _this.server.io.in("sub:rooms:" + id)).emit.apply(_a, ["edit:rooms", id, command].concat(callbackArgs));
                    callback(null, (callbackArgs[0] != null) ? callbackArgs[0].id : null);
                    var _a;
                }]));
            });
        };
        // Project
        this.onVacuumProject = function (callback) {
            if (!_this.errorIfCant("vacuumProject", callback))
                return;
            var trashedAssetsPath = path.join(_this.server.projectPath, "trashedAssets");
            fs.readdir(trashedAssetsPath, function (err, trashedAssetFolders) {
                if (err != null) {
                    if (err.code === "ENOENT")
                        trashedAssetFolders = [];
                    else
                        throw err;
                }
                var removedFolderCount = 0;
                async.each(trashedAssetFolders, function (trashedAssetFolder, cb) {
                    var folderPath = path.join(trashedAssetsPath, trashedAssetFolder);
                    rimraf(folderPath, function (err) {
                        if (err != null)
                            SupCore.log("Could not delete " + folderPath + ".\n" + err.stack);
                        else
                            removedFolderCount++;
                        cb();
                    });
                }, function () { callback(null, removedFolderCount); });
            });
        };
        this.onBuildProject = function (callback) {
            if (!_this.errorIfCant("buildProject", callback))
                return;
            // this.server.log("Building project...");
            var buildId = _this.server.nextBuildId;
            _this.server.nextBuildId++;
            var buildPath = _this.server.buildsPath + "/" + buildId;
            try {
                fs.mkdirSync(_this.server.buildsPath);
            }
            catch (e) { }
            try {
                fs.mkdirSync(buildPath);
            }
            catch (err) {
                callback("Could not create folder for build " + buildId);
                return;
            }
            _this.server.system.serverBuild(_this.server, buildPath, function (err) {
                if (err != null) {
                    callback("Failed to create build " + buildId + ": " + err);
                    return;
                }
                // Collect paths to all build files
                var files = [];
                recursiveReaddir(buildPath, function (err, entries) {
                    for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                        var entry = entries_1[_i];
                        var relativePath = path.relative(buildPath, entry);
                        if (path.sep === "\\")
                            relativePath = relativePath.replace(/\\/g, "/");
                        files.push("/builds/" + _this.server.data.manifest.pub.id + "/" + buildId + "/" + relativePath);
                    }
                    callback(null, buildId.toString());
                    // Remove an old build to avoid using too much disk space
                    var buildToDeleteId = buildId - config_1.server.maxRecentBuilds;
                    var buildToDeletePath = _this.server.buildsPath + "/" + buildToDeleteId;
                    rimraf(buildToDeletePath, function (err) {
                        if (err != null) {
                            _this.server.log("Failed to remove build " + buildToDeleteId + ":");
                            _this.server.log(err.toString());
                        }
                    });
                });
            });
        };
        this.id = id;
        this.socket.emit("welcome", this.id, {
            systemId: this.server.system.id,
            buildPort: config_1.server.buildPort,
            supportsServerBuild: this.server.system.serverBuild != null
        });
        // Manifest
        this.socket.on("setProperty:manifest", this.onSetManifestProperty);
        // Entries
        this.socket.on("add:entries", this.onAddEntry);
        this.socket.on("duplicate:entries", this.onDuplicateEntry);
        this.socket.on("move:entries", this.onMoveEntry);
        this.socket.on("trash:entries", this.onTrashEntry);
        this.socket.on("setProperty:entries", this.onSetEntryProperty);
        this.socket.on("save:entries", this.onSaveEntry);
        // Assets
        this.socket.on("edit:assets", this.onEditAsset);
        this.socket.on("restore:assets", this.onRestoreAsset);
        this.socket.on("getRevision:assets", this.onGetAssetRevision);
        // Resources
        this.socket.on("edit:resources", this.onEditResource);
        // Rooms
        this.socket.on("edit:rooms", this.onEditRoom);
        // Project
        this.socket.on("vacuum:project", this.onVacuumProject);
        this.socket.on("build:project", this.onBuildProject);
    }
    // TODO: Implement roles and capabilities
    RemoteProjectClient.prototype.can = function (action) { return true; };
    return RemoteProjectClient;
}(BaseRemoteClient_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = RemoteProjectClient;
