"use strict";
var fs = require("fs");
var mkdirp = require("mkdirp");
var utils = require("./utils");
function init(systemId, pluginFullName) {
    if (!utils.folderNameRegex.test(systemId))
        utils.emitError("Invalid system ID: only lowercase letters, numbers and dashes are allowed.");
    if (pluginFullName != null && !utils.pluginNameRegex.test(pluginFullName))
        utils.emitError("Invalid plugin name: only two sets of letters and numbers separated by a slash are allowed.");
    var systemFolderName = (utils.systemsById[systemId] != null) ? utils.systemsById[systemId].folderName : null;
    if (pluginFullName == null && systemFolderName != null)
        utils.emitError("You already have a system with the ID " + systemId + " installed as systems/" + systemFolderName + ".");
    else if (pluginFullName != null && systemFolderName == null)
        utils.emitError("You don't have a system with the ID " + systemId + " installed.");
    if (systemFolderName == null)
        systemFolderName = systemId;
    var systemStats;
    try {
        systemStats = fs.lstatSync(utils.systemsPath + "/" + systemFolderName);
    }
    catch (err) { }
    if (pluginFullName == null) {
        if (systemStats != null)
            utils.emitError("systems/" + systemFolderName + " already exists.");
    }
    else {
        if (systemStats == null)
            utils.emitError("systems/" + systemFolderName + " doesn't exist.");
        var pluginStats = void 0;
        try {
            pluginStats = fs.lstatSync(utils.systemsPath + "/" + systemFolderName + "/plugins/" + pluginFullName);
        }
        catch (err) { }
        if (pluginStats != null)
            utils.emitError("systems/" + systemFolderName + "/plugins/" + pluginFullName + " already exists.");
    }
    utils.getRegistry(function (err, registry) {
        var registrySystemEntry = registry.systems[systemId];
        if (pluginFullName == null && registrySystemEntry != null) {
            utils.emitError("System " + systemId + " already exists.");
        }
        else if (pluginFullName != null && registrySystemEntry != null) {
            var _a = pluginFullName.split("/"), pluginAuthor = _a[0], pluginName = _a[1];
            if (registrySystemEntry.plugins[pluginAuthor] != null && registrySystemEntry.plugins[pluginAuthor][pluginName] != null)
                utils.emitError("Plugin " + pluginFullName + " on system " + systemId + " already exists.");
        }
        if (pluginFullName == null)
            initSystem(systemId);
        else
            initPlugin(systemFolderName, systemId, pluginFullName);
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = init;
function initSystem(systemId) {
    var packageJSON = JSON.stringify({
        name: "superpowers-" + systemId,
        description: "A system for Superpowers, the HTML5 app for real-time collaborative projects",
        superpowers: {
            systemId: systemId,
            publishedPluginBundles: []
        }
    }, null, 2) + "\n";
    var systemPath = utils.systemsPath + "/" + systemId;
    fs.mkdirSync(systemPath);
    fs.writeFileSync(systemPath + "/package.json", packageJSON);
    var localeJSON = JSON.stringify({
        title: "" + systemId,
        description: "(Edit systems/" + systemId + "/public/locales/en/system.json to change the title and description)"
    }, null, 2) + "\n";
    mkdirp.sync(systemPath + "/public/locales/en");
    fs.writeFileSync(systemPath + "/public/locales/en/system.json", localeJSON);
    console.log("A system named " + systemId + " has been initialized.");
}
function initPlugin(systemFolderName, systemId, pluginName) {
    var pluginSlug = pluginName.replace(/\//g, "-").replace(/[A-Z]/g, function (x) { return ("-" + x.toLowerCase()); });
    var packageJSON = JSON.stringify({
        name: "superpowers-" + systemId + "-" + pluginSlug + "-plugin",
        description: "Plugin for Superpowers " + systemId,
        scripts: {
            "build": "gulp --gulpfile=../../../../../scripts/pluginGulpfile.js --cwd=."
        }
    }, null, 2) + "\n";
    var pluginPath = utils.systemsPath + "/" + systemFolderName + "/plugins/" + pluginName;
    mkdirp.sync(pluginPath);
    fs.writeFileSync(pluginPath + "/package.json", packageJSON);
    var tsconfigJSON = JSON.stringify({
        "compilerOptions": {
            "module": "commonjs",
            "target": "es5",
            "noImplicitAny": true
        },
        "exclude": [
            "node_modules",
            "typings"
        ]
    }, null, 2) + "\n";
    fs.writeFileSync(pluginPath + "/tsconfig.json", tsconfigJSON);
    var indexDTS = "/// <reference path=\"../../../../../SupClient/SupClient.d.ts\" />\n/// <reference path=\"../../../../../SupCore/SupCore.d.ts\" />\n";
    fs.writeFileSync(pluginPath + "/index.d.ts", indexDTS);
    console.log("A plugin named " + pluginName + " has been initialized in systems/" + systemFolderName + ".");
}
