"use strict";
var utils = require("./utils");
function install(systemId, pluginFullName) {
    var localSystem = utils.systemsById[systemId];
    if (utils.downloadURL != null) {
        if (pluginFullName == null) {
            if (localSystem != null)
                utils.emitError("System " + systemId + " is already installed.");
            installSystem(systemId, utils.downloadURL);
        }
        else {
            var _a = pluginFullName.split("/"), authorName = _a[0], pluginName = _a[1];
            var localPlugin = localSystem != null && localSystem.plugins[authorName] != null ? localSystem.plugins[authorName][pluginName] : null;
            if (localPlugin != null)
                utils.emitError("Plugin " + pluginFullName + " is already installed.");
            installPlugin(systemId, pluginFullName, utils.downloadURL);
        }
        return;
    }
    utils.getRegistry(function (err, registry) {
        if (err)
            utils.emitError("Error while fetching registry:", err.stack);
        var registrySystem = registry.systems[systemId];
        if (registrySystem == null) {
            console.error("System " + systemId + " is not on the registry.");
            utils.listAvailableSystems(registry);
            process.exit(1);
        }
        if (localSystem != null) {
            if (pluginFullName == null) {
                console.error("System " + systemId + " is already installed.");
                utils.listAvailableSystems(registry);
                process.exit(1);
            }
            else if (pluginFullName === "") {
                utils.listAvailablePlugins(registry, systemId);
                process.exit(0);
            }
            var _a = pluginFullName.split("/"), authorName = _a[0], pluginName = _a[1];
            var localPlugin = localSystem != null && localSystem.plugins[authorName] != null ? localSystem.plugins[authorName][pluginName] : null;
            var registryPlugin = registrySystem.plugins[authorName] != null ? registrySystem.plugins[authorName][pluginName] : null;
            if (registryPlugin == null) {
                console.error("Plugin " + pluginFullName + " is not on the registry.");
                utils.listAvailablePlugins(registry, systemId);
                process.exit(1);
            }
            if (localPlugin != null) {
                console.error("Plugin " + pluginFullName + " is already installed.");
                utils.listAvailablePlugins(registry, systemId);
                process.exit(1);
            }
            installPlugin(systemId, pluginFullName, registryPlugin.downloadURL);
        }
        else {
            if (pluginFullName != null)
                utils.emitError("System " + systemId + " is not installed.");
            installSystem(systemId, registrySystem.downloadURL);
        }
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = install;
function installSystem(systemId, downloadURL) {
    console.log("Installing system " + systemId + "...");
    var systemPath = utils.systemsPath + "/" + systemId;
    utils.downloadRelease(downloadURL, systemPath, function (err) {
        if (err != null)
            utils.emitError("Failed to install the system.", err);
        console.log("System successfully installed.");
        process.exit(0);
    });
}
function installPlugin(systemId, pluginFullName, downloadURL) {
    console.log("Installing plugin " + pluginFullName + " on system " + systemId + "...");
    var pluginPath = utils.systemsPath + "/" + utils.systemsById[systemId].folderName + "/plugins/" + pluginFullName;
    utils.downloadRelease(downloadURL, pluginPath, function (err) {
        if (err != null)
            utils.emitError("Failed to install the plugin.", err);
        console.log("Plugin successfully installed.");
        process.exit(0);
    });
}
