"use strict";
var path = require("path");
var fs = require("fs");
var crypto = require("crypto");
var http = require("http");
var express = require("express");
var url = require("url");
var socketio = require("socket.io");
var passportMiddleware_1 = require("../passportMiddleware");
var bodyParser = require("body-parser");
var cookieParser = require("cookie-parser");
var expressSession = require("express-session");
var passportSocketIo = require("passport.socketio");
var config = require("../config");
var schemas = require("../schemas");
var getLocalizedFilename_1 = require("../getLocalizedFilename");
var SupCore = require("../../SupCore");
var loadSystems_1 = require("../loadSystems");
var ProjectHub_1 = require("../ProjectHub");
// NOTE: We explicitly add core path to NODE_PATH so systems can load modules from core
process.env["NODE_PATH"] = path.resolve(__dirname + "/../../node_modules");
/* tslint:disable */
require("module").Module._initPaths();
/* tslint:enable */
var dataPath;
var hub = null;
var mainApp = null;
var mainHttpServer;
var io;
var buildApp = null;
var buildHttpServer;
var languageIds;
var isQuitting = false;
var memoryStore;
function onUncaughtException(err) {
    if (hub != null && hub.loadingProjectFolderName != null) {
        SupCore.log("The server crashed while loading project \"" + hub.loadingProjectFolderName + "\".\n" + err.stack);
    }
    else {
        SupCore.log("The server crashed.\n" + err.stack);
    }
    process.exit(1);
}
function start(serverDataPath) {
    dataPath = serverDataPath;
    SupCore.log("Using data from " + dataPath + ".");
    process.on("uncaughtException", onUncaughtException);
    loadConfig();
    var _a = JSON.parse(fs.readFileSync(__dirname + "/../../package.json", { encoding: "utf8" })), version = _a.version, appApiVersion = _a.superpowers.appApiVersion;
    SupCore.log("Server v" + version + " starting...");
    fs.writeFileSync(__dirname + "/../../public/superpowers.json", JSON.stringify({ version: version, appApiVersion: appApiVersion, hasPassword: config.server.password.length !== 0 }, null, 2));
    // SupCore
    global.SupCore = SupCore;
    SupCore.systemsPath = path.join(dataPath, "systems");
    // List available languages
    languageIds = fs.readdirSync(__dirname + "/../../public/locales");
    languageIds.unshift("none");
    // Main HTTP server
    mainApp = express();
    if (typeof config.server.sessionSecret !== "string")
        throw new Error("serverConfig.sessionSecret is null");
    memoryStore = new expressSession.MemoryStore();
    try {
        var sessionsJSON = fs.readFileSync(__dirname + "/../../sessions.json", { encoding: "utf8" });
        memoryStore.sessions = JSON.parse(sessionsJSON);
    }
    catch (err) {
    }
    var sessionSettings = {
        name: "supSession",
        secret: config.server.sessionSecret,
        store: memoryStore,
        resave: false,
        saveUninitialized: false,
        cookie: { maxAge: 1000 * 60 * 60 * 24 * 30 }
    };
    mainApp.use(cookieParser());
    mainApp.use(bodyParser.urlencoded({ extended: false }));
    mainApp.use(handleLanguage);
    mainApp.use(expressSession(sessionSettings));
    mainApp.use(passportMiddleware_1.default.initialize());
    mainApp.use(passportMiddleware_1.default.session());
    mainApp.get("/", function (req, res) { res.redirect("/hub"); });
    mainApp.post("/login", ensurePasswordFieldIsntEmpty, passportMiddleware_1.default.authenticate("local", { successReturnToOrRedirect: "/", failureRedirect: "/login" }));
    mainApp.get("/login", serveLoginIndex);
    mainApp.get("/logout", function (req, res) { req.logout(); res.redirect("/"); });
    mainApp.get("/hub", enforceAuth, serveHubIndex);
    mainApp.get("/project", enforceAuth, serveProjectIndex);
    mainApp.get("/build", enforceAuth, serveBuildIndex);
    mainApp.get("/serverBuild", enforceAuth, serveServerBuildIndex);
    mainApp.use("/projects/:projectId/*", serveProjectWildcard);
    mainApp.use("/", express.static(__dirname + "/../../public"));
    mainHttpServer = http.createServer(mainApp);
    mainHttpServer.on("error", onHttpServerError.bind(null, config.server.mainPort));
    io = socketio(mainHttpServer, { transports: ["websocket"] });
    io.use(passportSocketIo.authorize({
        cookieParser: cookieParser,
        key: sessionSettings.name,
        secret: sessionSettings.secret,
        store: sessionSettings.store
    }));
    // Build HTTP server
    buildApp = express();
    buildApp.get("/", redirectToHub);
    buildApp.get("/systems/:systemId/SupCore.js", serveSystemSupCore);
    buildApp.use("/", express.static(__dirname + "/../../public"));
    buildApp.use(function (req, res, next) {
        var originValue = req.get("origin");
        if (originValue == null) {
            next();
            return;
        }
        var origin = url.parse(originValue);
        if (origin.hostname === req.hostname && origin.port === config.server.mainPort.toString()) {
            res.header("Access-Control-Allow-Origin", originValue);
        }
        next();
    });
    buildApp.get("/builds/:projectId/:buildId/*", function (req, res) {
        var projectServer = hub.serversById[req.params.projectId];
        if (projectServer == null) {
            res.status(404).end("No such project");
            return;
        }
        var buildId = req.params.buildId;
        if (buildId === "latest")
            buildId = (projectServer.nextBuildId - 1).toString();
        res.sendFile(path.join(projectServer.buildsPath, buildId, req.params[0]));
    });
    buildHttpServer = http.createServer(buildApp);
    buildHttpServer.on("error", onHttpServerError.bind(null, config.server.buildPort));
    loadSystems_1.default(mainApp, buildApp, onSystemsLoaded);
    // Save on exit and handle crashes
    process.on("SIGTERM", onExit);
    process.on("SIGINT", onExit);
    process.on("message", function (msg) { if (msg === "stop")
        onExit(); });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = start;
function loadConfig() {
    var mustWriteConfig = false;
    var serverConfigPath = dataPath + "/config.json";
    if (fs.existsSync(serverConfigPath)) {
        config.server = JSON.parse(fs.readFileSync(serverConfigPath, { encoding: "utf8" }));
        schemas.validate(config, "config");
        for (var key in config.defaults) {
            if (config.server[key] == null)
                config.server[key] = config.defaults[key];
        }
    }
    else {
        mustWriteConfig = true;
        config.server = {};
        for (var key in config.defaults)
            config.server[key] = config.defaults[key];
    }
    if (config.server.sessionSecret == null) {
        config.server.sessionSecret = crypto.randomBytes(48).toString("hex");
        mustWriteConfig = true;
    }
    if (mustWriteConfig) {
        fs.writeFileSync(serverConfigPath, JSON.stringify(config.server, null, 2) + "\n", { encoding: "utf8" });
    }
}
function handleLanguage(req, res, next) {
    if (req.cookies["supLanguage"] == null) {
        var languageCode = req.header("Accept-Language");
        if (languageCode != null) {
            languageCode = languageCode.split(",")[0];
            if (languageIds.indexOf(languageCode) === -1 && languageCode.indexOf("-") !== -1) {
                languageCode = languageCode.split("-")[0];
            }
        }
        if (languageIds.indexOf(languageCode) === -1)
            languageCode = "en";
        res.cookie("supLanguage", languageCode);
    }
    next();
}
function enforceAuth(req, res, next) {
    if (!req.isAuthenticated()) {
        req.session["returnTo"] = req.originalUrl;
        res.redirect("/login");
        return;
    }
    next();
}
function serveHubIndex(req, res) {
    var localizedIndex = getLocalizedFilename_1.default("index.html", req.cookies["supLanguage"]);
    res.sendFile(path.resolve(__dirname + "/../../public/hub/" + localizedIndex));
}
function serveProjectIndex(req, res) {
    var localizedIndex = getLocalizedFilename_1.default("index.html", req.cookies["supLanguage"]);
    res.sendFile(path.resolve(__dirname + "/../../public/project/" + localizedIndex));
}
function ensurePasswordFieldIsntEmpty(req, res, next) {
    // This is required so that passport-local doesn't reject logins on servers without a password
    if (req.body.password === "" || req.body.password == null)
        req.body.password = "_";
    next();
}
function serveLoginIndex(req, res) {
    var localizedIndex = getLocalizedFilename_1.default("index.html", req.cookies["supLanguage"]);
    res.sendFile(path.resolve(__dirname + "/../../public/login/" + localizedIndex));
}
function serveBuildIndex(req, res) {
    var localizedIndex = getLocalizedFilename_1.default("index.html", req.cookies["supLanguage"]);
    res.sendFile(path.resolve(__dirname + "/../../public/build/" + localizedIndex));
}
function serveServerBuildIndex(req, res) {
    var localizedIndex = getLocalizedFilename_1.default("index.html", req.cookies["supLanguage"]);
    res.sendFile(path.resolve(__dirname + "/../../public/serverBuild/" + localizedIndex));
}
function serveProjectWildcard(req, res) {
    var projectPath = hub.serversById[req.params.projectId].projectPath;
    res.sendFile(req.params[0], { root: projectPath + "/public" }, function (err) {
        if (req.params[0] === "icon.png")
            res.sendFile("/images/default-project-icon.png", { root: __dirname + "/../../public" });
    });
}
function onHttpServerError(port, err) {
    if (err.code === "EADDRINUSE") {
        SupCore.log("Could not start the server: another application is already listening on port " + port + ".");
        process.exit(1);
    }
    else
        throw (err);
}
function redirectToHub(req, res) {
    res.redirect("http://" + req.hostname + ":" + config.server.mainPort + "/hub/");
}
function serveSystemSupCore(req, res) {
    res.sendFile("SupCore.js", { root: __dirname + "/../../public" });
}
function onSystemsLoaded() {
    mainApp.use(handle404);
    buildApp.use(handle404);
    // Project hub
    hub = new ProjectHub_1.default(io, dataPath, function (err) {
        if (err != null) {
            SupCore.log("Failed to start server:\n" + err.stack);
            return;
        }
        SupCore.log("Loaded " + Object.keys(hub.serversById).length + " projects from " + hub.projectsPath + ".");
        var hostname = (config.server.password.length === 0) ? "localhost" : "";
        mainHttpServer.listen(config.server.mainPort, hostname, function () {
            buildHttpServer.listen(config.server.buildPort, hostname, function () {
                SupCore.log("Main server started on port " + config.server.mainPort + ", build server started on port " + config.server.buildPort + ".");
                if (hostname === "localhost")
                    SupCore.log("NOTE: Setup a password to allow other people to connect to your server.");
                if (process != null && process.send != null)
                    process.send({ type: "started" });
            });
        });
    });
}
function handle404(err, req, res, next) {
    if (err.status === 404) {
        res.status(404).end("File not found");
        return;
    }
    next();
}
function onExit() {
    if (isQuitting)
        return;
    isQuitting = true;
    mainHttpServer.close();
    buildHttpServer.close();
    if (hub == null) {
        process.exit(0);
        return;
    }
    SupCore.log("Saving all projects...");
    hub.saveAll(function (err) {
        var hadError = false;
        if (err != null) {
            SupCore.log("Error while exiting:\n" + err.stack);
            hadError = true;
        }
        SupCore.log("Saving sessions...");
        try {
            var sessionsJSON = JSON.stringify(memoryStore.sessions, null, 2);
            fs.writeFileSync(__dirname + "/../../sessions.json", sessionsJSON);
        }
        catch (err) {
            SupCore.log("Failed to save sessions:\n" + err.stack);
            hadError = true;
        }
        if (!hadError)
            SupCore.log("Exited cleanly.");
        process.exit(0);
    });
}
