"use strict";
var readline = require("readline");
var rimraf = require("rimraf");
var fs = require("fs");
var utils = require("./utils");
function uninstall(systemId, pluginFullName) {
    var localSystem = utils.systemsById[systemId];
    if (localSystem == null)
        utils.emitError("System " + systemId + " is not installed.");
    if (pluginFullName == null) {
        // Uninstall system
        if (localSystem.isDev)
            utils.emitError("System " + systemId + " is a development version.");
        if (utils.force) {
            uninstallSystem(localSystem.folderName);
            return;
        }
        var r1 = readline.createInterface({ input: process.stdin, output: process.stdout });
        r1.question("Are you sure you want to uninstall the system " + systemId + "? (yes/no): ", function (answer) {
            if (answer === "yes") {
                console.log("Uninstalling system " + systemId + "...");
                uninstallSystem(localSystem.folderName);
            }
            else {
                console.log("Uninstall canceled.");
                process.exit(0);
            }
        });
    }
    else {
        // Uninstall plugin
        var _a = pluginFullName.split("/"), authorName_1 = _a[0], pluginName = _a[1];
        if (utils.builtInPluginAuthors.indexOf(authorName_1) !== -1)
            utils.emitError("Built-in plugins can not be uninstalled.");
        var localPlugin = localSystem.plugins[authorName_1] != null ? localSystem.plugins[authorName_1][pluginName] : null;
        if (localPlugin == null)
            utils.emitError("Plugin " + pluginFullName + " is not installed.");
        if (localPlugin.isDev)
            utils.emitError("Plugin " + pluginFullName + " is a development version.");
        if (utils.force) {
            uninstallPlugin(localSystem.folderName, pluginFullName, authorName_1);
            return;
        }
        var r1 = readline.createInterface({ input: process.stdin, output: process.stdout });
        r1.question("Are you sure you want to uninstall the plugin " + pluginFullName + "? (yes/no): ", function (answer) {
            if (answer === "yes") {
                console.log("Uninstalling plugin " + pluginFullName + " from system " + systemId + "...");
                uninstallPlugin(localSystem.folderName, pluginFullName, authorName_1);
            }
            else {
                console.log("Uninstall canceled.");
                process.exit(0);
            }
        });
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = uninstall;
function uninstallSystem(systemFolderName) {
    rimraf(utils.systemsPath + "/" + systemFolderName, function (err) {
        if (err != null) {
            utils.emitError("Failed to uninstalled system.");
        }
        else {
            console.log("System successfully uninstalled.");
            process.exit(0);
        }
    });
}
function uninstallPlugin(systemFolderName, pluginFullName, authorName) {
    rimraf(utils.systemsPath + "/" + systemFolderName + "/plugins/" + pluginFullName, function (err) {
        if (err != null) {
            utils.emitError("Failed to uninstalled plugin.");
        }
        else {
            if (fs.readdirSync(utils.systemsPath + "/" + systemFolderName + "/plugins/" + authorName).length === 0)
                fs.rmdirSync(utils.systemsPath + "/" + systemFolderName + "/plugins/" + authorName);
            console.log("Plugin successfully uninstalled.");
            process.exit(0);
        }
    });
}
