"use strict";
var yargs = require("yargs");
var mkdirp = require("mkdirp");
var path = require("path");
var fs = require("fs");
var async = require("async");
var readline = require("readline");
/* tslint:disable */
var https = require("follow-redirects").https;
var yauzl = require("yauzl");
/* tslint:enable */
exports.folderNameRegex = /^[a-z0-9_-]+$/;
exports.pluginNameRegex = /^[A-Za-z0-9]+\/[A-Za-z0-9]+$/;
// Data path
var argv = yargs
    .describe("data-path", "Path to store/read data files from, including config and projects")
    .describe("download-url", "Url to download a release")
    .boolean("force")
    .argv;
exports.dataPath = argv["data-path"] != null ? path.resolve(argv["data-path"]) : path.resolve(__dirname + "/../..");
mkdirp.sync(exports.dataPath);
mkdirp.sync(exports.dataPath + "/projects");
mkdirp.sync(exports.dataPath + "/builds");
exports.systemsPath = exports.dataPath + "/systems";
mkdirp.sync(exports.systemsPath);
exports.force = argv["force"];
exports.downloadURL = argv["download-url"];
// Systems and plugins
exports.builtInPluginAuthors = ["default", "common", "extra"];
exports.systemsById = {};
for (var _i = 0, _a = fs.readdirSync(exports.systemsPath); _i < _a.length; _i++) {
    var entry = _a[_i];
    if (!exports.folderNameRegex.test(entry))
        continue;
    if (!fs.statSync(exports.systemsPath + "/" + entry).isDirectory)
        continue;
    var systemId = void 0;
    var systemVersion = void 0;
    var systemPath = exports.systemsPath + "/" + entry;
    try {
        var packageDataFile = fs.readFileSync(systemPath + "/package.json", { encoding: "utf8" });
        var packageData = JSON.parse(packageDataFile);
        systemId = packageData.superpowers.systemId;
        systemVersion = packageData.version;
    }
    catch (err) {
        emitError("Could not load system id from systems/" + entry + "/package.json:", err.stack);
    }
    var isDev = true;
    try {
        if (!fs.lstatSync(systemPath + "/.git").isDirectory())
            isDev = false;
    }
    catch (err) {
        isDev = false;
    }
    exports.systemsById[systemId] = { folderName: entry, version: systemVersion, isDev: isDev, plugins: {} };
    var pluginAuthors = void 0;
    try {
        pluginAuthors = fs.readdirSync(systemPath + "/plugins");
    }
    catch (err) { }
    if (pluginAuthors == null)
        continue;
    for (var _b = 0, pluginAuthors_1 = pluginAuthors; _b < pluginAuthors_1.length; _b++) {
        var pluginAuthor = pluginAuthors_1[_b];
        if (exports.builtInPluginAuthors.indexOf(pluginAuthor) !== -1)
            continue;
        if (!exports.folderNameRegex.test(pluginAuthor))
            continue;
        exports.systemsById[systemId].plugins[pluginAuthor] = {};
        for (var _c = 0, _d = fs.readdirSync(systemPath + "/plugins/" + pluginAuthor); _c < _d.length; _c++) {
            var pluginName = _d[_c];
            if (!exports.folderNameRegex.test(pluginName))
                continue;
            var pluginPath = systemPath + "/plugins/" + pluginAuthor + "/" + pluginName;
            if (!fs.statSync(pluginPath).isDirectory)
                continue;
            var pluginVersion = void 0;
            try {
                var packageDataFile = fs.readFileSync(pluginPath + "/package.json", { encoding: "utf8" });
                var packageData = JSON.parse(packageDataFile);
                pluginVersion = packageData.version;
            }
            catch (err) {
                emitError("Could not load plugin verson from systems/" + entry + "/" + pluginAuthor + "/" + pluginName + "/package.json:", err.stack);
            }
            var isDev_1 = true;
            try {
                if (!fs.lstatSync(pluginPath + "/.git").isDirectory())
                    isDev_1 = false;
            }
            catch (err) {
                isDev_1 = false;
            }
            exports.systemsById[systemId].plugins[pluginAuthor][pluginName] = { version: pluginVersion, isDev: isDev_1 };
        }
    }
}
function listAvailableSystems(registry) { console.log("Available systems: " + Object.keys(registry.systems).join(", ") + "."); }
exports.listAvailableSystems = listAvailableSystems;
function listAvailablePlugins(registry, systemId) {
    var pluginAuthors = Object.keys(registry.systems[systemId].plugins);
    if (pluginAuthors.length === 0) {
        console.log("No plugins found.");
    }
    else {
        var pluginCount = 0;
        for (var _i = 0, pluginAuthors_2 = pluginAuthors; _i < pluginAuthors_2.length; _i++) {
            var pluginAuthor = pluginAuthors_2[_i];
            pluginCount += Object.keys(registry.systems[systemId].plugins[pluginAuthor]).length;
        }
        for (var _a = 0, pluginAuthors_3 = pluginAuthors; _a < pluginAuthors_3.length; _a++) {
            var pluginAuthor = pluginAuthors_3[_a];
            console.log("  " + pluginAuthor + "/");
            for (var _b = 0, _c = Object.keys(registry.systems[systemId].plugins[pluginAuthor]); _b < _c.length; _b++) {
                var pluginName = _c[_b];
                console.log("    " + pluginName);
            }
        }
    }
}
exports.listAvailablePlugins = listAvailablePlugins;
var currentRegistryVersion = 2;
;
function getRegistry(callback) {
    var registryUrl = "https://raw.githubusercontent.com/superpowers/superpowers-registry/master/registry.json";
    var request = https.get(registryUrl, function (res) {
        if (res.statusCode !== 200) {
            callback(new Error("Unexpected status code: " + res.statusCode), null);
            return;
        }
        var content = "";
        res.on("data", function (chunk) { content += chunk; });
        res.on("end", function () {
            var registry;
            try {
                registry = JSON.parse(content);
            }
            catch (err) {
                callback(new Error("Could not parse registry as JSON"), null);
                return;
            }
            if (registry.version !== currentRegistryVersion) {
                callback(new Error("The registry format has changed. Please update Superpowers."), null);
            }
            else {
                var packageData = fs.readFileSync(__dirname + "/../../package.json", { encoding: "utf8" });
                var localCoreVersion = JSON.parse(packageData).version;
                registry.core.localVersion = localCoreVersion;
                var isLocalCoreDev = true;
                try {
                    if (!fs.lstatSync(__dirname + "/../../.git").isDirectory())
                        isLocalCoreDev = false;
                }
                catch (err) {
                    isLocalCoreDev = false;
                }
                registry.core.isLocalDev = isLocalCoreDev;
                async.each(Object.keys(registry.systems), function (systemId, cb) {
                    var registrySystem = registry.systems[systemId];
                    var localSystem = exports.systemsById[systemId];
                    if (localSystem != null) {
                        registrySystem.localVersion = localSystem.version;
                        registrySystem.isLocalDev = localSystem.isDev;
                    }
                    else {
                        registrySystem.isLocalDev = false;
                    }
                    async.each(Object.keys(registrySystem.plugins), function (authorName, cb) {
                        async.each(Object.keys(registrySystem.plugins[authorName]), function (pluginName, cb) {
                            var registryPlugin = registrySystem.plugins[authorName][pluginName];
                            var localPlugin = localSystem != null && localSystem.plugins[authorName] != null ? localSystem.plugins[authorName][pluginName] : null;
                            if (localPlugin != null) {
                                registryPlugin.localVersion = localPlugin.version;
                                registryPlugin.isLocalDev = localPlugin.isDev;
                            }
                            else {
                                registryPlugin.isLocalDev = false;
                            }
                            cb();
                        }, cb);
                    }, cb);
                }, function (err) { callback(err, registry); });
            }
        });
    });
    request.on("error", function (err) {
        callback(err, null);
    });
}
exports.getRegistry = getRegistry;
function downloadRelease(downloadURL, downloadPath, callback) {
    console.log("0%");
    https.get({
        hostname: "github.com",
        path: downloadURL,
        headers: { "user-agent": "Superpowers" }
    }, function (res) {
        if (res.statusCode !== 200) {
            callback("Unexpected status code: " + res.statusCode);
            return;
        }
        var progress = 0;
        var progressMax = parseInt(res.headers["content-length"], 10) * 2;
        var buffers = [];
        res.on("data", function (data) { buffers.push(data); progress += data.length; onProgress(progress / progressMax); });
        res.on("end", function () {
            var zipBuffer = Buffer.concat(buffers);
            yauzl.fromBuffer(zipBuffer, { lazyEntries: true }, function (err, zipFile) {
                if (err != null)
                    throw err;
                progress = zipFile.entryCount;
                progressMax = zipFile.entryCount * 2;
                var rootFolderName = path.parse(downloadURL).name;
                zipFile.readEntry();
                zipFile.on("entry", function (entry) {
                    if (entry.fileName.indexOf(rootFolderName) !== 0)
                        throw new Error("Found file outside of root folder: " + entry.fileName + " (" + rootFolderName + ")");
                    var filename = path.join(downloadPath, entry.fileName.replace(rootFolderName, ""));
                    if (/\/$/.test(entry.fileName)) {
                        mkdirp(filename, function (err) {
                            if (err != null)
                                throw err;
                            progress++;
                            onProgress(progress / progressMax);
                            zipFile.readEntry();
                        });
                    }
                    else {
                        zipFile.openReadStream(entry, function (err, readStream) {
                            if (err)
                                throw err;
                            mkdirp(path.dirname(filename), function (err) {
                                if (err)
                                    throw err;
                                readStream.pipe(fs.createWriteStream(filename));
                                readStream.on("end", function () {
                                    progress++;
                                    onProgress(progress / progressMax);
                                    zipFile.readEntry();
                                });
                            });
                        });
                    }
                });
                zipFile.on("end", function () {
                    // NOTE: Necessary to allow manipulating files right after download
                    setTimeout(callback, 100);
                });
            });
        });
    });
}
exports.downloadRelease = downloadRelease;
function onProgress(value) {
    value = Math.round(value * 100);
    readline.clearLine(process.stdout, 0);
    readline.cursorTo(process.stdout, 0, 1);
    console.log(value + "%");
    if (process != null && process.send != null)
        process.send({ type: "progress", value: value });
}
function emitError(message, details) {
    console.error(message);
    if (details != null)
        console.error(details);
    if (process != null && process.send != null)
        process.send({ type: "error", message: message });
    process.exit(1);
}
exports.emitError = emitError;
