"use strict";
var path = require("path");
var fs = require("fs");
var express = require("express");
var async = require("async");
var getLocalizedFilename_1 = require("./getLocalizedFilename");
function shouldIgnoreFolder(pluginName) { return pluginName.indexOf(".") !== -1 || pluginName === "node_modules"; }
function default_1(mainApp, buildApp, callback) {
    async.eachSeries(fs.readdirSync(SupCore.systemsPath), function (systemFolderName, cb) {
        if (systemFolderName.indexOf(".") !== -1) {
            cb();
            return;
        }
        var systemPath = path.join(SupCore.systemsPath, systemFolderName);
        if (!fs.statSync(systemPath).isDirectory()) {
            cb();
            return;
        }
        var systemId = JSON.parse(fs.readFileSync(path.join(SupCore.systemsPath, systemFolderName, "package.json"), { encoding: "utf8" })).superpowers.systemId;
        SupCore.system = SupCore.systems[systemId] = new SupCore.System(systemId, systemFolderName);
        // Expose public stuff
        try {
            fs.mkdirSync(systemPath + "/public");
        }
        catch (err) { }
        mainApp.use("/systems/" + systemId, express.static(systemPath + "/public"));
        buildApp.use("/systems/" + systemId, express.static(systemPath + "/public"));
        // Write templates list
        var templatesList = [];
        var templatesFolder = systemPath + "/public/templates";
        if (fs.existsSync(templatesFolder))
            templatesList = fs.readdirSync(templatesFolder);
        fs.writeFileSync(systemPath + "/public/templates.json", JSON.stringify(templatesList, null, 2));
        // Load server-side system module
        var systemServerModulePath = systemPath + "/server";
        if (fs.existsSync(systemServerModulePath)) {
            /* tslint:disable */
            require(systemServerModulePath);
        }
        // Load plugins
        var pluginsInfo = SupCore.system.pluginsInfo = loadPlugins(systemId, systemPath + "/plugins", mainApp, buildApp);
        fs.writeFileSync(systemPath + "/public/plugins.json", JSON.stringify(pluginsInfo, null, 2));
        cb();
    }, function () {
        var systemsInfo = { list: Object.keys(SupCore.systems) };
        fs.writeFileSync(__dirname + "/../public/systems.json", JSON.stringify(systemsInfo, null, 2));
        SupCore.system = null;
        callback();
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
function loadPlugins(systemId, pluginsPath, mainApp, buildApp) {
    var pluginNamesByAuthor = {};
    var pluginsInfo = { list: [], paths: { editors: {}, tools: {} } };
    var pluginsFolder;
    try {
        pluginsFolder = fs.readdirSync(pluginsPath);
    }
    catch (err) { }
    if (pluginsFolder == null)
        return pluginsInfo;
    for (var _i = 0, pluginsFolder_1 = pluginsFolder; _i < pluginsFolder_1.length; _i++) {
        var pluginAuthor = pluginsFolder_1[_i];
        var pluginAuthorPath = pluginsPath + "/" + pluginAuthor;
        if (shouldIgnoreFolder(pluginAuthor))
            continue;
        pluginNamesByAuthor[pluginAuthor] = [];
        for (var _a = 0, _b = fs.readdirSync(pluginAuthorPath); _a < _b.length; _a++) {
            var pluginName = _b[_a];
            if (shouldIgnoreFolder(pluginName))
                continue;
            var pluginPath = pluginsPath + "/" + pluginAuthor + "/" + pluginName;
            if (!fs.statSync(pluginPath).isDirectory())
                continue;
            pluginNamesByAuthor[pluginAuthor].push(pluginName);
        }
    }
    Object.keys(pluginNamesByAuthor).forEach(function (pluginAuthor) {
        var pluginNames = pluginNamesByAuthor[pluginAuthor];
        var pluginAuthorPath = pluginsPath + "/" + pluginAuthor;
        pluginNames.forEach(function (pluginName) {
            var pluginPath = pluginAuthorPath + "/" + pluginName;
            // Load data module
            var dataModulePath = pluginPath + "/data/index.js";
            if (fs.existsSync(dataModulePath)) {
                /* tslint:disable */
                require(dataModulePath);
            }
            // Collect plugin info
            pluginsInfo.list.push(pluginAuthor + "/" + pluginName);
            if (fs.existsSync(pluginPath + "/public/editors")) {
                var editors = fs.readdirSync(pluginPath + "/public/editors");
                editors.forEach(function (editorName) {
                    // Ignore folders with no index.html
                    try {
                        var stats = fs.lstatSync(pluginPath + "/public/editors/" + editorName + "/index.html");
                        if (!stats.isFile())
                            return;
                    }
                    catch (err) {
                        if (err.code === "ENOENT")
                            return;
                        throw err;
                    }
                    if (SupCore.system.data.assetClasses[editorName] != null) {
                        pluginsInfo.paths.editors[editorName] = pluginAuthor + "/" + pluginName;
                    }
                    else {
                        pluginsInfo.paths.tools[editorName] = pluginAuthor + "/" + pluginName;
                    }
                    mainApp.get("/systems/" + systemId + "/plugins/" + pluginAuthor + "/" + pluginName + "/editors/" + editorName, function (req, res) {
                        var languageCode = req.cookies["supLanguage"];
                        var editorPath = path.join(pluginPath, "public/editors", editorName);
                        var localizedIndexFilename = getLocalizedFilename_1.default("index.html", languageCode);
                        fs.exists(path.join(editorPath, localizedIndexFilename), function (exists) {
                            if (exists)
                                res.sendFile(path.join(editorPath, localizedIndexFilename));
                            else
                                res.sendFile(path.join(editorPath, "index.html"));
                        });
                    });
                });
            }
            // Expose public stuff
            mainApp.get("/systems/" + systemId + "/plugins/" + pluginAuthor + "/" + pluginName + "/locales/*.json", function (req, res) {
                var localeFile = req.path.split("/locales/")[1];
                var localePath = path.join(pluginPath, "public/locales", localeFile);
                fs.exists(localePath, function (exists) {
                    if (exists)
                        res.sendFile(localePath);
                    else
                        res.send("{}");
                });
            });
            for (var _i = 0, _a = [mainApp, buildApp]; _i < _a.length; _i++) {
                var app = _a[_i];
                app.get("/systems/" + systemId + "/plugins/" + pluginAuthor + "/" + pluginName + "/bundles/*.js", function (req, res) {
                    var bundleFile = req.path.split("/bundles/")[1];
                    var bundlePath = path.join(pluginPath, "public/bundles", bundleFile);
                    fs.exists(bundlePath, function (exists) {
                        if (exists)
                            res.sendFile(bundlePath);
                        else
                            res.send("");
                    });
                });
                app.use("/systems/" + systemId + "/plugins/" + pluginAuthor + "/" + pluginName, express.static(pluginPath + "/public"));
            }
        });
    });
    return pluginsInfo;
}
