"use strict";
var fs = require("fs");
var path = require("path");
var async = require("async");
var mkdirp = require("mkdirp");
function default_1(server, callback) {
    var oldVersion = server.data.manifest.migratedFromFormatVersion;
    if (oldVersion == null) {
        callback(null);
        return;
    }
    SupCore.log("Migrating \"" + server.data.manifest.pub.name + "\" project (from format version " + oldVersion + " to " + SupCore.Data.ProjectManifest.currentFormatVersion + ")...");
    async.series([
        function (cb) { if (oldVersion < 1)
            migrateTo1(server, cb);
        else
            cb(); },
        function (cb) { if (oldVersion < 3)
            migrateTo3(server, cb);
        else
            cb(); }
    ], callback);
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
function migrateTo1(server, callback) {
    var assetsPath = path.join(server.projectPath, "assets");
    async.series([
        // Delete ArcadePhysics2DSettingsResource, removed in Superpowers v0.13
        // FIXME: This should be done by an init function in the plugin, probably.
        function (cb) { fs.unlink(path.join(server.projectPath, "resources/arcadePhysics2DSettings/resource.json"), function (err) { cb(); }); },
        function (cb) { fs.rmdir(path.join(server.projectPath, "resources/arcadePhysics2DSettings"), function (err) { cb(); }); },
        // Move trashed assets to "trashedAssets" folder
        function (cb) {
            fs.readdir(assetsPath, function (err, assetFolders) {
                if (err != null)
                    throw err;
                var assetFolderRegex = /^[0-9]+-.+$/;
                var trashedAssetFolders = [];
                for (var _i = 0, assetFolders_1 = assetFolders; _i < assetFolders_1.length; _i++) {
                    var assetFolder = assetFolders_1[_i];
                    if (!assetFolderRegex.test(assetFolder))
                        continue;
                    var assetId = assetFolder.substring(0, assetFolder.indexOf("-"));
                    if (server.data.entries.byId[assetId] == null)
                        trashedAssetFolders.push(assetFolder);
                }
                async.each(trashedAssetFolders, server.moveAssetFolderToTrash.bind(server), cb);
            });
        },
        // Delete internals.json and members.json
        function (cb) { fs.unlink(path.join(server.projectPath, "internals.json"), function (err) { cb(); }); },
        function (cb) { fs.unlink(path.join(server.projectPath, "members.json"), function (err) { cb(); }); }
    ], callback);
}
function migrateTo3(server, callback) {
    var assetsPath = path.join(server.projectPath, "assets");
    async.eachSeries(Object.keys(server.data.entries.byId), function (nodeId, cb) {
        var node = server.data.entries.byId[nodeId];
        var storagePath = server.data.entries.getStoragePathFromId(nodeId);
        if (node.type == null)
            cb();
        else {
            var index = storagePath.lastIndexOf("/");
            var parentStoragePath = storagePath;
            var oldStoragePath_1 = path.join(assetsPath, nodeId + "-" + server.data.entries.getPathFromId(nodeId).replace(new RegExp("/", "g"), "__"));
            if (index !== -1) {
                parentStoragePath = storagePath.slice(0, index);
                mkdirp(path.join(assetsPath, parentStoragePath), function (err) {
                    if (err != null && err.code !== "EEXIST") {
                        cb(err);
                        return;
                    }
                    fs.rename(oldStoragePath_1, path.join(assetsPath, storagePath), cb);
                });
            }
            else {
                fs.rename(oldStoragePath_1, path.join(assetsPath, storagePath), cb);
            }
        }
    }, callback);
}
