"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var THREE = require("three");
var ActorComponent_1 = require("../ActorComponent");
var tmpVector3 = new THREE.Vector3();
var Camera2DControls = (function (_super) {
    __extends(Camera2DControls, _super);
    function Camera2DControls(actor, camera, options, zoomCallback) {
        _super.call(this, actor, "Camera2DControls");
        this.multiplier = 1;
        this.actor = actor;
        this.camera = camera;
        this.options = options;
        this.zoomCallback = zoomCallback;
    }
    Camera2DControls.prototype.setIsLayerActive = function (active) { };
    Camera2DControls.prototype.setMultiplier = function (newMultiplier) {
        var newOrthographicScale = this.camera.orthographicScale * this.multiplier;
        this.multiplier = newMultiplier / 10;
        this.camera.actor.getLocalPosition(tmpVector3);
        var screenPosition = this.getScreenPosition(tmpVector3.x, tmpVector3.y);
        this.changeOrthographicScale(newOrthographicScale, screenPosition[0], screenPosition[1]);
    };
    Camera2DControls.prototype.changeOrthographicScale = function (newOrthographicScale, x, y) {
        var startPosition = this.getScenePosition(x, y);
        this.camera.setOrthographicScale(newOrthographicScale / this.multiplier);
        var endPosition = this.getScenePosition(x, y);
        this.camera.actor.moveLocal(new THREE.Vector3(startPosition[0] - endPosition[0], endPosition[1] - startPosition[1], 0));
        if (this.zoomCallback != null)
            this.zoomCallback();
    };
    Camera2DControls.prototype.update = function () {
        var input = this.actor.gameInstance.input;
        var keys = window.KeyEvent;
        // Move
        if (input.mouseButtons[1].isDown ||
            (input.mouseButtons[0].isDown && input.keyboardButtons[keys.DOM_VK_ALT].isDown)) {
            var mouseDelta = input.mouseDelta;
            mouseDelta.x /= this.actor.gameInstance.threeRenderer.domElement.width;
            mouseDelta.x *= this.camera.orthographicScale * this.camera.cachedRatio;
            mouseDelta.y /= this.actor.gameInstance.threeRenderer.domElement.height;
            mouseDelta.y *= this.camera.orthographicScale;
            if (mouseDelta.x !== 0 || mouseDelta.y !== 0)
                this.camera.actor.moveLocal(new THREE.Vector3(-mouseDelta.x, mouseDelta.y, 0));
        }
        else {
            var mousePosition = input.mousePosition;
            var newOrthographicScale = void 0;
            if (input.mouseButtons[5].isDown || input.keyboardButtons[keys.DOM_VK_ADD].wasJustPressed) {
                newOrthographicScale = Math.max(this.options.zoomMin, this.camera.orthographicScale * this.multiplier / this.options.zoomSpeed);
                if (input.keyboardButtons[keys.DOM_VK_ADD].wasJustPressed) {
                    mousePosition.x = this.actor.gameInstance.threeRenderer.domElement.width / 2;
                    mousePosition.y = this.actor.gameInstance.threeRenderer.domElement.height / 2;
                }
            }
            else if (input.mouseButtons[6].isDown || (input.keyboardButtons[keys.DOM_VK_SUBTRACT].wasJustPressed)) {
                newOrthographicScale = Math.min(this.options.zoomMax, this.camera.orthographicScale * this.multiplier * this.options.zoomSpeed);
                if (input.keyboardButtons[keys.DOM_VK_SUBTRACT].wasJustPressed) {
                    mousePosition.x = this.actor.gameInstance.threeRenderer.domElement.width / 2;
                    mousePosition.y = this.actor.gameInstance.threeRenderer.domElement.height / 2;
                }
            }
            if (newOrthographicScale != null && newOrthographicScale !== this.camera.orthographicScale) {
                this.changeOrthographicScale(newOrthographicScale, mousePosition.x, mousePosition.y);
            }
        }
    };
    Camera2DControls.prototype.getScreenPosition = function (x, y) {
        x /= this.camera.orthographicScale / 2 * this.camera.cachedRatio;
        x = (1 - x) / 2;
        x *= this.actor.gameInstance.threeRenderer.domElement.width;
        y /= this.camera.orthographicScale / 2;
        y = (y + 1) / 2;
        y *= this.actor.gameInstance.threeRenderer.domElement.height;
        return [x, y];
    };
    Camera2DControls.prototype.getScenePosition = function (x, y) {
        this.camera.actor.getLocalPosition(tmpVector3);
        x /= this.actor.gameInstance.threeRenderer.domElement.width;
        x = x * 2 - 1;
        x *= this.camera.orthographicScale / 2 * this.camera.cachedRatio;
        x += tmpVector3.x;
        y /= this.actor.gameInstance.threeRenderer.domElement.height;
        y = y * 2 - 1;
        y *= this.camera.orthographicScale / 2;
        y -= tmpVector3.y;
        return [x, y];
    };
    return Camera2DControls;
}(ActorComponent_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Camera2DControls;
