/// <reference path="../../../../typings/tsd.d.ts" />
/// <reference path="../SupRuntime.d.ts" />
/// <reference path="../../../../SupCore/SupCore.d.ts" />
/// <reference path="../../../../SupClient/typings/SupApp.d.ts" />
"use strict";
var async = require("async");
var querystring = require("querystring");
var fetch_1 = require("../../../../SupClient/src/fetch");
var Player_1 = require("./Player");
exports.Player = Player_1.default;
// Any export here will be exposed as SupRuntime.* by browserify
// (see ../gulpfile.js)
exports.plugins = {};
exports.resourcePlugins = {};
function registerPlugin(name, plugin) {
    if (exports.plugins[name] != null) {
        console.error("SupRuntime.registerPlugin: Tried to register two or more plugins named \"" + name + "\"");
        return;
    }
    exports.plugins[name] = plugin;
}
exports.registerPlugin = registerPlugin;
function registerResource(name, plugin) {
    if (exports.resourcePlugins[name] != null) {
        console.error("SupRuntime.registerResource: Tried to register two or more resources named \"" + name + "\"");
        return;
    }
    exports.resourcePlugins[name] = plugin;
}
exports.registerResource = registerResource;
SupCore.system = new SupCore.System("", "");
// Setup SupApp
if (global.SupApp == null) {
    global.SupApp = null;
    try {
        global.SupApp = (top.SupApp != null) ? top.SupApp : null;
    }
    catch (err) {
    }
}
// In app, open links in a browser window
var playerWindow;
var electron;
if (SupApp != null) {
    playerWindow = SupApp.getCurrentWindow();
}
else {
    var nodeRequire = require;
    try {
        electron = nodeRequire("electron");
    }
    catch (e) { }
    if (electron != null)
        playerWindow = electron.remote.getCurrentWindow();
}
if (playerWindow != null) {
    document.body.addEventListener("click", function (event) {
        if (event.target.tagName !== "A")
            return;
        event.preventDefault();
        var url = event.target.href;
        if (SupApp != null)
            SupApp.openLink(url);
        else
            electron.shell.openExternal(url);
    });
    SupApp.onMessage("force-quit", function () {
        playerWindow.close();
    });
}
var qs = querystring.parse(window.location.search.slice(1));
document.body.addEventListener("keydown", function (event) {
    if (event.keyCode === window["KeyEvent"].DOM_VK_F12) {
        if (qs.project != null && playerWindow != null)
            playerWindow.webContents.toggleDevTools();
    }
});
var progressBar = document.querySelector("progress");
var loadingElt = document.getElementById("loading");
var canvas = document.querySelector("canvas");
// Prevent keypress events from leaking out to a parent window
// They might trigger scrolling for instance
canvas.addEventListener("keypress", function (event) { event.preventDefault(); });
if (qs.debug != null && playerWindow != null)
    playerWindow.webContents.openDevTools();
var player;
var onLoadProgress = function (value, max) {
    progressBar.value = value;
    progressBar.max = max;
};
var onLoaded = function (err) {
    if (err != null) {
        console.error(err);
        var aElt = loadingElt.querySelector("a");
        aElt.parentElement.removeChild(aElt);
        var errorElt = document.createElement("div");
        errorElt.className = "error";
        errorElt.textContent = err.message;
        loadingElt.appendChild(errorElt);
        return;
    }
    setTimeout(function () {
        loadingElt.classList.remove("start");
        loadingElt.classList.add("end");
        setTimeout(function () {
            loadingElt.parentElement.removeChild(loadingElt);
            player.run();
            return;
        }, (qs.project == null) ? 500 : 0);
    }, (qs.project == null) ? 500 : 0);
};
// Load plugins
var pluginBundleNames = ["components", "runtime", "typescriptAPI"];
fetch_1.default("plugins.json", "json", function (err, pluginsInfo) {
    if (err != null) {
        console.log(err);
        onLoaded(new Error("Could not load plugins list."));
        return;
    }
    async.each(pluginsInfo.list, function (pluginName, cb) {
        async.each(pluginBundleNames, function (bundle, cb) {
            var script = document.createElement("script");
            script.src = "plugins/" + pluginName + "/bundles/" + bundle + ".js";
            script.addEventListener("load", function () { return cb(null); });
            script.addEventListener("error", function (err) { return cb(null); });
            document.body.appendChild(script);
        }, cb);
    }, function (err) {
        if (err != null)
            console.log(err);
        // Load game
        var buildPath = (qs.project != null) ? "/builds/" + qs.project + "/" + qs.build + "/" : "./";
        player = new Player_1.default(canvas, buildPath, { debug: qs.debug != null });
        player.load(onLoadProgress, onLoaded);
    });
});
loadingElt.classList.add("start");
