"use strict";
var fs = require("fs");
var path = require("path");
var textEditorUserSettings = require("../data/textEditorUserSettings");
var modes = fs.readdirSync(path.join(__dirname, "../node_modules/codemirror/theme"));
var TextEditorSettingsEditor = (function () {
    function TextEditorSettingsEditor(container, projectClient) {
        var _this = this;
        this.onResourceReceived = function (resourceId, resource) {
            _this.resource = resource;
            _this.tabSizeField.value = resource.pub.tabSize.toString();
            _this.softTabField.checked = resource.pub.softTab;
        };
        this.onResourceEdited = function (resourceId, command, propertyName) {
            switch (propertyName) {
                case "tabSize":
                    _this.tabSizeField.value = _this.resource.pub.tabSize.toString();
                    break;
                case "softTab":
                    _this.softTabField.checked = _this.resource.pub.softTab;
                    break;
            }
        };
        var tbody = SupClient.table.createTable(container).tbody;
        // Project settings
        var tabSizeRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TextEditor.tabSize"));
        this.tabSizeField = SupClient.table.appendNumberField(tabSizeRow.valueCell, "", { min: 1 });
        this.tabSizeField.addEventListener("change", function (event) {
            projectClient.editResource("textEditorSettings", "setProperty", "tabSize", parseInt(event.target.value, 10));
        });
        var softTabRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TextEditor.useSoftTab"));
        this.softTabField = SupClient.table.appendBooleanField(softTabRow.valueCell, true);
        this.softTabField.addEventListener("change", function (event) {
            projectClient.editResource("textEditorSettings", "setProperty", "softTab", event.target.checked);
        });
        projectClient.subResource("textEditorSettings", this);
        // User settings
        var keyMapRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TextEditor.keyMap"));
        this.keyMapField = SupClient.table.appendSelectBox(keyMapRow.valueCell, { "sublime": "Sublime", "emacs": "Emacs", "vim": "Vim" }, textEditorUserSettings.pub.keyMap);
        this.keyMapField.addEventListener("change", function (event) {
            textEditorUserSettings.edit("keyMap", event.target.value);
        });
        textEditorUserSettings.emitter.addListener("keyMap", function () {
            _this.keyMapField.value = textEditorUserSettings.pub.keyMap;
        });
        var themeRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TextEditor.theme"));
        var themeValues = { "default": "default" };
        for (var _i = 0, modes_1 = modes; _i < modes_1.length; _i++) {
            var mode = modes_1[_i];
            var modeNoExtension = mode.slice(0, mode.length - 4);
            themeValues[modeNoExtension] = modeNoExtension;
        }
        this.themeField = SupClient.table.appendSelectBox(themeRow.valueCell, themeValues, textEditorUserSettings.pub.theme);
        this.themeField.addEventListener("change", function (event) {
            textEditorUserSettings.edit("theme", event.target.value);
        });
        textEditorUserSettings.emitter.addListener("theme", function () {
            _this.themeField.value = textEditorUserSettings.pub.theme;
        });
    }
    return TextEditorSettingsEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TextEditorSettingsEditor;
