"use strict";
var SelectionBoxRenderer = (function () {
    function SelectionBoxRenderer(root) {
        this.mesh = new THREE.Mesh(new THREE.BoxGeometry(1, 1, 1), new THREE.MeshBasicMaterial({ color: 0x00ffff, side: THREE.BackSide }));
        root.add(this.mesh);
    }
    SelectionBoxRenderer.prototype.setTarget = function (target) {
        this.target = target;
        this.mesh.visible = true;
        this.move();
        this.resize();
        return this;
    };
    SelectionBoxRenderer.prototype.move = function () {
        this.mesh.position.copy(this.target.getWorldPosition());
        this.mesh.quaternion.copy(this.target.getWorldQuaternion());
        this.mesh.updateMatrixWorld(false);
        return this;
    };
    SelectionBoxRenderer.prototype.resize = function () {
        var vec = new THREE.Vector3();
        var box = new THREE.Box3();
        var inverseTargetMatrixWorld = new THREE.Matrix4().compose(this.target.getWorldPosition(), this.target.getWorldQuaternion(), { x: 1, y: 1, z: 1 });
        inverseTargetMatrixWorld.getInverse(inverseTargetMatrixWorld);
        this.target.traverse(function (node) {
            var geometry = node.geometry;
            if (geometry != null) {
                node.updateMatrixWorld(false);
                if (geometry instanceof THREE.Geometry) {
                    var vertices = geometry.vertices;
                    for (var i = 0, il = vertices.length; i < il; i++) {
                        vec.copy(vertices[i]).applyMatrix4(node.matrixWorld).applyMatrix4(inverseTargetMatrixWorld);
                        box.expandByPoint(vec);
                    }
                }
                else if (geometry instanceof THREE.BufferGeometry && geometry.attributes["position"] != null) {
                    var positions = geometry.attributes["position"].array;
                    for (var i = 0, il = positions.length; i < il; i += 3) {
                        vec.set(positions[i], positions[i + 1], positions[i + 2]);
                        vec.applyMatrix4(node.matrixWorld).applyMatrix4(inverseTargetMatrixWorld);
                        box.expandByPoint(vec);
                    }
                }
            }
        });
        var size = box.size();
        var thickness = 0.1;
        this.mesh.scale.copy(size).add(new THREE.Vector3(thickness, thickness, thickness));
        this.mesh.updateMatrixWorld(false);
        return this;
    };
    SelectionBoxRenderer.prototype.hide = function () {
        this.mesh.visible = false;
        return this;
    };
    return SelectionBoxRenderer;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = SelectionBoxRenderer;
