/**
 * https://github.com/mrdoob/three.js/blob/master/examples/js/controls/TransformControls.js
 * Rewritten in TypeScript and modified by bilou84
 */
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var TransformGizmos_1 = require("./TransformGizmos");
var ray = new THREE.Raycaster();
var pointerVector = new THREE.Vector2();
var point = new THREE.Vector3();
var offset = new THREE.Vector3();
var rotation = new THREE.Vector3();
var offsetRotation = new THREE.Vector3();
var lookAtMatrix = new THREE.Matrix4();
var eye = new THREE.Vector3();
var tempMatrix = new THREE.Matrix4();
var tempVector = new THREE.Vector3();
var tempQuaternion = new THREE.Quaternion();
var unitX = new THREE.Vector3(1, 0, 0);
var unitY = new THREE.Vector3(0, 1, 0);
var unitZ = new THREE.Vector3(0, 0, 1);
var quaternionXYZ = new THREE.Quaternion();
var quaternionX = new THREE.Quaternion();
var quaternionY = new THREE.Quaternion();
var quaternionZ = new THREE.Quaternion();
var quaternionE = new THREE.Quaternion();
var oldPosition = new THREE.Vector3();
var oldScale = new THREE.Vector3();
var oldRotationMatrix = new THREE.Matrix4();
var parentRotationMatrix = new THREE.Matrix4();
var parentScale = new THREE.Vector3();
var worldPosition = new THREE.Vector3();
var worldRotation = new THREE.Euler();
var worldRotationMatrix = new THREE.Matrix4();
var camPosition = new THREE.Vector3();
var camRotation = new THREE.Euler();
var TransformControls = (function (_super) {
    __extends(TransformControls, _super);
    function TransformControls(scene, camera, domElement) {
        var _this = this;
        _super.call(this);
        this.camera = camera;
        this.domElement = domElement;
        this.visible = false;
        this.root = new THREE.Object3D();
        this.externVisible = true;
        this.size = 1;
        this.mode = "translate";
        this.space = "local";
        this.dragging = false;
        this.disabled = true;
        this.gizmo = {
            "translate": new TransformGizmos_1.TransformGizmoTranslate(),
            "rotate": new TransformGizmos_1.TransformGizmoRotate(),
            "scale": new TransformGizmos_1.TransformGizmoScale(),
            "resize": new TransformGizmos_1.TransformGizmoResize()
        };
        this.changeEvent = { type: "change", target: null };
        this.mouseDownEvent = { type: "mouseDown", target: null };
        this.mouseUpEvent = { type: "mouseUp", mode: this.mode, target: null };
        this.objectChangeEvent = { type: "objectChange", target: null };
        this.onPointerDown = function (event) {
            if (_this.target == null || _this.dragging || (event.button != null && event.button !== 0))
                return;
            var pointer = event.changedTouches ? event.changedTouches[0] : event;
            if (pointer.button === 0 || pointer.button == null) {
                var intersect = _this.intersectObjects(pointer, _this.gizmo[_this.mode].pickersRoot.children);
                if (intersect != null) {
                    event.preventDefault();
                    event.stopPropagation();
                    _this.dispatchEvent(_this.mouseDownEvent);
                    _this.axis = intersect.object.name;
                    _this.update();
                    eye.copy(camPosition).sub(worldPosition).normalize();
                    _this.gizmo[_this.mode].setActivePlane(_this.axis, eye);
                    var planeIntersect = _this.intersectObjects(pointer, [_this.gizmo[_this.mode].activePlane]);
                    if (planeIntersect != null) {
                        oldPosition.copy(_this.root.position);
                        oldScale.copy(_this.root.scale);
                        oldRotationMatrix.extractRotation(_this.root.matrix);
                        worldRotationMatrix.extractRotation(_this.root.matrixWorld);
                        parentRotationMatrix.extractRotation(_this.root.parent.matrixWorld);
                        parentScale.setFromMatrixScale(tempMatrix.getInverse(_this.root.parent.matrixWorld));
                        offset.copy(planeIntersect.point);
                    }
                }
            }
            _this.dragging = true;
        };
        this.onPointerHover = function (event) {
            if (_this.target == null || _this.dragging || (event.button != null && event.button !== 0))
                return;
            var pointer = event.changedTouches ? event.changedTouches[0] : event;
            var newAxis;
            var intersect = _this.intersectObjects(pointer, _this.gizmo[_this.mode].pickersRoot.children);
            if (intersect != null) {
                newAxis = intersect.object.name;
                event.preventDefault();
            }
            if (_this.axis !== newAxis) {
                _this.axis = newAxis;
                _this.update();
                _this.dispatchEvent(_this.changeEvent);
            }
        };
        this.onPointerMove = function (event) {
            if (_this.target == null || _this.axis == null || !_this.dragging || (event.button != null && event.button !== 0))
                return;
            var pointer = event.changedTouches ? event.changedTouches[0] : event;
            var planeIntersect = _this.intersectObjects(pointer, [_this.gizmo[_this.mode].activePlane]);
            if (planeIntersect == null)
                return;
            event.preventDefault();
            event.stopPropagation();
            point.copy(planeIntersect.point);
            switch (_this.mode) {
                case "translate":
                    {
                        point.sub(offset);
                        point.multiply(parentScale);
                        if (_this.space === "local") {
                            point.applyMatrix4(tempMatrix.getInverse(worldRotationMatrix));
                            if (_this.axis.search("X") === -1)
                                point.x = 0;
                            if (_this.axis.search("Y") === -1)
                                point.y = 0;
                            if (_this.axis.search("Z") === -1)
                                point.z = 0;
                            point.applyMatrix4(oldRotationMatrix);
                            _this.root.position.copy(oldPosition);
                            _this.root.position.add(point);
                        }
                        if (_this.space === "world" || _this.axis.search("XYZ") !== -1) {
                            if (_this.axis.search("X") === -1)
                                point.x = 0;
                            if (_this.axis.search("Y") === -1)
                                point.y = 0;
                            if (_this.axis.search("Z") === -1)
                                point.z = 0;
                            point.applyMatrix4(tempMatrix.getInverse(parentRotationMatrix));
                            _this.root.position.copy(oldPosition);
                            _this.root.position.add(point);
                        }
                        if (_this.translationSnap !== null) {
                            if (_this.space === "local")
                                _this.root.position.sub(worldPosition).applyMatrix4(tempMatrix.getInverse(worldRotationMatrix));
                            if (_this.axis.search("X") !== -1)
                                _this.root.position.x = Math.round(_this.root.position.x / _this.translationSnap) * _this.translationSnap;
                            if (_this.axis.search("Y") !== -1)
                                _this.root.position.y = Math.round(_this.root.position.y / _this.translationSnap) * _this.translationSnap;
                            if (_this.axis.search("Z") !== -1)
                                _this.root.position.z = Math.round(_this.root.position.z / _this.translationSnap) * _this.translationSnap;
                            if (_this.space === "local")
                                _this.root.position.applyMatrix4(worldRotationMatrix).add(worldPosition);
                        }
                    }
                    break;
                case "scale":
                    {
                        point.sub(offset);
                        point.multiply(parentScale);
                        if (_this.axis === "XYZ") {
                            var scale = 1 + ((point.y) / Math.max(oldScale.x, oldScale.y, oldScale.z));
                            _this.root.scale.x = oldScale.x * scale;
                            _this.root.scale.y = oldScale.y * scale;
                            _this.root.scale.z = oldScale.z * scale;
                        }
                        else {
                            point.applyMatrix4(tempMatrix.getInverse(worldRotationMatrix));
                            if (_this.axis === "X")
                                _this.root.scale.x = oldScale.x * (1 + point.x / oldScale.x);
                            if (_this.axis === "Y")
                                _this.root.scale.y = oldScale.y * (1 + point.y / oldScale.y);
                            if (_this.axis === "Z")
                                _this.root.scale.z = oldScale.z * (1 + point.z / oldScale.z);
                        }
                    }
                    break;
                case "resize":
                    {
                        point.sub(offset);
                        point.multiply(parentScale);
                        var multiplier = 16;
                        if (_this.axis === "XYZ") {
                            var scale = 1 + ((point.y) / Math.max(oldScale.x, oldScale.y, oldScale.z) * multiplier);
                            _this.root.scale.x = oldScale.x * scale;
                            _this.root.scale.y = oldScale.y * scale;
                            _this.root.scale.z = oldScale.z * scale;
                        }
                        else {
                            point.applyMatrix4(tempMatrix.getInverse(worldRotationMatrix));
                            if (_this.axis === "X")
                                _this.root.scale.x = oldScale.x * (1 + point.x / oldScale.x * multiplier);
                            if (_this.axis === "Y")
                                _this.root.scale.y = oldScale.y * (1 + point.y / oldScale.y * multiplier);
                            if (_this.axis === "Z")
                                _this.root.scale.z = oldScale.z * (1 + point.z / oldScale.z * multiplier);
                        }
                        _this.root.scale.x = Math.round(Math.max(1, _this.root.scale.x));
                        _this.root.scale.y = Math.round(Math.max(1, _this.root.scale.y));
                        _this.root.scale.z = Math.round(Math.max(1, _this.root.scale.z));
                    }
                    break;
                case "rotate":
                    {
                        point.sub(worldPosition);
                        point.multiply(parentScale);
                        tempVector.copy(offset).sub(worldPosition);
                        tempVector.multiply(parentScale);
                        if (_this.axis === "E") {
                            point.applyMatrix4(tempMatrix.getInverse(lookAtMatrix));
                            tempVector.applyMatrix4(tempMatrix.getInverse(lookAtMatrix));
                            rotation.set(Math.atan2(point.z, point.y), Math.atan2(point.x, point.z), Math.atan2(point.y, point.x));
                            offsetRotation.set(Math.atan2(tempVector.z, tempVector.y), Math.atan2(tempVector.x, tempVector.z), Math.atan2(tempVector.y, tempVector.x));
                            tempQuaternion.setFromRotationMatrix(tempMatrix.getInverse(parentRotationMatrix));
                            quaternionE.setFromAxisAngle(eye, rotation.z - offsetRotation.z);
                            quaternionXYZ.setFromRotationMatrix(worldRotationMatrix);
                            tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionE);
                            tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionXYZ);
                            _this.root.quaternion.copy(tempQuaternion);
                        }
                        else if (_this.axis === "XYZE") {
                            quaternionE.setFromEuler(point.clone().cross(tempVector).normalize()); // rotation axis
                            tempQuaternion.setFromRotationMatrix(tempMatrix.getInverse(parentRotationMatrix));
                            quaternionX.setFromAxisAngle(quaternionE, -point.clone().angleTo(tempVector));
                            quaternionXYZ.setFromRotationMatrix(worldRotationMatrix);
                            tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionX);
                            tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionXYZ);
                            _this.root.quaternion.copy(tempQuaternion);
                        }
                        else if (_this.space === "local") {
                            point.applyMatrix4(tempMatrix.getInverse(worldRotationMatrix));
                            tempVector.applyMatrix4(tempMatrix.getInverse(worldRotationMatrix));
                            rotation.set(Math.atan2(point.z, point.y), Math.atan2(point.x, point.z), Math.atan2(point.y, point.x));
                            offsetRotation.set(Math.atan2(tempVector.z, tempVector.y), Math.atan2(tempVector.x, tempVector.z), Math.atan2(tempVector.y, tempVector.x));
                            quaternionXYZ.setFromRotationMatrix(oldRotationMatrix);
                            if (_this.rotationSnap !== null) {
                                quaternionX.setFromAxisAngle(unitX, Math.round((rotation.x - offsetRotation.x) / _this.rotationSnap) * _this.rotationSnap);
                                quaternionY.setFromAxisAngle(unitY, Math.round((rotation.y - offsetRotation.y) / _this.rotationSnap) * _this.rotationSnap);
                                quaternionZ.setFromAxisAngle(unitZ, Math.round((rotation.z - offsetRotation.z) / _this.rotationSnap) * _this.rotationSnap);
                            }
                            else {
                                quaternionX.setFromAxisAngle(unitX, rotation.x - offsetRotation.x);
                                quaternionY.setFromAxisAngle(unitY, rotation.y - offsetRotation.y);
                                quaternionZ.setFromAxisAngle(unitZ, rotation.z - offsetRotation.z);
                            }
                            if (_this.axis === "X")
                                quaternionXYZ.multiplyQuaternions(quaternionXYZ, quaternionX);
                            if (_this.axis === "Y")
                                quaternionXYZ.multiplyQuaternions(quaternionXYZ, quaternionY);
                            if (_this.axis === "Z")
                                quaternionXYZ.multiplyQuaternions(quaternionXYZ, quaternionZ);
                            _this.root.quaternion.copy(quaternionXYZ);
                        }
                        else if (_this.space === "world") {
                            rotation.set(Math.atan2(point.z, point.y), Math.atan2(point.x, point.z), Math.atan2(point.y, point.x));
                            offsetRotation.set(Math.atan2(tempVector.z, tempVector.y), Math.atan2(tempVector.x, tempVector.z), Math.atan2(tempVector.y, tempVector.x));
                            tempQuaternion.setFromRotationMatrix(tempMatrix.getInverse(parentRotationMatrix));
                            if (_this.rotationSnap !== null) {
                                quaternionX.setFromAxisAngle(unitX, Math.round((rotation.x - offsetRotation.x) / _this.rotationSnap) * _this.rotationSnap);
                                quaternionY.setFromAxisAngle(unitY, Math.round((rotation.y - offsetRotation.y) / _this.rotationSnap) * _this.rotationSnap);
                                quaternionZ.setFromAxisAngle(unitZ, Math.round((rotation.z - offsetRotation.z) / _this.rotationSnap) * _this.rotationSnap);
                            }
                            else {
                                quaternionX.setFromAxisAngle(unitX, rotation.x - offsetRotation.x);
                                quaternionY.setFromAxisAngle(unitY, rotation.y - offsetRotation.y);
                                quaternionZ.setFromAxisAngle(unitZ, rotation.z - offsetRotation.z);
                            }
                            quaternionXYZ.setFromRotationMatrix(worldRotationMatrix);
                            if (_this.axis === "X")
                                tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionX);
                            if (_this.axis === "Y")
                                tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionY);
                            if (_this.axis === "Z")
                                tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionZ);
                            tempQuaternion.multiplyQuaternions(tempQuaternion, quaternionXYZ);
                            _this.root.quaternion.copy(tempQuaternion);
                        }
                    }
                    break;
            }
            _this.update(false);
            _this.dispatchEvent(_this.changeEvent);
            _this.dispatchEvent(_this.objectChangeEvent);
        };
        this.onPointerUp = function (event) {
            if (event.button != null && event.button !== 0)
                return;
            if (_this.dragging && (_this.axis !== null)) {
                _this.mouseUpEvent.mode = _this.mode;
                _this.dispatchEvent(_this.mouseUpEvent);
            }
            _this.dragging = false;
            _this.onPointerHover(event);
        };
        scene.add(this);
        scene.add(this.root);
        for (var type in this.gizmo) {
            var gizmoObj = this.gizmo[type];
            gizmoObj.visible = (type === this.mode);
            this.add(gizmoObj);
        }
        this.enable();
    }
    TransformControls.prototype.dispose = function () {
        this.disable();
    };
    TransformControls.prototype.setVisible = function (visible) {
        this.externVisible = visible;
        this.visible = this.externVisible && this.target != null;
        return this;
    };
    TransformControls.prototype.attach = function (object) {
        this.target = object;
        this.visible = this.externVisible;
        this.update();
        return this;
    };
    ;
    TransformControls.prototype.detach = function () {
        this.target = null;
        this.visible = false;
        this.axis = null;
        return this;
    };
    ;
    TransformControls.prototype.getMode = function () { return this.mode; };
    ;
    TransformControls.prototype.setMode = function (mode) {
        this.mode = mode;
        for (var type in this.gizmo)
            this.gizmo[type].visible = (type === mode);
        if (this.target == null)
            return this;
        this.update();
        this.dispatchEvent(this.changeEvent);
        return this;
    };
    ;
    TransformControls.prototype.setSize = function (size) {
        this.size = size;
        if (this.target == null)
            return this;
        this.update();
        this.dispatchEvent(this.changeEvent);
        return this;
    };
    ;
    TransformControls.prototype.setSpace = function (space) {
        this.space = space;
        if (this.target == null)
            return this;
        this.update();
        this.dispatchEvent(this.changeEvent);
        return this;
    };
    ;
    TransformControls.prototype.enable = function () {
        if (!this.disabled)
            return this;
        this.domElement.addEventListener("mousedown", this.onPointerDown, false);
        this.domElement.addEventListener("touchstart", this.onPointerDown, false);
        this.domElement.addEventListener("mousemove", this.onPointerHover, false);
        this.domElement.addEventListener("touchmove", this.onPointerHover, false);
        this.domElement.addEventListener("mousemove", this.onPointerMove, false);
        this.domElement.addEventListener("touchmove", this.onPointerMove, false);
        this.domElement.addEventListener("mouseup", this.onPointerUp, false);
        this.domElement.addEventListener("mouseout", this.onPointerUp, false);
        this.domElement.addEventListener("touchend", this.onPointerUp, false);
        this.domElement.addEventListener("touchcancel", this.onPointerUp, false);
        this.domElement.addEventListener("touchleave", this.onPointerUp, false);
        this.dragging = false;
        this.disabled = false;
        for (var gizmoName in this.gizmo)
            this.gizmo[gizmoName].setDisabled(false);
        return this;
    };
    TransformControls.prototype.disable = function () {
        if (this.disabled)
            return this;
        this.domElement.removeEventListener("mousedown", this.onPointerDown);
        this.domElement.removeEventListener("touchstart", this.onPointerDown);
        this.domElement.removeEventListener("mousemove", this.onPointerHover);
        this.domElement.removeEventListener("touchmove", this.onPointerHover);
        this.domElement.removeEventListener("mousemove", this.onPointerMove);
        this.domElement.removeEventListener("touchmove", this.onPointerMove);
        this.domElement.removeEventListener("mouseup", this.onPointerUp);
        this.domElement.removeEventListener("mouseout", this.onPointerUp);
        this.domElement.removeEventListener("touchend", this.onPointerUp);
        this.domElement.removeEventListener("touchcancel", this.onPointerUp);
        this.domElement.removeEventListener("touchleave", this.onPointerUp);
        this.dragging = false;
        this.disabled = true;
        for (var gizmoName in this.gizmo)
            this.gizmo[gizmoName].setDisabled(true);
        return this;
    };
    TransformControls.prototype.update = function (copyTarget) {
        if (copyTarget === void 0) { copyTarget = true; }
        if (this.target == null)
            return;
        if (copyTarget) {
            this.root.position.copy(this.target.getWorldPosition());
            this.root.quaternion.copy(this.target.getWorldQuaternion());
            var geometry = this.target.geometry;
            var width = (geometry != null && geometry.parameters.width != null) ? geometry.parameters.width : 1;
            var height = (geometry != null && geometry.parameters.height != null) ? geometry.parameters.height : 1;
            var depth = (geometry != null && geometry.parameters.depth != null) ? geometry.parameters.depth : 1;
            if (this.mode === "resize") {
                this.root.scale.x = Math.abs(width);
                this.root.scale.y = Math.abs(height);
                this.root.scale.z = Math.abs(depth);
            }
            else {
                this.root.scale.x = (width < 0 ? -1 : 1) * this.target.scale.x;
                this.root.scale.y = (height < 0 ? -1 : 1) * this.target.scale.y;
                this.root.scale.z = (depth < 0 ? -1 : 1) * this.target.scale.z;
            }
        }
        this.root.updateMatrixWorld(false);
        worldPosition.setFromMatrixPosition(this.root.matrixWorld);
        // NOTE: Workaround for negative scales messing with extracted rotation — elisee
        var scaleX = this.root.scale.x / Math.abs(this.root.scale.x);
        var scaleY = this.root.scale.y / Math.abs(this.root.scale.y);
        var scaleZ = this.root.scale.z / Math.abs(this.root.scale.z);
        var negativeScaleFixMatrix = new THREE.Matrix4().makeScale(scaleX, scaleY, scaleZ);
        worldRotation.setFromRotationMatrix(tempMatrix.extractRotation(this.root.matrixWorld).multiply(negativeScaleFixMatrix));
        this.camera.updateMatrixWorld(false);
        camPosition.setFromMatrixPosition(this.camera.matrixWorld);
        camRotation.setFromRotationMatrix(tempMatrix.extractRotation(this.camera.matrixWorld));
        var scale = worldPosition.distanceTo(camPosition) / 8 * this.size;
        this.position.copy(worldPosition);
        this.scale.set(scale, scale, scale);
        eye.copy(camPosition).sub(worldPosition).normalize();
        if (this.space === "local" || this.mode === "scale" || this.mode === "resize")
            this.gizmo[this.mode].update(worldRotation, eye);
        else if (this.space === "world")
            this.gizmo[this.mode].update(new THREE.Euler(), eye);
        if (!this.disabled)
            this.gizmo[this.mode].highlight(this.axis);
        this.updateMatrixWorld(true);
    };
    ;
    TransformControls.prototype.intersectObjects = function (pointer, objects) {
        var rect = this.domElement.getBoundingClientRect();
        var x = (pointer.clientX - rect.left) / rect.width;
        var y = (pointer.clientY - rect.top) / rect.height;
        pointerVector.set((x * 2) - 1, -(y * 2) + 1);
        ray.setFromCamera(pointerVector, this.camera);
        var intersections = ray.intersectObjects(objects, true);
        return intersections[0];
    };
    return TransformControls;
}(THREE.Object3D));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TransformControls;
