"use strict";
var Camera2DControls = (function () {
    function Camera2DControls(camera, canvas, options) {
        var _this = this;
        this.camera = camera;
        this.canvas = canvas;
        this.mousePosition = new THREE.Vector3(0, 0, 0);
        this.multiplier = 1;
        this.isMoving = false;
        this.onMouseDown = function (event) {
            if (event.button === 1 || (event.button === 0 && event.altKey))
                _this.isMoving = true;
        };
        this.onMouseUp = function (event) {
            if (event.button === 0 || event.button === 1)
                _this.isMoving = false;
        };
        this.onMouseMove = function (event) {
            var rect = _this.canvas.getBoundingClientRect();
            _this.mousePosition.x = (event.clientX - rect.left) / _this.canvas.clientWidth * 2 - 1;
            _this.mousePosition.y = -((event.clientY - rect.top) / _this.canvas.clientHeight * 2 - 1);
            if (_this.isMoving) {
                var cameraZ = _this.camera.threeCamera.position.z;
                _this.camera.threeCamera.position
                    .set(-event.movementX / _this.canvas.clientWidth * 2, event.movementY / _this.canvas.clientHeight * 2, 0)
                    .unproject(_this.camera.threeCamera)
                    .z = cameraZ;
                _this.camera.threeCamera.updateMatrixWorld(false);
                if (_this.options.moveCallback != null)
                    _this.options.moveCallback();
            }
        };
        this.onWheel = function (event) {
            if (event.ctrlKey)
                return;
            var newOrthographicScale;
            if (event.deltaY > 0)
                newOrthographicScale = Math.min(_this.options.zoomMax, _this.camera.orthographicScale * _this.multiplier * _this.options.zoomSpeed);
            else if (event.deltaY < 0)
                newOrthographicScale = Math.max(_this.options.zoomMin, _this.camera.orthographicScale * _this.multiplier / _this.options.zoomSpeed);
            else
                return;
            _this.changeOrthographicScale(newOrthographicScale, _this.mousePosition);
        };
        this.onKeyPress = function (event) {
            if (SupClient.Dialogs.BaseDialog.activeDialog != null)
                return;
            if (event.keyCode === 43 /* Ctrl+Numpad+ */) {
                var newOrthographicScale = Math.max(_this.options.zoomMin, _this.camera.orthographicScale * _this.multiplier / _this.options.zoomSpeed);
                _this.changeOrthographicScale(newOrthographicScale);
            }
            if (event.keyCode === 45 /* Ctrl+Numpad- */) {
                var newOrthographicScale = Math.min(_this.options.zoomMax, _this.camera.orthographicScale * _this.multiplier * _this.options.zoomSpeed);
                _this.changeOrthographicScale(newOrthographicScale);
            }
        };
        this.options = options != null ? options : {};
        if (this.options.zoomSpeed == null)
            this.options.zoomSpeed = 1.5;
        if (this.options.zoomMin == null)
            this.options.zoomMin = 0.1;
        if (this.options.zoomMax == null)
            this.options.zoomMax = 10000;
        canvas.addEventListener("mousedown", this.onMouseDown);
        canvas.addEventListener("mousemove", this.onMouseMove);
        canvas.addEventListener("wheel", this.onWheel);
        canvas.addEventListener("keypress", this.onKeyPress);
        document.addEventListener("mouseup", this.onMouseUp);
        canvas.addEventListener("mouseout", this.onMouseUp);
        canvas.addEventListener("contextmenu", function (event) { event.preventDefault(); });
    }
    Camera2DControls.prototype.setMultiplier = function (newMultiplier) {
        this.multiplier = newMultiplier;
        var newOrthographicScale = this.camera.orthographicScale * this.multiplier;
        this.changeOrthographicScale(newOrthographicScale);
    };
    Camera2DControls.prototype.changeOrthographicScale = function (newOrthographicScale, mousePosition) {
        if (mousePosition === void 0) { mousePosition = { x: 0, y: 0 }; }
        var startPosition = new THREE.Vector3(mousePosition.x, mousePosition.y, 0).unproject(this.camera.threeCamera);
        this.camera.setOrthographicScale(newOrthographicScale / this.multiplier);
        var endPosition = new THREE.Vector3(mousePosition.x, mousePosition.y, 0).unproject(this.camera.threeCamera);
        this.camera.threeCamera.position.x += startPosition.x - endPosition.x;
        this.camera.threeCamera.position.y += startPosition.y - endPosition.y;
        this.camera.threeCamera.updateMatrixWorld(false);
        if (this.options.zoomCallback != null)
            this.options.zoomCallback();
    };
    return Camera2DControls;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Camera2DControls;
