"use strict";
var tmpVector3 = new THREE.Vector3();
var tmpQuaternion = new THREE.Quaternion();
var upVector = new THREE.Vector3(0, 1, 0);
var lerpFactor = 0.3;
var minOrbitRadius = 0.5;
var maxOrbitRadius = 500;
var initialOrbitRadius = 10;
var panningSpeed = 0.005;
var orbitingSpeed = 0.008;
var zoomingSpeed = 1.3;
var Camera3DControls = (function () {
    function Camera3DControls(root, camera, canvas) {
        var _this = this;
        this.root = root;
        this.camera = camera;
        this.canvas = canvas;
        this.moveSpeed = 0.3;
        this.isPanning = false;
        this.isOrbiting = false;
        this.orbitRadius = initialOrbitRadius;
        this.targetOrbitRadius = initialOrbitRadius;
        this.moveVector = new THREE.Vector3();
        this.pivotMarkerOpacity = 0;
        this.onMouseDown = function (event) {
            if (_this.isPanning || _this.isOrbiting)
                return;
            if (event.button === 2) {
                _this.isPanning = true;
            }
            else if (event.button === 1 || (event.button === 0 && event.altKey)) {
                _this.isOrbiting = true;
                if (_this.canvas.requestPointerLock)
                    _this.canvas.requestPointerLock();
                else if (_this.canvas.webkitRequestPointerLock)
                    _this.canvas.webkitRequestPointerLock();
                else if (_this.canvas.mozRequestPointerLock)
                    _this.canvas.mozRequestPointerLock();
                _this.targetOrbitPivot = new THREE.Vector3(0, 0, -_this.targetOrbitRadius).applyQuaternion(_this.camera.threeCamera.quaternion).add(_this.camera.threeCamera.position);
                tmpQuaternion.setFromUnitVectors(_this.camera.threeCamera.up, upVector);
                tmpVector3.copy(_this.camera.threeCamera.position).sub(_this.targetOrbitPivot).applyQuaternion(tmpQuaternion);
                _this.theta = Math.atan2(tmpVector3.x, tmpVector3.z);
                _this.targetTheta = _this.theta;
                _this.phi = Math.atan2(Math.sqrt(tmpVector3.x * tmpVector3.x + tmpVector3.z * tmpVector3.z), tmpVector3.y);
                _this.targetPhi = _this.phi;
            }
        };
        this.onMouseMove = function (event) {
            if (_this.isPanning) {
                var panningMultiplier = panningSpeed * (1 + Math.sqrt(_this.targetOrbitRadius));
                tmpVector3.set(-event.movementX * panningMultiplier, event.movementY * panningMultiplier, 0).applyQuaternion(_this.camera.threeCamera.quaternion);
                _this.targetOrbitPivot.add(tmpVector3);
                _this.camera.threeCamera.position.add(tmpVector3);
                _this.camera.threeCamera.updateMatrixWorld(false);
            }
            else if (_this.isOrbiting) {
                _this.targetTheta -= event.movementX * orbitingSpeed;
                _this.targetPhi -= event.movementY * orbitingSpeed;
                _this.targetPhi = Math.max(0.001, Math.min(Math.PI - 0.001, _this.targetPhi));
            }
        };
        this.onWheel = function (event) {
            var multiplier = event.deltaY / 100 * zoomingSpeed;
            if (event.deltaY > 0)
                _this.targetOrbitRadius *= multiplier;
            else if (event.deltaY < 0)
                _this.targetOrbitRadius /= -multiplier;
            else
                return;
            _this.targetOrbitRadius = Math.min(Math.max(_this.targetOrbitRadius, minOrbitRadius), maxOrbitRadius);
        };
        this.onKeyDown = function (event) {
            if (event.ctrlKey || event.altKey || event.metaKey)
                return;
            if (event.keyCode === 87 /* W */ || event.keyCode === 90 /* Z */) {
                _this.moveVector.z = -1;
            }
            else if (event.keyCode === 83 /* S */) {
                _this.moveVector.z = 1;
            }
            else if (event.keyCode === 81 /* W */ || event.keyCode === 65 /* A */) {
                _this.moveVector.x = -1;
            }
            else if (event.keyCode === 68 /* D */) {
                _this.moveVector.x = 1;
            }
            else if (event.keyCode === 32 /* SPACE */) {
                _this.moveVector.y = 1;
            }
            else if (event.keyCode === 16 /* SHIFT */) {
                _this.moveVector.y = -1;
            }
        };
        this.onKeyUp = function (event) {
            if (event.keyCode === 87 /* W */ || event.keyCode === 90 /* Z */) {
                _this.moveVector.z = 0;
            }
            else if (event.keyCode === 83 /* S */) {
                _this.moveVector.z = 0;
            }
            else if (event.keyCode === 81 /* W */ || event.keyCode === 65 /* A */) {
                _this.moveVector.x = 0;
            }
            else if (event.keyCode === 68 /* D */) {
                _this.moveVector.x = 0;
            }
            else if (event.keyCode === 32 /* SPACE */) {
                _this.moveVector.y = 0;
            }
            else if (event.keyCode === 16 /* SHIFT */) {
                _this.moveVector.y = 0;
            }
        };
        this.onMouseUp = function (event) {
            if (event.button === 2) {
                _this.isPanning = false;
            }
            else if (event.button === 1 || event.button === 0) {
                _this.isOrbiting = false;
                if (document.exitPointerLock)
                    document.exitPointerLock();
                else if (document.webkitExitPointerLock)
                    document.webkitExitPointerLock();
                else if (document.mozExitPointerLock)
                    document.mozExitPointerLock();
            }
        };
        this.onBlur = function (event) {
            _this.moveVector.set(0, 0, 0);
            _this.isPanning = false;
            if (_this.isOrbiting) {
                _this.isOrbiting = false;
                if (document.exitPointerLock)
                    document.exitPointerLock();
                else if (document.webkitExitPointerLock)
                    document.webkitExitPointerLock();
                else if (document.mozExitPointerLock)
                    document.mozExitPointerLock();
            }
        };
        this.orbitPivot = new THREE.Vector3(0, 0, -this.orbitRadius).applyQuaternion(this.camera.threeCamera.quaternion).add(this.camera.threeCamera.position);
        this.targetOrbitPivot = this.orbitPivot.clone();
        tmpQuaternion.setFromUnitVectors(this.camera.threeCamera.up, upVector);
        tmpVector3.copy(this.camera.threeCamera.position).sub(this.orbitPivot).applyQuaternion(tmpQuaternion);
        this.theta = Math.atan2(tmpVector3.x, tmpVector3.z);
        this.targetTheta = this.theta;
        this.phi = Math.atan2(Math.sqrt(tmpVector3.x * tmpVector3.x + tmpVector3.z * tmpVector3.z), tmpVector3.y);
        this.targetPhi = this.phi;
        var pivotGeometry = new THREE.Geometry();
        pivotGeometry.vertices.push(new THREE.Vector3(-0.5, 0, 0), new THREE.Vector3(0.5, 0, 0), new THREE.Vector3(0, -0.5, 0), new THREE.Vector3(0, 0.5, 0), new THREE.Vector3(0, 0, -0.5), new THREE.Vector3(0, 0, 0.5));
        this.pivotMarker = new THREE.LineSegments(pivotGeometry, new THREE.LineBasicMaterial({ color: 0xffffff, opacity: this.pivotMarkerOpacity, transparent: true }));
        this.pivotMarker.layers.set(1);
        root.add(this.pivotMarker);
        canvas.addEventListener("mousedown", this.onMouseDown);
        canvas.addEventListener("mousemove", this.onMouseMove);
        canvas.addEventListener("wheel", this.onWheel);
        canvas.addEventListener("keydown", this.onKeyDown);
        document.addEventListener("keyup", this.onKeyUp);
        document.addEventListener("mouseup", this.onMouseUp);
        canvas.addEventListener("mouseout", this.onMouseUp);
        canvas.addEventListener("contextmenu", function (event) { event.preventDefault(); });
        window.addEventListener("blur", this.onBlur);
    }
    Camera3DControls.prototype.resetOrbitPivot = function (position) {
        this.targetOrbitPivot.copy(position);
        this.targetOrbitRadius = Math.max(this.targetOrbitRadius, initialOrbitRadius);
        return this;
    };
    Camera3DControls.prototype.setMoveSpeed = function (moveSpeed) {
        this.moveSpeed = moveSpeed;
        return this;
    };
    Camera3DControls.prototype.update = function () {
        if (this.moveVector.length() !== 0) {
            var rotatedMoveVector = this.moveVector.clone();
            rotatedMoveVector.applyQuaternion(this.camera.threeCamera.quaternion).normalize().multiplyScalar(this.moveSpeed);
            this.camera.threeCamera.position.add(rotatedMoveVector);
            this.targetOrbitPivot.add(rotatedMoveVector);
        }
        this.orbitPivot.lerp(this.targetOrbitPivot, lerpFactor);
        this.orbitRadius += (this.targetOrbitRadius - this.orbitRadius) * lerpFactor;
        this.theta += (this.targetTheta - this.theta) * lerpFactor;
        this.phi += (this.targetPhi - this.phi) * lerpFactor;
        tmpVector3.x = this.orbitRadius * Math.sin(this.phi) * Math.sin(this.theta);
        tmpVector3.y = this.orbitRadius * Math.cos(this.phi);
        tmpVector3.z = this.orbitRadius * Math.sin(this.phi) * Math.cos(this.theta);
        tmpVector3.applyQuaternion(tmpQuaternion.clone().inverse());
        this.camera.threeCamera.position.copy(this.orbitPivot).add(tmpVector3);
        this.camera.threeCamera.lookAt(this.orbitPivot);
        this.camera.threeCamera.updateMatrixWorld(false);
        // Update marker
        if (this.orbitPivot.distanceTo(this.targetOrbitPivot) > 0.1 ||
            Math.abs(this.orbitRadius - this.targetOrbitRadius) > 0.1 || this.isOrbiting || this.isPanning) {
            this.pivotMarker.material.opacity = 0.5;
        }
        else {
            this.pivotMarker.material.opacity *= 1 - lerpFactor;
        }
        this.pivotMarker.position.copy(this.orbitPivot);
        this.pivotMarker.updateMatrixWorld(false);
    };
    return Camera3DControls;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Camera3DControls;
