"use strict";
var TextRendererUpdater = (function () {
    function TextRendererUpdater(client, textRenderer, config, externalSubscriber) {
        var _this = this;
        this.client = client;
        this.textRenderer = textRenderer;
        this.externalSubscriber = externalSubscriber;
        this.onFontAssetReceived = function (assetId, asset) {
            _this.fontAsset = asset;
            _this.textRenderer.setText(_this.text);
            _this.textRenderer.setOptions(_this.options);
            if (!_this.overrideOpacity)
                _this.textRenderer.opacity = asset.pub.opacity;
            _this.setupFont();
            if (_this.externalSubscriber.onAssetReceived)
                _this.externalSubscriber.onAssetReceived(assetId, asset);
        };
        this.onFontAssetEdited = function (assetId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var commandFunction = _this.onEditCommands[command];
            if (commandFunction != null)
                commandFunction.apply(_this, args);
            if (_this.externalSubscriber.onAssetEdited)
                (_a = _this.externalSubscriber).onAssetEdited.apply(_a, [assetId, command].concat(args));
            var _a;
        };
        this.onEditCommands = {
            upload: function () { _this.setupFont(); },
            setProperty: function (path, value) {
                switch (path) {
                    case "isBitmap":
                        _this.setupFont();
                        break;
                    case "opacity":
                        if (!_this.overrideOpacity)
                            _this.textRenderer.setOpacity(value);
                        break;
                    default: _this.textRenderer.setFont(_this.fontAsset.pub);
                }
            }
        };
        this.onFontAssetTrashed = function (assetId) {
            _this.textRenderer.clearMesh();
            if (_this.externalSubscriber.onAssetTrashed != null)
                _this.externalSubscriber.onAssetTrashed(assetId);
        };
        this.fontAssetId = config.fontAssetId;
        this.text = config.text;
        this.options = {
            alignment: config.alignment,
            verticalAlignment: config.verticalAlignment,
            size: config.size,
            color: config.color,
        };
        this.overrideOpacity = config.overrideOpacity;
        this.opacity = config.opacity;
        if (this.overrideOpacity)
            this.textRenderer.setOpacity(this.opacity);
        if (this.externalSubscriber == null)
            this.externalSubscriber = {};
        this.fontSubscriber = {
            onAssetReceived: this.onFontAssetReceived,
            onAssetEdited: this.onFontAssetEdited,
            onAssetTrashed: this.onFontAssetTrashed
        };
        if (this.fontAssetId != null)
            this.client.subAsset(this.fontAssetId, "font", this.fontSubscriber);
    }
    TextRendererUpdater.prototype.destroy = function () {
        if (this.fontAssetId != null)
            this.client.unsubAsset(this.fontAssetId, this.fontSubscriber);
    };
    TextRendererUpdater.prototype.config_setProperty = function (path, value) {
        switch (path) {
            case "fontAssetId":
                {
                    if (this.fontAssetId != null)
                        this.client.unsubAsset(this.fontAssetId, this.fontSubscriber);
                    this.fontAssetId = value;
                    this.fontAsset = null;
                    this.textRenderer.setFont(null);
                    if (this.fontAssetId != null)
                        this.client.subAsset(this.fontAssetId, "font", this.fontSubscriber);
                }
                break;
            case "text":
                {
                    this.text = value;
                    this.textRenderer.setText(this.text);
                }
                break;
            case "alignment":
            case "verticalAlignment":
            case "size":
            case "color":
                {
                    this.options[path] = (value !== "") ? value : null;
                    this.textRenderer.setOptions(this.options);
                }
                break;
            case "overrideOpacity":
            case "opacity":
                {
                    this[path] = value;
                    if (this.overrideOpacity)
                        this.textRenderer.setOpacity(this.opacity);
                    else if (this.fontAsset != null)
                        this.textRenderer.setOpacity(this.fontAsset.pub.opacity);
                }
                break;
        }
    };
    TextRendererUpdater.prototype.setupFont = function () {
        var _this = this;
        this.textRenderer.clearMesh();
        if (this.fontAsset.pub.isBitmap) {
            if (this.fontAsset.pub.texture != null) {
                var image = this.fontAsset.pub.texture.image;
                if (image.complete)
                    this.textRenderer.setFont(this.fontAsset.pub);
                else
                    image.addEventListener("load", function () { _this.textRenderer.setFont(_this.fontAsset.pub); });
            }
        }
        else {
            if (this.fontAsset.font == null)
                this.textRenderer.setFont(this.fontAsset.pub);
            else {
                this.fontAsset.font.load().then(function () { _this.textRenderer.setFont(_this.fontAsset.pub); }, function () { _this.textRenderer.setFont(_this.fontAsset.pub); });
            }
        }
    };
    return TextRendererUpdater;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TextRendererUpdater;
