"use strict";
var outputFolder;
var GameBuildSettingsEditor = (function () {
    function GameBuildSettingsEditor(container, entries, entriesTreeView) {
        var _this = this;
        this.entries = entries;
        this.entriesTreeView = entriesTreeView;
        var _a = SupClient.table.createTable(container), table = _a.table, tbody = _a.tbody;
        this.table = table;
        table.classList.add("properties");
        table.hidden = true;
        var outputFolderRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("buildSettingsEditors:game.outputFolder"));
        var inputs = SupClient.html("div", "inputs", { parent: outputFolderRow.valueCell });
        var value = outputFolder != null ? outputFolder : "";
        this.outputFolderTextfield = SupClient.html("input", { parent: inputs, type: "text", value: value, readOnly: true, style: { cursor: "pointer" } });
        this.outputFolderButton = SupClient.html("button", { parent: inputs, textContent: SupClient.i18n.t("common:actions.select") });
        this.outputFolderTextfield.addEventListener("click", function (event) { event.preventDefault(); _this.selectOutputfolder(); });
        this.outputFolderButton.addEventListener("click", function (event) { event.preventDefault(); _this.selectOutputfolder(); });
        var errorRow = SupClient.table.appendRow(tbody, "Error");
        this.errorRowElt = errorRow.row;
        this.errorRowElt.hidden = true;
        this.errorInput = SupClient.html("input", { parent: errorRow.valueCell, type: "text", readOnly: true });
    }
    GameBuildSettingsEditor.prototype.setVisible = function (visible) {
        this.table.hidden = !visible;
    };
    GameBuildSettingsEditor.prototype.getSettings = function (callback) {
        this.ensureOutputFolderValid(function (outputFolderValid) {
            callback(outputFolderValid ? { outputFolder: outputFolder } : null);
        });
    };
    GameBuildSettingsEditor.prototype.selectOutputfolder = function () {
        var _this = this;
        SupApp.chooseFolder(function (folderPath) {
            if (folderPath == null)
                return;
            outputFolder = _this.outputFolderTextfield.value = folderPath;
            _this.ensureOutputFolderValid();
        });
    };
    GameBuildSettingsEditor.prototype.ensureOutputFolderValid = function (callback) {
        var _this = this;
        if (outputFolder == null) {
            this.displayError(SupClient.i18n.t("buildSettingsEditors:game.errors.selectDestionationFolder"));
            if (callback != null)
                callback(false);
            return;
        }
        SupApp.readDir(outputFolder, function (err, files) {
            if (err != null) {
                _this.displayError(SupClient.i18n.t("buildSettingsEditors:game.errors.emptyDirectoryCheckFail"));
                console.log(err);
                if (callback != null)
                    callback(false);
                return;
            }
            var index = 0;
            while (index < files.length) {
                var item = files[index];
                if (item[0] === "." || item === "Thumbs.db")
                    files.splice(index, 1);
                else
                    index++;
            }
            if (files.length > 0) {
                _this.displayError(SupClient.i18n.t("buildSettingsEditors:game.errors.destinationFolderEmpty"));
                if (callback != null)
                    callback(false);
            }
            else {
                _this.errorRowElt.hidden = true;
                if (callback != null)
                    callback(true);
            }
        });
    };
    GameBuildSettingsEditor.prototype.displayError = function (err) {
        this.errorRowElt.hidden = false;
        this.errorInput.value = err;
        this.errorRowElt.animate([
            { transform: "translateX(0)" },
            { transform: "translateX(1.5vh)" },
            { transform: "translateX(0)" }
        ], { duration: 100 });
    };
    return GameBuildSettingsEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = GameBuildSettingsEditor;
