"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var path = require("path");
var GameSettingsResource = (function (_super) {
    __extends(GameSettingsResource, _super);
    function GameSettingsResource(id, pub, server) {
        _super.call(this, id, pub, GameSettingsResource.schema, server);
    }
    GameSettingsResource.prototype.init = function (callback) {
        this.pub = {
            formatVersion: GameSettingsResource.currentFormatVersion,
            startupSceneId: null,
            framesPerSecond: 60,
            ratioNumerator: null, ratioDenominator: null,
            customLayers: []
        };
        _super.prototype.init.call(this, callback);
    };
    GameSettingsResource.prototype.migrate = function (resourcePath, pub, callback) {
        var _this = this;
        if (pub.formatVersion === GameSettingsResource.currentFormatVersion) {
            callback(false);
            return;
        }
        if (pub.formatVersion == null) {
            // NOTE: Custom layers were introduced in Superpowers 0.8
            if (pub.customLayers == null)
                pub.customLayers = [];
            this.server.data.entries.walk(function (node) {
                var path = _this.server.data.entries.getPathFromId(node.id);
                if (path === pub.startupScene)
                    pub.startupSceneId = node.id;
            });
            delete pub.startupScene;
            pub.formatVersion = 1;
        }
        callback(true);
    };
    GameSettingsResource.prototype.restore = function () {
        if (this.pub.startupSceneId != null && this.server.data.entries.byId[this.pub.startupSceneId] != null) {
            this.emit("setAssetBadge", this.pub.startupSceneId, "startupScene", "info");
        }
    };
    GameSettingsResource.prototype.clientExport = function (outputPath, callback) {
        SupApp.writeFile(path.join(outputPath, "resource.json"), JSON.stringify(this.pub), callback);
    };
    GameSettingsResource.prototype.server_setProperty = function (client, path, value, callback) {
        var _this = this;
        var oldSceneId;
        if (path === "startupSceneId")
            oldSceneId = this.pub.startupSceneId;
        this.setProperty(path, value, function (err, actualValue) {
            if (err != null) {
                callback(err);
                return;
            }
            if (path === "startupSceneId") {
                if (oldSceneId != null && _this.server.data.entries.byId[oldSceneId] != null)
                    _this.emit("clearAssetBadge", oldSceneId, "startupScene");
                if (actualValue != null && _this.server.data.entries.byId[actualValue] != null)
                    _this.emit("setAssetBadge", actualValue, "startupScene", "info");
            }
            callback(null, null, path, actualValue);
        });
    };
    GameSettingsResource.currentFormatVersion = 1;
    GameSettingsResource.schema = {
        formatVersion: { type: "integer" },
        startupSceneId: { type: "string?", mutable: true },
        framesPerSecond: { type: "integer", minExcluded: 0, mutable: true },
        ratioNumerator: { type: "integer?", mutable: true },
        ratioDenominator: { type: "integer?", mutable: true },
        customLayers: {
            type: "array", mutable: true, minLength: 0, maxLength: 8,
            items: { type: "string", minLength: 1, maxLength: 80 }
        }
    };
    return GameSettingsResource;
}(SupCore.Data.Base.Resource));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = GameSettingsResource;
