"use strict";
var GameSettingsResource_1 = require("../data/GameSettingsResource");
var GameSettingsEditor = (function () {
    function GameSettingsEditor(container, projectClient) {
        var _this = this;
        this.fields = {};
        this.onResourceReceived = function (resourceId, resource) {
            _this.resource = resource;
            _this._setupCustomLayers();
            for (var setting in resource.pub) {
                if (setting === "formatVersion" || setting === "customLayers")
                    continue;
                if (setting === "startupSceneId")
                    _this._setStartupScene(resource.pub.startupSceneId);
                else
                    _this.fields[setting].value = resource.pub[setting];
            }
        };
        this.onResourceEdited = function (resourceId, command, propertyName) {
            if (propertyName === "customLayers")
                _this._setupCustomLayers();
            else if (propertyName === "startupSceneId")
                _this._setStartupScene(_this.resource.pub.startupSceneId);
            else
                _this.fields[propertyName].value = _this.resource.pub[propertyName];
        };
        this.onCustomLayerFieldChange = function (event) {
            var index = parseInt(event.target.dataset["customLayerIndex"], 10);
            if (index > _this.customLayers.length)
                return;
            if (index === _this.customLayers.length) {
                if (event.target.value === "")
                    return;
                _this.customLayers.push(event.target.value);
            }
            else {
                if (event.target.value === "") {
                    if (index === _this.customLayers.length - 1) {
                        _this.customLayers.pop();
                    }
                    else {
                        new SupClient.Dialogs.InfoDialog("Layer name cannot be empty");
                        event.target.value = _this.customLayers[index];
                        return;
                    }
                }
                else {
                    _this.customLayers[index] = event.target.value;
                }
            }
            _this.projectClient.editResource("gameSettings", "setProperty", "customLayers", _this.customLayers);
        };
        this.projectClient = projectClient;
        var tbody = SupClient.table.createTable(container).tbody;
        this.startupSceneRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:Game.startupScene"));
        this.sceneFieldSubscriber = SupClient.table.appendAssetField(this.startupSceneRow.valueCell, this.sceneAssetId, "scene", projectClient);
        this.sceneFieldSubscriber.on("select", function (assetId) {
            _this.projectClient.editResource("gameSettings", "setProperty", "startupSceneId", assetId);
        });
        this.fpsRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:Game.framesPerSecond"));
        this.fields["framesPerSecond"] = SupClient.table.appendNumberField(this.fpsRow.valueCell, "", { min: 1 });
        this.ratioRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:Game.screenRatio"));
        var ratioContainer = document.createElement("div");
        ratioContainer.className = "";
        this.ratioRow.valueCell.appendChild(ratioContainer);
        _a = SupClient.table.appendNumberFields(this.ratioRow.valueCell, ["", ""]), this.fields["ratioNumerator"] = _a[0], this.fields["ratioDenominator"] = _a[1];
        this.fields["ratioNumerator"].placeholder = SupClient.i18n.t("settingsEditors:Game.width");
        this.fields["ratioDenominator"].placeholder = SupClient.i18n.t("settingsEditors:Game.height");
        this.customLayersRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:Game.layers"));
        this.layerContainers = document.createElement("div");
        this.layerContainers.className = "list";
        this.customLayersRow.valueCell.appendChild(this.layerContainers);
        this.fields["defaultLayer"] = SupClient.table.appendTextField(this.layerContainers, "Default");
        this.fields["defaultLayer"].readOnly = true;
        for (var i = 0; i < GameSettingsResource_1.default.schema["customLayers"].maxLength; i++) {
            var field = this.fields[("customLayer" + i)] = SupClient.table.appendTextField(this.layerContainers, "");
            field.dataset["customLayerIndex"] = i.toString();
            field.addEventListener("change", this.onCustomLayerFieldChange);
        }
        this.fields["framesPerSecond"].addEventListener("change", function (event) {
            _this.projectClient.editResource("gameSettings", "setProperty", "framesPerSecond", parseInt(event.target.value, 10));
        });
        this.fields["ratioNumerator"].addEventListener("change", function (event) {
            _this.projectClient.editResource("gameSettings", "setProperty", "ratioNumerator", parseInt(event.target.value, 10));
        });
        this.fields["ratioDenominator"].addEventListener("change", function (event) {
            _this.projectClient.editResource("gameSettings", "setProperty", "ratioDenominator", parseInt(event.target.value, 10));
        });
        this.projectClient.subResource("gameSettings", this);
        var _a;
    }
    GameSettingsEditor.prototype._setStartupScene = function (id) {
        this.sceneAssetId = id;
        this.sceneFieldSubscriber.onChangeAssetId(id);
    };
    GameSettingsEditor.prototype._setupCustomLayers = function () {
        this.customLayers = this.resource.pub.customLayers.slice(0);
        for (var i = 0; i < GameSettingsResource_1.default.schema["customLayers"].maxLength; i++) {
            var field = this.fields[("customLayer" + i)];
            if (i === this.customLayers.length) {
                field.placeholder = SupClient.i18n.t("settingsEditors:Game.newLayer");
                field.value = "";
            }
            else {
                field.placeholder = "";
            }
            if (i > this.customLayers.length) {
                if (field.parentElement != null)
                    this.layerContainers.removeChild(field);
            }
            else {
                if (field.parentElement == null)
                    this.layerContainers.appendChild(field);
                if (i < this.customLayers.length)
                    field.value = this.customLayers[i];
            }
        }
    };
    return GameSettingsEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = GameSettingsEditor;
