"use strict";
var ModelRendererEditor = (function () {
    function ModelRendererEditor(tbody, config, projectClient, editConfig) {
        var _this = this;
        this.tbody = tbody;
        this.projectClient = projectClient;
        this.editConfig = editConfig;
        this.modelAssetId = config.modelAssetId;
        this.animationId = config.animationId;
        this.shaderAssetId = config.shaderAssetId;
        this.overrideOpacity = config.overrideOpacity;
        this.opacity = config.opacity;
        var modelRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:ModelRenderer.model"));
        this.modelFieldSubscriber = SupClient.table.appendAssetField(modelRow.valueCell, this.modelAssetId, "model", projectClient);
        this.modelFieldSubscriber.on("select", function (assetId) {
            _this.editConfig("setProperty", "modelAssetId", assetId);
            _this.editConfig("setProperty", "animationId", null);
        });
        var animationRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:ModelRenderer.animation"));
        this.animationSelectBox = SupClient.table.appendSelectBox(animationRow.valueCell, { "": SupClient.i18n.t("common:none") });
        this.animationSelectBox.addEventListener("change", function (event) {
            var animationId = (event.target.value === "") ? null : event.target.value;
            _this.editConfig("setProperty", "animationId", animationId);
        });
        this.animationSelectBox.disabled = true;
        var shadowRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:ModelRenderer.shadow.title"));
        var shadowDiv = document.createElement("div");
        shadowDiv.classList.add("inputs");
        shadowRow.valueCell.appendChild(shadowDiv);
        var castSpan = document.createElement("span");
        castSpan.style.marginLeft = "5px";
        castSpan.textContent = SupClient.i18n.t("componentEditors:ModelRenderer.shadow.cast");
        shadowDiv.appendChild(castSpan);
        this.castShadowField = SupClient.table.appendBooleanField(shadowDiv, config.castShadow);
        this.castShadowField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "castShadow", event.target.checked);
        });
        var receiveSpan = document.createElement("span");
        receiveSpan.style.marginLeft = "5px";
        receiveSpan.textContent = SupClient.i18n.t("componentEditors:ModelRenderer.shadow.receive");
        shadowDiv.appendChild(receiveSpan);
        this.receiveShadowField = SupClient.table.appendBooleanField(shadowDiv, config.receiveShadow);
        this.receiveShadowField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "receiveShadow", event.target.checked);
        });
        var colorRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:ModelRenderer.color"));
        var colorInputs = SupClient.table.appendColorField(colorRow.valueCell, config.color);
        this.colorField = colorInputs.textField;
        this.colorField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "color", event.target.value);
        });
        this.colorPicker = colorInputs.pickerField;
        this.colorPicker.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "color", event.target.value.slice(1));
        });
        var opacityRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.opacity"), { checkbox: true });
        this.overrideOpacityField = opacityRow.checkbox;
        this.overrideOpacityField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "opacity", _this.asset != null ? _this.asset.pub.opacity : null);
            _this.editConfig("setProperty", "overrideOpacity", event.target.checked);
        });
        var opacityParent = document.createElement("div");
        opacityRow.valueCell.appendChild(opacityParent);
        var transparentOptions = {
            empty: "",
            opaque: SupClient.i18n.t("componentEditors:SpriteRenderer.opaque"),
            transparent: SupClient.i18n.t("componentEditors:SpriteRenderer.transparent"),
        };
        this.transparentField = SupClient.table.appendSelectBox(opacityParent, transparentOptions);
        this.transparentField.children[0].hidden = true;
        this.transparentField.addEventListener("change", function (event) {
            var opacity = _this.transparentField.value === "transparent" ? 1 : null;
            _this.editConfig("setProperty", "opacity", opacity);
        });
        this.opacityFields = SupClient.table.appendSliderField(opacityParent, "", { min: 0, max: 1, step: 0.1, sliderStep: 0.01 });
        this.opacityFields.numberField.parentElement.addEventListener("input", function (event) {
            _this.editConfig("setProperty", "opacity", parseFloat(event.target.value));
        });
        this.updateOpacityField();
        var materialRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:ModelRenderer.material"));
        this.materialSelectBox = SupClient.table.appendSelectBox(materialRow.valueCell, { "basic": "Basic", "phong": "Phong", "shader": "Shader" }, config.materialType);
        this.materialSelectBox.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "materialType", event.target.value);
        });
        var shaderRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:ModelRenderer.shader"));
        this.shaderRow = shaderRow.row;
        this.shaderFieldSubscriber = SupClient.table.appendAssetField(shaderRow.valueCell, this.shaderAssetId, "shader", this.projectClient);
        this.shaderFieldSubscriber.on("select", function (assetId) {
            _this.editConfig("setProperty", "shaderAssetId", assetId);
        });
        this.shaderRow.hidden = config.materialType !== "shader";
        if (this.modelAssetId != null)
            this.projectClient.subAsset(this.modelAssetId, "model", this);
    }
    ModelRendererEditor.prototype.destroy = function () {
        this.modelFieldSubscriber.destroy();
        this.shaderFieldSubscriber.destroy();
        if (this.modelAssetId != null)
            this.projectClient.unsubAsset(this.modelAssetId, this);
    };
    ModelRendererEditor.prototype.config_setProperty = function (path, value) {
        if (this.projectClient.entries == null)
            return;
        switch (path) {
            case "modelAssetId":
                if (this.modelAssetId != null) {
                    this.projectClient.unsubAsset(this.modelAssetId, this);
                    this.asset = null;
                }
                this.modelAssetId = value;
                this.animationSelectBox.disabled = true;
                if (this.modelAssetId != null)
                    this.projectClient.subAsset(this.modelAssetId, "model", this);
                this.modelFieldSubscriber.onChangeAssetId(this.modelAssetId);
                break;
            case "animationId":
                if (!this.animationSelectBox.disabled) {
                    this.animationSelectBox.value = (value != null) ? value : "";
                }
                this.animationId = value;
                break;
            case "castShadow":
                this.castShadowField.value = value;
                break;
            case "receiveShadow":
                this.receiveShadowField.value = value;
                break;
            case "color":
                this.colorField.value = value;
                this.colorPicker.value = "#" + value;
                break;
            case "overrideOpacity":
                this.overrideOpacity = value;
                this.updateOpacityField();
                break;
            case "opacity":
                this.opacity = value;
                this.updateOpacityField();
                break;
            case "materialType":
                this.materialSelectBox.value = value;
                this.shaderRow.hidden = value !== "shader";
                break;
            case "shaderAssetId":
                this.shaderAssetId = value;
                this.shaderFieldSubscriber.onChangeAssetId(this.shaderAssetId);
                break;
        }
    };
    // Network callbacks
    ModelRendererEditor.prototype.onAssetReceived = function (assetId, asset) {
        if (assetId !== this.modelAssetId)
            return;
        this.asset = asset;
        this._clearAnimations();
        for (var _i = 0, _a = this.asset.pub.animations; _i < _a.length; _i++) {
            var animation = _a[_i];
            SupClient.table.appendSelectOption(this.animationSelectBox, animation.id, animation.name);
        }
        this.animationSelectBox.value = (this.animationId != null) ? this.animationId : "";
        this.animationSelectBox.disabled = false;
        this.updateOpacityField();
    };
    ModelRendererEditor.prototype.onAssetEdited = function (assetId, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        if (assetId !== this.modelAssetId)
            return;
        if (command === "setProperty" && args[0] === "opacity")
            this.updateOpacityField();
        if (command.indexOf("Animation") !== -1)
            return;
        var animationId = this.animationSelectBox.value;
        this._clearAnimations();
        for (var _a = 0, _b = this.asset.pub.animations; _a < _b.length; _a++) {
            var animation = _b[_a];
            SupClient.table.appendSelectOption(this.animationSelectBox, animation.id, animation.name);
        }
        if (animationId != null && this.asset.animations.byId[animationId] != null)
            this.animationSelectBox.value = animationId;
        else
            this.editConfig("setProperty", "animationId", "");
    };
    ModelRendererEditor.prototype.onAssetTrashed = function () {
        this.asset = null;
        this._clearAnimations();
        this.animationSelectBox.value = "";
        this.animationSelectBox.disabled = true;
    };
    // User interface
    ModelRendererEditor.prototype._clearAnimations = function () {
        while (true) {
            var child = this.animationSelectBox.children[1];
            if (child == null)
                break;
            this.animationSelectBox.removeChild(child);
        }
    };
    ModelRendererEditor.prototype.updateOpacityField = function () {
        this.overrideOpacityField.checked = this.overrideOpacity;
        this.transparentField.disabled = !this.overrideOpacity;
        this.opacityFields.sliderField.disabled = !this.overrideOpacity;
        this.opacityFields.numberField.disabled = !this.overrideOpacity;
        if (!this.overrideOpacity && this.asset == null) {
            this.transparentField.value = "empty";
            this.opacityFields.numberField.parentElement.hidden = true;
        }
        else {
            var opacity = this.overrideOpacity ? this.opacity : this.asset.pub.opacity;
            if (opacity != null) {
                this.transparentField.value = "transparent";
                this.opacityFields.numberField.parentElement.hidden = false;
                this.opacityFields.sliderField.value = opacity.toString();
                this.opacityFields.numberField.value = opacity.toString();
            }
            else {
                this.transparentField.value = "opaque";
                this.opacityFields.numberField.parentElement.hidden = true;
            }
        }
    };
    return ModelRendererEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = ModelRendererEditor;
