"use strict";
var CameraEditor = (function () {
    function CameraEditor(tbody, config, projectClient, editConfig) {
        var _this = this;
        this.viewportFields = {};
        this.onChangeMode = function (event) { _this.editConfig("setProperty", "mode", event.target.value); };
        this.onChangeFOV = function (event) { _this.editConfig("setProperty", "fov", parseFloat(event.target.value)); };
        this.onChangeOrthographicScale = function (event) { _this.editConfig("setProperty", "orthographicScale", parseFloat(event.target.value)); };
        this.onChangeDepth = function (event) { _this.editConfig("setProperty", "depth", parseFloat(event.target.value)); };
        this.onChangeNearClippingPlane = function (event) { _this.editConfig("setProperty", "nearClippingPlane", parseFloat(event.target.value)); };
        this.onChangeFarClippingPlane = function (event) { _this.editConfig("setProperty", "farClippingPlane", parseFloat(event.target.value)); };
        this.onChangeViewportX = function (event) { _this.editConfig("setProperty", "viewport.x", parseFloat(event.target.value)); };
        this.onChangeViewportY = function (event) { _this.editConfig("setProperty", "viewport.y", parseFloat(event.target.value)); };
        this.onChangeViewportWidth = function (event) { _this.editConfig("setProperty", "viewport.width", parseFloat(event.target.value)); };
        this.onChangeViewportHeight = function (event) { _this.editConfig("setProperty", "viewport.height", parseFloat(event.target.value)); };
        this.projectClient = projectClient;
        this.editConfig = editConfig;
        var modeRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.mode"));
        var modeOptions = {
            perspective: SupClient.i18n.t("componentEditors:Camera.modeOptions.perspective"),
            orthographic: SupClient.i18n.t("componentEditors:Camera.modeOptions.orthographic")
        };
        this.modeSelectBox = SupClient.table.appendSelectBox(modeRow.valueCell, modeOptions, config.mode);
        this.fovRowParts = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.fieldOfView"));
        this.fovField = SupClient.table.appendNumberField(this.fovRowParts.valueCell, config.fov, { min: 0.1, max: 179.9, step: 0.1 });
        this.orthographicScaleRowParts = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.orthographicScale"));
        this.orthographicScaleField = SupClient.table.appendNumberField(this.orthographicScaleRowParts.valueCell, config.orthographicScale, { min: 0.1, step: 0.1 });
        if (config.mode === "perspective")
            this.orthographicScaleRowParts.row.style.display = "none";
        else
            this.fovRowParts.row.style.display = "none";
        var depthOptions = { title: SupClient.i18n.t("componentEditors:Camera.depthTitle") };
        var depthRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.depth"), depthOptions);
        this.depthField = SupClient.table.appendNumberField(depthRow.valueCell, config.depth);
        var layersOptions = { title: SupClient.i18n.t("componentEditors:Camera.layersTitle") };
        var layersRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.layers"), layersOptions);
        var layersField = SupClient.table.appendTextField(layersRow.valueCell, "");
        layersField.disabled = true;
        layersField.placeholder = "(not yet customizable)";
        var nearClippingPlaneRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.nearPlane"));
        this.nearClippingPlaneField = SupClient.table.appendNumberField(nearClippingPlaneRow.valueCell, config.nearClippingPlane, { min: 0.1 });
        var farClippingPlaneRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.farPlane"));
        this.farClippingPlaneField = SupClient.table.appendNumberField(farClippingPlaneRow.valueCell, config.farClippingPlane, { min: 0.1 });
        SupClient.table.appendHeader(tbody, SupClient.i18n.t("componentEditors:Camera.viewport.title"));
        var viewportXRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.viewport.position"));
        _a = SupClient.table.appendNumberFields(viewportXRow.valueCell, [config.viewport.x, config.viewport.y], { min: 0, max: 1, step: 0.1 }), this.viewportFields.x = _a[0], this.viewportFields.y = _a[1];
        var widthRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:Camera.viewport.size"));
        _b = SupClient.table.appendNumberFields(widthRow.valueCell, [config.viewport.width, config.viewport.height], { min: 0, max: 1, step: 0.1 }), this.viewportFields.width = _b[0], this.viewportFields.height = _b[1];
        this.modeSelectBox.addEventListener("change", this.onChangeMode);
        this.fovField.addEventListener("input", this.onChangeFOV);
        this.orthographicScaleField.addEventListener("input", this.onChangeOrthographicScale);
        this.depthField.addEventListener("change", this.onChangeDepth);
        this.nearClippingPlaneField.addEventListener("change", this.onChangeNearClippingPlane);
        this.farClippingPlaneField.addEventListener("change", this.onChangeFarClippingPlane);
        this.viewportFields.x.addEventListener("change", this.onChangeViewportX);
        this.viewportFields.y.addEventListener("change", this.onChangeViewportY);
        this.viewportFields.width.addEventListener("change", this.onChangeViewportWidth);
        this.viewportFields.height.addEventListener("change", this.onChangeViewportHeight);
        var _a, _b;
    }
    CameraEditor.prototype.destroy = function () { };
    CameraEditor.prototype.config_setProperty = function (path, value) {
        switch (path) {
            case "mode":
                {
                    this.modeSelectBox.value = value;
                    this.orthographicScaleRowParts.row.style.display = (value === "perspective") ? "none" : "";
                    this.fovRowParts.row.style.display = (value === "perspective") ? "" : "none";
                }
                break;
            case "fov":
                {
                    this.fovField.value = value;
                }
                break;
            case "orthographicScale":
                {
                    this.orthographicScaleField.value = value;
                }
                break;
            case "depth":
                {
                    this.depthField.value = value;
                }
                break;
            case "nearClippingPlane":
                {
                    this.nearClippingPlaneField.value = value;
                }
                break;
            case "farClippingPlane":
                {
                    this.farClippingPlaneField.value = value;
                }
                break;
            case "viewport.x":
                {
                    this.viewportFields.x.value = value;
                }
                break;
            case "viewport.y":
                {
                    this.viewportFields.y.value = value;
                }
                break;
            case "viewport.width":
                {
                    this.viewportFields.width.value = value;
                }
                break;
            case "viewport.height":
                {
                    this.viewportFields.height.value = value;
                }
                break;
        }
    };
    return CameraEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = CameraEditor;
