"use strict";
var tmpVector3 = new SupEngine.THREE.Vector3();
var tmpQuaternion = new SupEngine.THREE.Quaternion();
var SceneUpdater = (function () {
    function SceneUpdater(client, engine, config, externalSubscriber) {
        var _this = this;
        this.client = client;
        this.externalSubscriber = externalSubscriber;
        this.bySceneNodeId = {};
        this.onSceneAssetReceived = function (assetId, asset) {
            _this.sceneAsset = asset;
            var walk = function (node) {
                _this.createNodeActor(node);
                if (node.children != null && node.children.length > 0) {
                    for (var _i = 0, _a = node.children; _i < _a.length; _i++) {
                        var child = _a[_i];
                        walk(child);
                    }
                }
            };
            for (var _i = 0, _a = asset.nodes.pub; _i < _a.length; _i++) {
                var node = _a[_i];
                walk(node);
            }
            if (_this.externalSubscriber.onAssetReceived != null)
                _this.externalSubscriber.onAssetReceived(assetId, asset);
        };
        this.onSceneAssetEdited = function (assetId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var commandFunction = _this.onEditCommands[command];
            if (commandFunction != null)
                commandFunction.apply(_this, args);
            if (_this.externalSubscriber.onAssetEdited != null)
                (_a = _this.externalSubscriber).onAssetEdited.apply(_a, [assetId, command].concat(args));
            var _a;
        };
        this.onEditCommands = {
            addNode: function (node, parentId, index) {
                _this.createNodeActor(node);
            },
            moveNode: function (id, parentId, index) {
                var nodeActor = _this.bySceneNodeId[id].actor;
                var parentNodeActor = (_this.bySceneNodeId[parentId] != null) ? _this.bySceneNodeId[parentId].actor : null;
                nodeActor.setParent(parentNodeActor);
                _this.onUpdateMarkerRecursive(id);
            },
            setNodeProperty: function (id, path, value) {
                var nodeEditorData = _this.bySceneNodeId[id];
                switch (path) {
                    case "position":
                        nodeEditorData.actor.setLocalPosition(value);
                        if (!_this.isInPrefab)
                            _this.onUpdateMarkerRecursive(id);
                        break;
                    case "orientation":
                        nodeEditorData.actor.setLocalOrientation(value);
                        if (!_this.isInPrefab)
                            _this.onUpdateMarkerRecursive(id);
                        break;
                    case "scale":
                        nodeEditorData.actor.setLocalScale(value);
                        if (!_this.isInPrefab)
                            _this.onUpdateMarkerRecursive(id);
                        break;
                    case "prefab.sceneAssetId":
                        nodeEditorData.prefabUpdater.config_setProperty("sceneAssetId", value);
                        break;
                }
            },
            duplicateNode: function (rootNode, newNodes) {
                for (var _i = 0, newNodes_1 = newNodes; _i < newNodes_1.length; _i++) {
                    var newNode = newNodes_1[_i];
                    _this.createNodeActor(newNode.node);
                }
            },
            removeNode: function (id) {
                _this.recurseClearActor(id);
            },
            addComponent: function (nodeComponent, nodeId, index) {
                this.createNodeActorComponent(this.sceneAsset.nodes.byId[nodeId], nodeComponent, this.bySceneNodeId[nodeId].actor);
            },
            editComponent: function (nodeId, componentId, command) {
                var args = [];
                for (var _i = 3; _i < arguments.length; _i++) {
                    args[_i - 3] = arguments[_i];
                }
                var componentUpdater = this.bySceneNodeId[nodeId].bySceneComponentId[componentId].componentUpdater;
                if (componentUpdater[("config_" + command)] != null)
                    componentUpdater[("config_" + command)].apply(componentUpdater, args);
            },
            removeComponent: function (nodeId, componentId) {
                this.gameInstance.destroyComponent(this.bySceneNodeId[nodeId].bySceneComponentId[componentId].component);
                this.bySceneNodeId[nodeId].bySceneComponentId[componentId].componentUpdater.destroy();
                delete this.bySceneNodeId[nodeId].bySceneComponentId[componentId];
            }
        };
        this.onSceneAssetTrashed = function (assetId) {
            _this.clearScene();
            if (_this.sceneSubscriber.onAssetTrashed != null)
                _this.sceneSubscriber.onAssetTrashed(assetId);
        };
        this.gameInstance = engine.gameInstance;
        this.rootActor = engine.actor;
        this.sceneAssetId = config.sceneAssetId;
        this.isInPrefab = config.isInPrefab;
        if (this.externalSubscriber == null)
            this.externalSubscriber = {};
        this.sceneSubscriber = {
            onAssetReceived: this.onSceneAssetReceived,
            onAssetEdited: this.onSceneAssetEdited,
            onAssetTrashed: this.onSceneAssetTrashed
        };
        if (this.sceneAssetId != null)
            this.client.subAsset(this.sceneAssetId, "scene", this.sceneSubscriber);
    }
    SceneUpdater.prototype.destroy = function () {
        this.clearScene();
        if (this.sceneAssetId != null)
            this.client.unsubAsset(this.sceneAssetId, this.sceneSubscriber);
    };
    SceneUpdater.prototype.onUpdateMarkerRecursive = function (nodeId) {
        var _this = this;
        this.sceneAsset.nodes.walkNode(this.sceneAsset.nodes.byId[nodeId], null, function (descendantNode) {
            var nodeEditorData = _this.bySceneNodeId[descendantNode.id];
            nodeEditorData.markerActor.setGlobalPosition(nodeEditorData.actor.getGlobalPosition(tmpVector3));
            nodeEditorData.markerActor.setGlobalOrientation(nodeEditorData.actor.getGlobalOrientation(tmpQuaternion));
        });
    };
    SceneUpdater.prototype.recurseClearActor = function (nodeId) {
        var nodeEditorData = this.bySceneNodeId[nodeId];
        if (nodeEditorData.prefabUpdater == null) {
            for (var _i = 0, _a = nodeEditorData.actor.children; _i < _a.length; _i++) {
                var childActor = _a[_i];
                var sceneNodeId = childActor.sceneNodeId;
                if (sceneNodeId != null)
                    this.recurseClearActor(sceneNodeId);
            }
        }
        else {
            nodeEditorData.prefabUpdater.destroy();
        }
        for (var componentId in nodeEditorData.bySceneComponentId) {
            nodeEditorData.bySceneComponentId[componentId].componentUpdater.destroy();
        }
        if (!this.isInPrefab)
            this.gameInstance.destroyActor(nodeEditorData.markerActor);
        this.gameInstance.destroyActor(nodeEditorData.actor);
        delete this.bySceneNodeId[nodeId];
    };
    SceneUpdater.prototype.config_setProperty = function (path, value) {
        switch (path) {
            case "sceneAssetId":
                if (this.sceneAssetId != null)
                    this.client.unsubAsset(this.sceneAssetId, this.sceneSubscriber);
                this.sceneAssetId = value;
                this.clearScene();
                this.sceneAsset = null;
                if (this.sceneAssetId != null)
                    this.client.subAsset(this.sceneAssetId, "scene", this.sceneSubscriber);
                break;
        }
    };
    SceneUpdater.prototype.createNodeActor = function (node) {
        var parentNode = this.sceneAsset.nodes.parentNodesById[node.id];
        var parentActor;
        if (parentNode != null)
            parentActor = this.bySceneNodeId[parentNode.id].actor;
        else
            parentActor = this.rootActor;
        var nodeActor = new SupEngine.Actor(this.gameInstance, node.name, parentActor);
        var nodeId = (this.rootActor == null) ? node.id : this.rootActor.threeObject.userData.nodeId;
        nodeActor.threeObject.userData.nodeId = nodeId;
        nodeActor.threeObject.position.copy(node.position);
        nodeActor.threeObject.quaternion.copy(node.orientation);
        nodeActor.threeObject.scale.copy(node.scale);
        nodeActor.threeObject.updateMatrixWorld(false);
        nodeActor.sceneNodeId = node.id;
        var markerActor;
        if (!this.isInPrefab) {
            markerActor = new SupEngine.Actor(this.gameInstance, nodeId + " Marker", null, { layer: -1 });
            markerActor.setGlobalPosition(nodeActor.getGlobalPosition(tmpVector3));
            markerActor.setGlobalOrientation(nodeActor.getGlobalOrientation(tmpQuaternion));
            new SupEngine.editorComponentClasses["TransformMarker"](markerActor);
        }
        this.bySceneNodeId[node.id] = { actor: nodeActor, markerActor: markerActor, bySceneComponentId: {}, prefabUpdater: null };
        if (node.prefab != null) {
            this.bySceneNodeId[node.id].prefabUpdater = new SceneUpdater(this.client, { gameInstance: this.gameInstance, actor: nodeActor }, { sceneAssetId: node.prefab.sceneAssetId, isInPrefab: true });
        }
        if (node.components != null)
            for (var _i = 0, _a = node.components; _i < _a.length; _i++) {
                var component = _a[_i];
                this.createNodeActorComponent(node, component, nodeActor);
            }
        return nodeActor;
    };
    SceneUpdater.prototype.createNodeActorComponent = function (sceneNode, sceneComponent, nodeActor) {
        var componentClass = SupEngine.editorComponentClasses[(sceneComponent.type + "Marker")];
        if (componentClass == null)
            componentClass = SupEngine.componentClasses[sceneComponent.type];
        var actorComponent = new componentClass(nodeActor);
        this.bySceneNodeId[sceneNode.id].bySceneComponentId[sceneComponent.id] = {
            component: actorComponent,
            componentUpdater: new componentClass.Updater(this.client, actorComponent, sceneComponent.config),
        };
    };
    SceneUpdater.prototype.clearScene = function () {
        for (var sceneNodeId in this.bySceneNodeId) {
            var sceneNode = this.bySceneNodeId[sceneNodeId];
            if (!this.isInPrefab)
                this.gameInstance.destroyActor(sceneNode.markerActor);
            for (var componentId in sceneNode.bySceneComponentId)
                sceneNode.bySceneComponentId[componentId].componentUpdater.destroy();
            this.gameInstance.destroyActor(sceneNode.actor);
        }
        this.bySceneNodeId = {};
    };
    return SceneUpdater;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = SceneUpdater;
