"use strict";
var SpriteRendererEditor = (function () {
    function SpriteRendererEditor(tbody, config, projectClient, editConfig) {
        var _this = this;
        this.tbody = tbody;
        this.projectClient = projectClient;
        this.editConfig = editConfig;
        this.spriteAssetId = config.spriteAssetId;
        this.animationId = config.animationId;
        this.shaderAssetId = config.shaderAssetId;
        this.overrideOpacity = config.overrideOpacity;
        this.opacity = config.opacity;
        var spriteRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.sprite"));
        this.spriteFieldSubscriber = SupClient.table.appendAssetField(spriteRow.valueCell, this.spriteAssetId, "sprite", projectClient);
        this.spriteFieldSubscriber.on("select", function (assetId) {
            _this.editConfig("setProperty", "spriteAssetId", assetId);
            _this.editConfig("setProperty", "animationId", null);
        });
        var animationRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.animation"));
        this.animationSelectBox = SupClient.table.appendSelectBox(animationRow.valueCell, { "": SupClient.i18n.t("componentEditors:SpriteRenderer.animationNone") });
        this.animationSelectBox.addEventListener("change", function (event) {
            var animationId = (event.target.value === "") ? null : event.target.value;
            _this.editConfig("setProperty", "animationId", animationId);
        });
        this.animationSelectBox.disabled = true;
        var flipRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.flip"));
        var flipDiv = document.createElement("div");
        flipDiv.classList.add("inputs");
        flipRow.valueCell.appendChild(flipDiv);
        var horizontalSpan = document.createElement("span");
        horizontalSpan.style.marginLeft = "5px";
        horizontalSpan.textContent = "H";
        flipDiv.appendChild(horizontalSpan);
        this.horizontalFlipField = SupClient.table.appendBooleanField(flipDiv, config.horizontalFlip);
        this.horizontalFlipField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "horizontalFlip", event.target.checked);
        });
        var verticalSpan = document.createElement("span");
        verticalSpan.style.marginLeft = "5px";
        verticalSpan.textContent = "V";
        flipDiv.appendChild(verticalSpan);
        this.verticalFlipField = SupClient.table.appendBooleanField(flipDiv, config.verticalFlip);
        this.verticalFlipField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "verticalFlip", event.target.checked);
        });
        var shadowRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.shadow.title"));
        var shadowDiv = document.createElement("div");
        shadowDiv.classList.add("inputs");
        shadowRow.valueCell.appendChild(shadowDiv);
        var castSpan = document.createElement("span");
        castSpan.style.marginLeft = "5px";
        castSpan.textContent = SupClient.i18n.t("componentEditors:SpriteRenderer.shadow.cast");
        shadowDiv.appendChild(castSpan);
        this.castShadowField = SupClient.table.appendBooleanField(shadowDiv, config.castShadow);
        this.castShadowField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "castShadow", event.target.checked);
        });
        var receiveSpan = document.createElement("span");
        receiveSpan.style.marginLeft = "5px";
        receiveSpan.textContent = SupClient.i18n.t("componentEditors:SpriteRenderer.shadow.receive");
        shadowDiv.appendChild(receiveSpan);
        this.receiveShadowField = SupClient.table.appendBooleanField(shadowDiv, config.receiveShadow);
        this.receiveShadowField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "receiveShadow", event.target.checked);
        });
        var colorRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.color"));
        var colorInputs = SupClient.table.appendColorField(colorRow.valueCell, config.color);
        this.colorField = colorInputs.textField;
        this.colorField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "color", event.target.value);
        });
        this.colorPicker = colorInputs.pickerField;
        this.colorPicker.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "color", event.target.value.slice(1));
        });
        var opacityRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.opacity"), { checkbox: true });
        this.overrideOpacityField = opacityRow.checkbox;
        this.overrideOpacityField.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "opacity", _this.asset != null ? _this.asset.pub.opacity : null);
            _this.editConfig("setProperty", "overrideOpacity", event.target.checked);
        });
        var opacityParent = document.createElement("div");
        opacityRow.valueCell.appendChild(opacityParent);
        var transparentOptions = {
            empty: "",
            opaque: SupClient.i18n.t("componentEditors:SpriteRenderer.opaque"),
            transparent: SupClient.i18n.t("componentEditors:SpriteRenderer.transparent"),
        };
        this.transparentField = SupClient.table.appendSelectBox(opacityParent, transparentOptions);
        this.transparentField.children[0].hidden = true;
        this.transparentField.addEventListener("change", function (event) {
            var opacity = _this.transparentField.value === "transparent" ? 1 : null;
            _this.editConfig("setProperty", "opacity", opacity);
        });
        this.opacityFields = SupClient.table.appendSliderField(opacityParent, "", { min: 0, max: 1, step: 0.1, sliderStep: 0.01 });
        this.opacityFields.numberField.parentElement.addEventListener("input", function (event) {
            _this.editConfig("setProperty", "opacity", parseFloat(event.target.value));
        });
        this.updateOpacityField();
        var materialRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.material"));
        this.materialSelectBox = SupClient.table.appendSelectBox(materialRow.valueCell, { "basic": "Basic", "phong": "Phong", "shader": "Shader" }, config.materialType);
        this.materialSelectBox.addEventListener("change", function (event) {
            _this.editConfig("setProperty", "materialType", event.target.value);
        });
        var shaderRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("componentEditors:SpriteRenderer.shader"));
        this.shaderRow = shaderRow.row;
        this.shaderFieldSubscriber = SupClient.table.appendAssetField(shaderRow.valueCell, this.shaderAssetId, "shader", projectClient);
        this.shaderFieldSubscriber.on("select", function (assetId) {
            _this.editConfig("setProperty", "shaderAssetId", assetId);
        });
        this.shaderRow.hidden = config.materialType !== "shader";
        if (this.spriteAssetId != null)
            this.projectClient.subAsset(this.spriteAssetId, "sprite", this);
    }
    SpriteRendererEditor.prototype.destroy = function () {
        this.spriteFieldSubscriber.destroy();
        this.shaderFieldSubscriber.destroy();
        if (this.spriteAssetId != null)
            this.projectClient.unsubAsset(this.spriteAssetId, this);
    };
    SpriteRendererEditor.prototype.config_setProperty = function (path, value) {
        if (this.projectClient.entries == null)
            return;
        switch (path) {
            case "spriteAssetId":
                if (this.spriteAssetId != null) {
                    this.projectClient.unsubAsset(this.spriteAssetId, this);
                    this.asset = null;
                }
                this.spriteAssetId = value;
                this.animationSelectBox.disabled = true;
                if (this.spriteAssetId != null)
                    this.projectClient.subAsset(this.spriteAssetId, "sprite", this);
                this.spriteFieldSubscriber.onChangeAssetId(this.spriteAssetId);
                break;
            case "animationId":
                if (!this.animationSelectBox.disabled)
                    this.animationSelectBox.value = (value != null) ? value : "";
                this.animationId = value;
                break;
            case "horizontalFlip":
                this.horizontalFlipField.checked = value;
                break;
            case "verticalFlip":
                this.verticalFlipField.checked = value;
                break;
            case "castShadow":
                this.castShadowField.checked = value;
                break;
            case "receiveShadow":
                this.receiveShadowField.checked = value;
                break;
            case "color":
                this.colorField.value = value;
                this.colorPicker.value = "#" + value;
                break;
            case "overrideOpacity":
                this.overrideOpacity = value;
                this.updateOpacityField();
                break;
            case "opacity":
                this.opacity = value;
                this.updateOpacityField();
                break;
            case "materialType":
                this.materialSelectBox.value = value;
                this.shaderRow.hidden = value !== "shader";
                break;
            case "shaderAssetId":
                this.shaderAssetId = value;
                this.shaderFieldSubscriber.onChangeAssetId(this.shaderAssetId);
                break;
        }
    };
    // Network callbacks
    SpriteRendererEditor.prototype.onAssetReceived = function (assetId, asset) {
        if (assetId !== this.spriteAssetId)
            return;
        this.asset = asset;
        this._clearAnimations();
        for (var _i = 0, _a = this.asset.pub.animations; _i < _a.length; _i++) {
            var animation = _a[_i];
            SupClient.table.appendSelectOption(this.animationSelectBox, animation.id, animation.name);
        }
        this.animationSelectBox.value = (this.animationId != null) ? this.animationId : "";
        this.animationSelectBox.disabled = false;
        this.updateOpacityField();
    };
    SpriteRendererEditor.prototype.onAssetEdited = function (assetId, command) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        if (assetId !== this.spriteAssetId)
            return;
        if (command === "setProperty" && args[0] === "opacity")
            this.updateOpacityField();
        if (command.indexOf("Animation") === -1)
            return;
        var animationId = this.animationSelectBox.value;
        this._clearAnimations();
        for (var _a = 0, _b = this.asset.pub.animations; _a < _b.length; _a++) {
            var animation = _b[_a];
            SupClient.table.appendSelectOption(this.animationSelectBox, animation.id, animation.name);
        }
        if (animationId != null && this.asset.animations.byId[animationId] != null)
            this.animationSelectBox.value = animationId;
        else
            this.editConfig("setProperty", "animationId", "");
    };
    SpriteRendererEditor.prototype.onAssetTrashed = function () {
        this.asset = null;
        this._clearAnimations();
        this.animationSelectBox.value = "";
        this.animationSelectBox.disabled = true;
    };
    // User interface
    SpriteRendererEditor.prototype._clearAnimations = function () {
        while (true) {
            var child = this.animationSelectBox.children[1];
            if (child == null)
                break;
            this.animationSelectBox.removeChild(child);
        }
    };
    SpriteRendererEditor.prototype.updateOpacityField = function () {
        this.overrideOpacityField.checked = this.overrideOpacity;
        this.transparentField.disabled = !this.overrideOpacity;
        this.opacityFields.sliderField.disabled = !this.overrideOpacity;
        this.opacityFields.numberField.disabled = !this.overrideOpacity;
        if (!this.overrideOpacity && this.asset == null) {
            this.transparentField.value = "empty";
            this.opacityFields.numberField.parentElement.hidden = true;
        }
        else {
            var opacity = this.overrideOpacity ? this.opacity : this.asset.pub.opacity;
            if (opacity != null) {
                this.transparentField.value = "transparent";
                this.opacityFields.numberField.parentElement.hidden = false;
                this.opacityFields.sliderField.value = opacity.toString();
                this.opacityFields.numberField.value = opacity.toString();
            }
            else {
                this.transparentField.value = "opaque";
                this.opacityFields.numberField.parentElement.hidden = true;
            }
        }
    };
    return SpriteRendererEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = SpriteRendererEditor;
