"use strict";
var SpriteRendererUpdater = (function () {
    function SpriteRendererUpdater(client, spriteRenderer, config, externalSubscriber) {
        var _this = this;
        this.client = client;
        this.spriteRenderer = spriteRenderer;
        this.externalSubscriber = externalSubscriber;
        this.looping = true;
        this.overrideOpacity = false;
        this.onSpriteAssetReceived = function (assetId, asset) {
            if (!_this.overrideOpacity)
                _this.spriteRenderer.opacity = asset.pub.opacity;
            _this.prepareMaps(asset.pub.textures, function () {
                _this.spriteAsset = asset;
                _this.setSprite();
                if (_this.externalSubscriber.onAssetReceived != null)
                    _this.externalSubscriber.onAssetReceived(assetId, asset);
            });
        };
        this.onSpriteAssetEdited = function (assetId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var callEditCallback = true;
            var commandFunction = _this.onEditCommands[command];
            if (commandFunction != null && commandFunction.apply(_this, args) === false)
                callEditCallback = false;
            if (callEditCallback && _this.externalSubscriber.onAssetEdited != null) {
                (_a = _this.externalSubscriber).onAssetEdited.apply(_a, [assetId, command].concat(args));
            }
            var _a;
        };
        this.onEditCommands = {
            setMaps: function (maps) {
                // TODO: Only update the maps that changed, don't recreate the whole model
                _this.prepareMaps(_this.spriteAsset.pub.textures, function () {
                    _this.setSprite();
                    if (_this.externalSubscriber.onAssetEdited != null) {
                        _this.externalSubscriber.onAssetEdited(_this.spriteAsset.id, "setMaps");
                    }
                });
                return false;
            },
            setMapSlot: function (slot, name) { _this.setSprite(); },
            deleteMap: function (name) { _this.setSprite(); },
            setProperty: function (path, value) {
                switch (path) {
                    case "filtering":
                        break;
                    case "opacity":
                        if (!_this.overrideOpacity)
                            _this.spriteRenderer.setOpacity(value);
                        break;
                    case "alphaTest":
                        _this.spriteRenderer.material.alphaTest = value;
                        _this.spriteRenderer.material.needsUpdate = true;
                        break;
                    case "pixelsPerUnit":
                    case "origin.x":
                    case "origin.y":
                        _this.spriteRenderer.updateShape();
                        break;
                    default:
                        _this.setSprite();
                        break;
                }
            },
            newAnimation: function () {
                _this.spriteRenderer.updateAnimationsByName();
                _this.playAnimation();
            },
            deleteAnimation: function () {
                _this.spriteRenderer.updateAnimationsByName();
                _this.playAnimation();
            },
            setAnimationProperty: function () {
                _this.spriteRenderer.updateAnimationsByName();
                _this.playAnimation();
            }
        };
        this.onSpriteAssetTrashed = function (assetId) {
            _this.spriteAsset = null;
            _this.spriteRenderer.setSprite(null);
            if (_this.externalSubscriber.onAssetTrashed != null)
                _this.externalSubscriber.onAssetTrashed(assetId);
        };
        this.onShaderAssetReceived = function (assetId, asset) {
            _this.shaderPub = asset.pub;
            _this.setSprite();
        };
        this.onShaderAssetEdited = function (id, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            if (command !== "editVertexShader" && command !== "editFragmentShader")
                _this.setSprite();
        };
        this.onShaderAssetTrashed = function () {
            _this.shaderPub = null;
            _this.setSprite();
        };
        this.spriteAssetId = config.spriteAssetId;
        this.animationId = config.animationId;
        this.materialType = config.materialType;
        this.shaderAssetId = config.shaderAssetId;
        if (this.externalSubscriber == null)
            this.externalSubscriber = {};
        this.spriteRenderer.horizontalFlip = config.horizontalFlip;
        this.spriteRenderer.verticalFlip = config.verticalFlip;
        this.spriteRenderer.castShadow = config.castShadow;
        this.spriteRenderer.receiveShadow = config.receiveShadow;
        this.overrideOpacity = config.overrideOpacity;
        this.opacity = config.opacity;
        if (this.overrideOpacity)
            this.spriteRenderer.setOpacity(this.opacity);
        var hex = parseInt(config.color, 16);
        var r = (hex >> 16 & 255) / 255;
        var g = (hex >> 8 & 255) / 255;
        var b = (hex & 255) / 255;
        this.spriteRenderer.setColor(r, g, b);
        this.spriteSubscriber = {
            onAssetReceived: this.onSpriteAssetReceived,
            onAssetEdited: this.onSpriteAssetEdited,
            onAssetTrashed: this.onSpriteAssetTrashed
        };
        this.shaderSubscriber = {
            onAssetReceived: this.onShaderAssetReceived,
            onAssetEdited: this.onShaderAssetEdited,
            onAssetTrashed: this.onShaderAssetTrashed
        };
        if (this.spriteAssetId != null)
            this.client.subAsset(this.spriteAssetId, "sprite", this.spriteSubscriber);
        if (this.shaderAssetId != null)
            this.client.subAsset(this.shaderAssetId, "shader", this.shaderSubscriber);
    }
    SpriteRendererUpdater.prototype.destroy = function () {
        if (this.spriteAssetId != null)
            this.client.unsubAsset(this.spriteAssetId, this.spriteSubscriber);
        if (this.shaderAssetId != null)
            this.client.unsubAsset(this.shaderAssetId, this.shaderSubscriber);
    };
    SpriteRendererUpdater.prototype.prepareMaps = function (textures, callback) {
        var textureNames = Object.keys(textures);
        var texturesToLoad = textureNames.length;
        if (texturesToLoad === 0) {
            callback();
            return;
        }
        function onTextureLoaded() {
            texturesToLoad--;
            if (texturesToLoad === 0)
                callback();
        }
        textureNames.forEach(function (key) {
            var image = textures[key].image;
            if (!image.complete)
                image.addEventListener("load", onTextureLoaded);
            else
                onTextureLoaded();
        });
    };
    SpriteRendererUpdater.prototype.setSprite = function () {
        if (this.spriteAsset == null || (this.materialType === "shader" && this.shaderPub == null)) {
            this.spriteRenderer.setSprite(null);
            return;
        }
        this.spriteRenderer.setSprite(this.spriteAsset.pub, this.materialType, this.shaderPub);
        if (this.animationId != null)
            this.playAnimation();
    };
    SpriteRendererUpdater.prototype.playAnimation = function () {
        var animation = this.spriteAsset.animations.byId[this.animationId];
        if (animation == null)
            return;
        this.spriteRenderer.setAnimation(animation.name, this.looping);
    };
    SpriteRendererUpdater.prototype.config_setProperty = function (path, value) {
        switch (path) {
            case "spriteAssetId":
                if (this.spriteAssetId != null)
                    this.client.unsubAsset(this.spriteAssetId, this.spriteSubscriber);
                this.spriteAssetId = value;
                this.spriteAsset = null;
                this.spriteRenderer.setSprite(null);
                if (this.spriteAssetId != null)
                    this.client.subAsset(this.spriteAssetId, "sprite", this.spriteSubscriber);
                break;
            case "animationId":
                this.animationId = value;
                this.setSprite();
                break;
            case "looping":
                this.looping = value;
                if (this.animationId != null)
                    this.playAnimation();
                break;
            case "horizontalFlip":
                this.spriteRenderer.setHorizontalFlip(value);
                break;
            case "verticalFlip":
                this.spriteRenderer.setVerticalFlip(value);
                break;
            case "castShadow":
                this.spriteRenderer.setCastShadow(value);
                break;
            case "receiveShadow":
                this.spriteRenderer.receiveShadow = value;
                this.spriteRenderer.threeMesh.receiveShadow = value;
                this.spriteRenderer.threeMesh.material.needsUpdate = true;
                break;
            case "color":
                var hex = parseInt(value, 16);
                var r = (hex >> 16 & 255) / 255;
                var g = (hex >> 8 & 255) / 255;
                var b = (hex & 255) / 255;
                this.spriteRenderer.setColor(r, g, b);
                break;
            case "overrideOpacity":
                this.overrideOpacity = value;
                this.spriteRenderer.setOpacity(value ? this.opacity : (this.spriteAsset != null ? this.spriteAsset.pub.opacity : null));
                break;
            case "opacity":
                this.opacity = value;
                this.spriteRenderer.setOpacity(value);
                break;
            case "materialType":
                this.materialType = value;
                this.setSprite();
                break;
            case "shaderAssetId":
                if (this.shaderAssetId != null)
                    this.client.unsubAsset(this.shaderAssetId, this.shaderSubscriber);
                this.shaderAssetId = value;
                this.shaderPub = null;
                this.spriteRenderer.setSprite(null);
                if (this.shaderAssetId != null)
                    this.client.subAsset(this.shaderAssetId, "shader", this.shaderSubscriber);
                break;
        }
    };
    return SpriteRendererUpdater;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = SpriteRendererUpdater;
