"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var path = require("path");
var fs = require("fs");
var async = require("async");
var SpriteAnimations_1 = require("./SpriteAnimations");
// Reference to THREE, client-side only
var THREE;
if (global.window != null && window.SupEngine != null)
    THREE = SupEngine.THREE;
var SpriteAsset = (function (_super) {
    __extends(SpriteAsset, _super);
    function SpriteAsset(id, pub, server) {
        _super.call(this, id, pub, SpriteAsset.schema, server);
    }
    SpriteAsset.prototype.init = function (options, callback) {
        var _this = this;
        this.server.data.resources.acquire("spriteSettings", null, function (err, spriteSettings) {
            _this.server.data.resources.release("spriteSettings", null);
            _this.pub = {
                formatVersion: SpriteAsset.currentFormatVersion,
                maps: { map: new Buffer(0) },
                filtering: spriteSettings.pub.filtering,
                wrapping: "clampToEdge",
                pixelsPerUnit: spriteSettings.pub.pixelsPerUnit,
                framesPerSecond: spriteSettings.pub.framesPerSecond,
                opacity: null,
                alphaTest: spriteSettings.pub.alphaTest,
                frameOrder: "rows",
                grid: { width: 100, height: 100 },
                origin: { x: 0.5, y: 0.5 },
                animations: [],
                mapSlots: {
                    map: "map",
                    light: null,
                    specular: null,
                    alpha: null,
                    normal: null
                }
            };
            _super.prototype.init.call(_this, options, callback);
        });
    };
    SpriteAsset.prototype.setup = function () {
        this.animations = new SpriteAnimations_1.default(this.pub.animations);
    };
    SpriteAsset.prototype.load = function (assetPath) {
        var _this = this;
        var pub;
        var loadMaps = function () {
            var mapsName = pub.maps;
            // NOTE: Support for multiple maps was introduced in Superpowers 0.11
            if (mapsName == null)
                mapsName = ["map"];
            pub.maps = {};
            async.series([
                function (callback) {
                    async.each(mapsName, function (key, cb) {
                        fs.readFile(path.join(assetPath, "map-" + key + ".dat"), function (err, buffer) {
                            // TODO: Handle error but ignore ENOENT
                            if (err != null) {
                                // NOTE: image.dat was renamed to "map-map.dat" in Superpowers 0.11
                                if (err.code === "ENOENT" && key === "map") {
                                    fs.readFile(path.join(assetPath, "image.dat"), function (err, buffer) {
                                        pub.maps[key] = buffer;
                                        fs.writeFile(path.join(assetPath, "map-" + key + ".dat"), buffer);
                                        fs.unlink(path.join(assetPath, "image.dat"));
                                        cb();
                                    });
                                }
                                else
                                    cb();
                                return;
                            }
                            pub.maps[key] = buffer;
                            cb();
                        });
                    }, function (err) { callback(err, null); });
                }
            ], function (err) { _this._onLoaded(assetPath, pub); });
        };
        fs.readFile(path.join(assetPath, "sprite.json"), { encoding: "utf8" }, function (err, json) {
            // NOTE: "asset.json" was renamed to "sprite.json" in Superpowers 0.11
            if (err != null && err.code === "ENOENT") {
                fs.readFile(path.join(assetPath, "asset.json"), { encoding: "utf8" }, function (err, json) {
                    fs.rename(path.join(assetPath, "asset.json"), path.join(assetPath, "sprite.json"), function (err) {
                        pub = JSON.parse(json);
                        loadMaps();
                    });
                });
            }
            else {
                pub = JSON.parse(json);
                loadMaps();
            }
        });
    };
    SpriteAsset.prototype.migrate = function (assetPath, pub, callback) {
        if (pub.formatVersion === SpriteAsset.currentFormatVersion) {
            callback(false);
            return;
        }
        if (pub.formatVersion == null) {
            // NOTE: Opacity setting was introduced in Superpowers 0.8
            if (typeof pub.opacity === "undefined")
                pub.opacity = 1;
            // NOTE: Support for multiple maps was introduced in Superpowers 0.11
            if (pub.frameOrder == null)
                pub.frameOrder = "rows";
            if (pub.advancedTextures == null) {
                pub.advancedTextures = false;
                pub.mapSlots = {
                    map: "map",
                    light: null,
                    specular: null,
                    alpha: null,
                    normal: null
                };
            }
            // NOTE: Animation speed was introduced in Superpowers 0.12
            for (var _i = 0, _a = pub.animations; _i < _a.length; _i++) {
                var animation = _a[_i];
                if (animation.speed == null)
                    animation.speed = 1;
            }
            pub.formatVersion = 1;
        }
        if (pub.formatVersion === 1) {
            delete pub.advancedTextures;
            pub.formatVersion = 2;
        }
        // NOTE : Wrapping was introduced in Superpowers 0.18
        if (pub.formatVersion === 2) {
            pub.wrapping = "clampToEdge";
            pub.formatVersion = 3;
        }
        callback(true);
    };
    SpriteAsset.prototype.client_load = function () {
        this.mapObjectURLs = {};
        this.loadTextures();
    };
    SpriteAsset.prototype.client_unload = function () {
        this.unloadTextures();
    };
    SpriteAsset.prototype.save = function (outputPath, callback) {
        var _this = this;
        this.write(fs.writeFile, outputPath, function (err) {
            if (err != null) {
                callback(err);
                return;
            }
            // Clean up old maps from disk
            async.each(Object.keys(_this.pub.maps), function (key, cb) {
                var value = _this.pub.maps[key];
                if (value != null) {
                    cb();
                    return;
                }
                fs.unlink(path.join(outputPath, "map-" + key + ".dat"), function (err) {
                    if (err != null && err.code !== "ENOENT") {
                        cb(err);
                        return;
                    }
                    cb();
                });
            }, callback);
        });
    };
    SpriteAsset.prototype.clientExport = function (outputPath, callback) {
        this.write(SupApp.writeFile, outputPath, callback);
    };
    SpriteAsset.prototype.write = function (writeFile, outputPath, writeCallback) {
        var maps = this.pub.maps;
        this.pub.maps = [];
        for (var mapName in maps) {
            if (maps[mapName] != null)
                this.pub.maps.push(mapName);
        }
        var textures = this.pub.textures;
        delete this.pub.textures;
        var json = JSON.stringify(this.pub, null, 2);
        this.pub.maps = maps;
        this.pub.textures = textures;
        async.series([
            function (callback) { writeFile(path.join(outputPath, "sprite.json"), json, { encoding: "utf8" }, callback); },
            function (callback) {
                async.each(Object.keys(maps), function (key, cb) {
                    var value = maps[key];
                    if (value == null) {
                        cb();
                        return;
                    }
                    if (value instanceof ArrayBuffer)
                        value = new Buffer(value);
                    writeFile(path.join(outputPath, "map-" + key + ".dat"), value, cb);
                }, callback);
            }
        ], writeCallback);
    };
    SpriteAsset.prototype.unloadTextures = function () {
        for (var textureName in this.pub.textures)
            this.pub.textures[textureName].dispose();
        for (var key in this.mapObjectURLs) {
            URL.revokeObjectURL(this.mapObjectURLs[key]);
            delete this.mapObjectURLs[key];
        }
    };
    SpriteAsset.prototype.loadTextures = function () {
        var _this = this;
        this.unloadTextures();
        this.pub.textures = {};
        Object.keys(this.pub.maps).forEach(function (key) {
            var buffer = _this.pub.maps[key];
            if (buffer == null || buffer.byteLength === 0)
                return;
            var texture = _this.pub.textures[key];
            var image = (texture != null) ? texture.image : null;
            if (image == null) {
                image = new Image;
                texture = _this.pub.textures[key] = new THREE.Texture(image);
                if (_this.pub.filtering === "pixelated") {
                    texture.magFilter = THREE.NearestFilter;
                    texture.minFilter = THREE.NearestFilter;
                }
                if (_this.pub.wrapping === "repeat") {
                    texture.wrapS = SupEngine.THREE.RepeatWrapping;
                    texture.wrapT = SupEngine.THREE.RepeatWrapping;
                }
                else if (_this.pub.wrapping === "mirroredRepeat") {
                    texture.wrapS = SupEngine.THREE.MirroredRepeatWrapping;
                    texture.wrapT = SupEngine.THREE.MirroredRepeatWrapping;
                }
                var typedArray = new Uint8Array(buffer);
                var blob = new Blob([typedArray], { type: "image/*" });
                image.src = _this.mapObjectURLs[key] = URL.createObjectURL(blob);
            }
            if (!image.complete) {
                image.addEventListener("load", function () {
                    // Three.js might resize our texture to make its dimensions power-of-twos
                    // because of WebGL limitations (see https://developer.mozilla.org/en-US/docs/Web/API/WebGL_API/Tutorial/Using_textures_in_WebGL#Non_power-of-two_textures)
                    // so we store its original, non-power-of-two size for later use
                    texture.size = { width: image.width, height: image.height };
                    texture.needsUpdate = true;
                });
            }
        });
    };
    SpriteAsset.prototype.client_setProperty = function (path, value) {
        _super.prototype.client_setProperty.call(this, path, value);
        switch (path) {
            case "filtering":
                for (var textureName in this.pub.textures) {
                    var texture = this.pub.textures[textureName];
                    if (this.pub.filtering === "pixelated") {
                        texture.magFilter = THREE.NearestFilter;
                        texture.minFilter = THREE.NearestFilter;
                    }
                    else {
                        texture.magFilter = THREE.LinearFilter;
                        texture.minFilter = THREE.LinearMipMapLinearFilter;
                    }
                    texture.needsUpdate = true;
                }
                break;
            case "wrapping":
                for (var textureName in this.pub.textures) {
                    var texture = this.pub.textures[textureName];
                    if (value === "clampToEdge") {
                        texture.wrapS = SupEngine.THREE.ClampToEdgeWrapping;
                        texture.wrapT = SupEngine.THREE.ClampToEdgeWrapping;
                    }
                    else if (value === "repeat") {
                        texture.wrapS = SupEngine.THREE.RepeatWrapping;
                        texture.wrapT = SupEngine.THREE.RepeatWrapping;
                    }
                    else if (value === "mirroredRepeat") {
                        texture.wrapS = SupEngine.THREE.MirroredRepeatWrapping;
                        texture.wrapT = SupEngine.THREE.MirroredRepeatWrapping;
                    }
                    texture.needsUpdate = true;
                }
                break;
        }
    };
    SpriteAsset.prototype.server_setMaps = function (client, maps, callback) {
        if (maps == null || typeof maps !== "object") {
            callback("Maps must be an object");
            return;
        }
        for (var key in maps) {
            var value = maps[key];
            if (this.pub.maps[key] == null) {
                callback("The map " + key + " doesn't exist");
                return;
            }
            if (value != null && !(value instanceof Buffer)) {
                callback("Value for " + key + " must be an ArrayBuffer or null");
                return;
            }
        }
        for (var key in maps)
            this.pub.maps[key] = maps[key];
        callback(null, null, maps);
        this.emit("change");
    };
    SpriteAsset.prototype.client_setMaps = function (maps) {
        for (var key in maps)
            this.pub.maps[key] = maps[key];
        this.loadTextures();
    };
    SpriteAsset.prototype.server_newMap = function (client, name, callback) {
        if (name == null || typeof name !== "string") {
            callback("Name of the map must be a string");
            return;
        }
        if (this.pub.maps[name] != null) {
            callback("The map " + name + " already exists");
            return;
        }
        this.pub.maps[name] = new Buffer(0);
        callback(null, null, name);
        this.emit("change");
    };
    SpriteAsset.prototype.client_newMap = function (name) {
        this.pub.maps[name] = new Buffer(0);
    };
    SpriteAsset.prototype.server_deleteMap = function (client, name, callback) {
        if (name == null || typeof name !== "string") {
            callback("Name of the map must be a string");
            return;
        }
        if (this.pub.maps[name] == null) {
            callback("The map " + name + " doesn't exist");
            return;
        }
        if (this.pub.mapSlots["map"] === name) {
            callback("The main map can't be deleted");
            return;
        }
        this.client_deleteMap(name);
        callback(null, null, name);
        this.emit("change");
    };
    SpriteAsset.prototype.client_deleteMap = function (name) {
        for (var slotName in this.pub.mapSlots) {
            var map = this.pub.mapSlots[slotName];
            if (map === name)
                this.pub.mapSlots[slotName] = null;
        }
        // NOTE: do not delete, the key must exist so the file can be deleted from the disk when the asset is saved
        this.pub.maps[name] = null;
    };
    SpriteAsset.prototype.server_renameMap = function (client, oldName, newName, callback) {
        if (oldName == null || typeof oldName !== "string") {
            callback("Name of the map must be a string");
            return;
        }
        if (newName == null || typeof newName !== "string") {
            callback("New name of the map must be a string");
            return;
        }
        if (this.pub.maps[newName] != null) {
            callback("The map " + newName + " already exists");
            return;
        }
        this.client_renameMap(oldName, newName);
        callback(null, null, oldName, newName);
        this.emit("change");
    };
    SpriteAsset.prototype.client_renameMap = function (oldName, newName) {
        this.pub.maps[newName] = this.pub.maps[oldName];
        this.pub.maps[oldName] = null;
        for (var slotName in this.pub.mapSlots) {
            var map = this.pub.mapSlots[slotName];
            if (map === oldName)
                this.pub.mapSlots[slotName] = newName;
        }
    };
    SpriteAsset.prototype.server_setMapSlot = function (client, slot, map, callback) {
        if (slot == null || typeof slot !== "string") {
            callback("Name of the slot must be a string");
            return;
        }
        if (map != null && typeof map !== "string") {
            callback("Name of the map must be a string");
            return;
        }
        if (map != null && this.pub.maps[map] == null) {
            callback("The map " + map + " doesn't exist");
            return;
        }
        if (slot === "map" && map == null) {
            callback("The main map can't be empty");
            return;
        }
        this.pub.mapSlots[slot] = map;
        callback(null, null, slot, map);
        this.emit("change");
    };
    SpriteAsset.prototype.client_setMapSlot = function (slot, map) {
        this.pub.mapSlots[slot] = map;
    };
    SpriteAsset.prototype.server_newAnimation = function (client, name, callback) {
        var _this = this;
        var animation = { id: null, name: name, startFrameIndex: 0, endFrameIndex: 0, speed: 1 };
        this.animations.add(animation, null, function (err, actualIndex) {
            if (err != null) {
                callback(err);
                return;
            }
            animation.name = SupCore.Data.ensureUniqueName(animation.id, animation.name, _this.animations.pub);
            callback(null, animation.id, animation, actualIndex);
            _this.emit("change");
        });
    };
    SpriteAsset.prototype.client_newAnimation = function (animation, actualIndex) {
        this.animations.client_add(animation, actualIndex);
    };
    SpriteAsset.prototype.server_deleteAnimation = function (client, id, callback) {
        var _this = this;
        this.animations.remove(id, function (err) {
            if (err != null) {
                callback(err);
                return;
            }
            callback(null, null, id);
            _this.emit("change");
        });
    };
    SpriteAsset.prototype.client_deleteAnimation = function (id) {
        this.animations.client_remove(id);
        return;
    };
    SpriteAsset.prototype.server_moveAnimation = function (client, id, newIndex, callback) {
        var _this = this;
        this.animations.move(id, newIndex, function (err, actualIndex) {
            if (err != null) {
                callback(err);
                return;
            }
            callback(null, null, id, actualIndex);
            _this.emit("change");
        });
    };
    SpriteAsset.prototype.client_moveAnimation = function (id, newIndex) {
        this.animations.client_move(id, newIndex);
    };
    SpriteAsset.prototype.server_setAnimationProperty = function (client, id, key, value, callback) {
        var _this = this;
        if (key === "name") {
            if (typeof (value) !== "string") {
                callback("Invalid value");
                return;
            }
            value = value.trim();
            if (SupCore.Data.hasDuplicateName(id, value, this.animations.pub)) {
                callback("There's already an animation with this name");
                return;
            }
        }
        this.animations.setProperty(id, key, value, function (err, actualValue) {
            if (err != null) {
                callback(err);
                return;
            }
            callback(null, null, id, key, actualValue);
            _this.emit("change");
        });
    };
    SpriteAsset.prototype.client_setAnimationProperty = function (id, key, actualValue) {
        this.animations.client_setProperty(id, key, actualValue);
    };
    SpriteAsset.currentFormatVersion = 3;
    SpriteAsset.schema = {
        formatVersion: { type: "integer" },
        maps: {
            type: "hash",
            values: {
                type: "buffer",
            }
        },
        filtering: { type: "enum", items: ["pixelated", "smooth"], mutable: true },
        wrapping: { type: "enum", items: ["clampToEdge", "repeat", "mirroredRepeat"], mutable: true },
        pixelsPerUnit: { type: "number", minExcluded: 0, mutable: true },
        framesPerSecond: { type: "number", minExcluded: 0, mutable: true },
        opacity: { type: "number?", min: 0, max: 1, mutable: true },
        alphaTest: { type: "number", min: 0, max: 1, mutable: true },
        frameOrder: { type: "enum", items: ["rows", "columns"], mutable: true },
        grid: {
            type: "hash",
            properties: {
                width: { type: "integer", min: 1, mutable: true },
                height: { type: "integer", min: 1, mutable: true }
            },
            mutable: true
        },
        origin: {
            type: "hash",
            properties: {
                x: { type: "number", min: 0, max: 1, mutable: true },
                y: { type: "number", min: 0, max: 1, mutable: true }
            }
        },
        animations: { type: "array" },
        mapSlots: {
            type: "hash",
            properties: {
                map: { type: "string?", mutable: true },
                light: { type: "string?", mutable: true },
                specular: { type: "string?", mutable: true },
                alpha: { type: "string?", mutable: true },
                normal: { type: "string?", mutable: true }
            }
        }
    };
    return SpriteAsset;
}(SupCore.Data.Base.Asset));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = SpriteAsset;
