"use strict";
var TileMap_1 = require("./TileMap");
var TileSet_1 = require("./TileSet");
var TileMapRendererUpdater = (function () {
    function TileMapRendererUpdater(client, tileMapRenderer, config, externalSubscribers) {
        var _this = this;
        this.client = client;
        this.tileMapRenderer = tileMapRenderer;
        this.externalSubscribers = externalSubscribers;
        this.onTileMapAssetReceived = function (assetId, asset) {
            _this.tileMapAsset = asset;
            _this.setTileMap();
            if (_this.tileMapAsset.pub.tileSetId != null)
                _this.client.subAsset(_this.tileMapAsset.pub.tileSetId, "tileSet", _this.tileSetSubscriber);
            var subscriber = _this.externalSubscribers.tileMap;
            if (subscriber.onAssetReceived != null)
                subscriber.onAssetReceived(assetId, asset);
        };
        this.onTileMapAssetEdited = function (assetId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            if (_this.tileSetAsset != null || command === "changeTileSet") {
                var commandFunction = _this.onEditCommands[command];
                if (commandFunction != null)
                    commandFunction.apply(_this, args);
            }
            var subscriber = _this.externalSubscribers.tileMap;
            if (subscriber.onAssetEdited)
                subscriber.onAssetEdited.apply(subscriber, [assetId, command].concat(args));
        };
        this.onEditCommands = {
            changeTileSet: function () {
                if (_this.tileSetAssetId != null)
                    _this.client.unsubAsset(_this.tileSetAssetId, _this.tileSetSubscriber);
                _this.tileSetAsset = null;
                _this.tileMapRenderer.setTileSet(null);
                _this.tileSetAssetId = _this.tileMapAsset.pub.tileSetId;
                if (_this.tileSetAssetId != null)
                    _this.client.subAsset(_this.tileSetAssetId, "tileSet", _this.tileSetSubscriber);
            },
            resizeMap: function () { _this.setTileMap(); },
            moveMap: function () { _this.tileMapRenderer.refreshEntireMap(); },
            setProperty: function (path, value) {
                switch (path) {
                    case "pixelsPerUnit":
                        _this.tileMapRenderer.refreshPixelsPerUnit(value);
                        break;
                    case "layerDepthOffset":
                        _this.tileMapRenderer.refreshLayersDepth();
                        break;
                }
            },
            editMap: function (layerId, edits) {
                var index = _this.tileMapAsset.pub.layers.indexOf(_this.tileMapAsset.layers.byId[layerId]);
                for (var _i = 0, edits_1 = edits; _i < edits_1.length; _i++) {
                    var edit = edits_1[_i];
                    _this.tileMapRenderer.refreshTileAt(index, edit.x, edit.y);
                }
            },
            newLayer: function (layer, index) { _this.tileMapRenderer.addLayer(layer.id, index); },
            deleteLayer: function (id, index) { _this.tileMapRenderer.deleteLayer(index); },
            moveLayer: function (id, newIndex) { _this.tileMapRenderer.moveLayer(id, newIndex); }
        };
        this.onTileMapAssetTrashed = function (assetId) {
            _this.tileMapRenderer.setTileMap(null);
            var subscriber = _this.externalSubscribers.tileMap;
            if (subscriber.onAssetTrashed != null)
                subscriber.onAssetTrashed(assetId);
        };
        this.onTileSetAssetReceived = function (assetId, asset) {
            _this.prepareTexture(asset.pub.texture, function () {
                _this.tileSetAsset = asset;
                _this.tileMapRenderer.setTileSet(new TileSet_1.default(asset.pub));
                var subscriber = _this.externalSubscribers.tileSet;
                if (subscriber.onAssetReceived != null)
                    subscriber.onAssetReceived(assetId, asset);
            });
        };
        this.onTileSetAssetEdited = function (assetId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var commandFunction = _this.onTileSetEditCommands[command];
            if (commandFunction != null)
                commandFunction.apply(_this, args);
            var subscriber = _this.externalSubscribers.tileSet;
            if (subscriber.onAssetEdited)
                subscriber.onAssetEdited.apply(subscriber, [assetId, command].concat(args));
        };
        this.onTileSetEditCommands = {
            upload: function () {
                var _this = this;
                this.prepareTexture(this.tileSetAsset.pub.texture, function () {
                    _this.tileMapRenderer.setTileSet(new TileSet_1.default(_this.tileSetAsset.pub));
                });
            },
            setProperty: function () {
                this.tileMapRenderer.setTileSet(new TileSet_1.default(this.tileSetAsset.pub));
            }
        };
        this.onTileSetAssetTrashed = function (assetId) {
            _this.tileMapRenderer.setTileSet(null);
            var subscriber = _this.externalSubscribers.tileSet;
            if (subscriber.onAssetTrashed)
                subscriber.onAssetTrashed(assetId);
        };
        this.onShaderAssetReceived = function (assetId, asset) {
            _this.shaderPub = asset.pub;
            _this.setTileMap();
        };
        this.onShaderAssetEdited = function (id, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            if (command !== "editVertexShader" && command !== "editFragmentShader")
                _this.setTileMap();
        };
        this.onShaderAssetTrashed = function () {
            _this.shaderPub = null;
            _this.setTileMap();
        };
        this.tileMapAssetId = config.tileMapAssetId;
        this.tileSetAssetId = config.tileSetAssetId;
        this.materialType = config.materialType;
        this.shaderAssetId = config.shaderAssetId;
        this.tileMapRenderer.receiveShadow = config.receiveShadow;
        if (this.externalSubscribers == null)
            this.externalSubscribers = {};
        if (this.externalSubscribers.tileMap == null)
            this.externalSubscribers.tileMap = {};
        if (this.externalSubscribers.tileSet == null)
            this.externalSubscribers.tileSet = {};
        this.tileMapSubscriber = {
            onAssetReceived: this.onTileMapAssetReceived,
            onAssetEdited: this.onTileMapAssetEdited,
            onAssetTrashed: this.onTileMapAssetTrashed
        };
        this.tileSetSubscriber = {
            onAssetReceived: this.onTileSetAssetReceived,
            onAssetEdited: this.onTileSetAssetEdited,
            onAssetTrashed: this.onTileSetAssetTrashed
        };
        this.shaderSubscriber = {
            onAssetReceived: this.onShaderAssetReceived,
            onAssetEdited: this.onShaderAssetEdited,
            onAssetTrashed: this.onShaderAssetTrashed
        };
        if (this.tileMapAssetId != null)
            this.client.subAsset(this.tileMapAssetId, "tileMap", this.tileMapSubscriber);
        if (this.shaderAssetId != null)
            this.client.subAsset(this.shaderAssetId, "shader", this.shaderSubscriber);
    }
    TileMapRendererUpdater.prototype.destroy = function () {
        if (this.tileMapAssetId != null)
            this.client.unsubAsset(this.tileMapAssetId, this.tileMapSubscriber);
        if (this.tileSetAssetId != null)
            this.client.unsubAsset(this.tileSetAssetId, this.tileSetSubscriber);
        if (this.shaderAssetId != null)
            this.client.unsubAsset(this.shaderAssetId, this.shaderSubscriber);
    };
    TileMapRendererUpdater.prototype.setTileMap = function () {
        if (this.tileMapAsset == null || (this.materialType === "shader" && this.shaderPub == null)) {
            this.tileMapRenderer.setTileMap(null);
            return;
        }
        this.tileMapRenderer.setTileMap(new TileMap_1.default(this.tileMapAsset.pub), this.materialType, this.shaderPub);
    };
    TileMapRendererUpdater.prototype.prepareTexture = function (texture, callback) {
        if (texture == null) {
            callback();
            return;
        }
        if (texture.image.complete)
            callback();
        else
            texture.image.addEventListener("load", callback);
    };
    TileMapRendererUpdater.prototype.config_setProperty = function (path, value) {
        switch (path) {
            case "tileMapAssetId":
                if (this.tileMapAssetId != null)
                    this.client.unsubAsset(this.tileMapAssetId, this.tileMapSubscriber);
                this.tileMapAssetId = value;
                this.tileMapAsset = null;
                this.tileMapRenderer.setTileMap(null);
                if (this.tileSetAssetId != null)
                    this.client.unsubAsset(this.tileSetAssetId, this.tileSetSubscriber);
                this.tileSetAsset = null;
                this.tileMapRenderer.setTileSet(null);
                if (this.tileMapAssetId != null)
                    this.client.subAsset(this.tileMapAssetId, "tileMap", this.tileMapSubscriber);
                break;
            // case "tileSetAssetId":
            case "castShadow":
                this.tileMapRenderer.setCastShadow(value);
                break;
            case "receiveShadow":
                this.tileMapRenderer.setReceiveShadow(value);
                break;
            case "materialType":
                this.materialType = value;
                this.setTileMap();
                break;
            case "shaderAssetId":
                if (this.shaderAssetId != null)
                    this.client.unsubAsset(this.shaderAssetId, this.shaderSubscriber);
                this.shaderAssetId = value;
                this.shaderPub = null;
                this.tileMapRenderer.setTileMap(null);
                if (this.shaderAssetId != null)
                    this.client.subAsset(this.shaderAssetId, "shader", this.shaderSubscriber);
                break;
        }
    };
    return TileMapRendererUpdater;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TileMapRendererUpdater;
