"use strict";
var TileSet_1 = require("./TileSet");
var TileSetRendererUpdater = (function () {
    function TileSetRendererUpdater(client, tileSetRenderer, config, externalSubscriber) {
        var _this = this;
        this.client = client;
        this.tileSetRenderer = tileSetRenderer;
        this.externalSubscriber = externalSubscriber;
        this.onTileSetAssetReceived = function (assetId, asset) {
            _this.prepareTexture(asset.pub.texture, function () {
                _this.tileSetAsset = asset;
                if (asset.pub.texture != null) {
                    _this.tileSetRenderer.setTileSet(new TileSet_1.default(asset.pub));
                    _this.tileSetRenderer.gridRenderer.setGrid({
                        width: asset.pub.texture.image.width / asset.pub.grid.width,
                        height: asset.pub.texture.image.height / asset.pub.grid.height,
                        direction: -1,
                        orthographicScale: 10,
                        ratio: { x: 1, y: asset.pub.grid.width / asset.pub.grid.height }
                    });
                }
                if (_this.externalSubscriber.onAssetReceived != null)
                    _this.externalSubscriber.onAssetReceived(assetId, asset);
            });
        };
        this.onTileSetAssetEdited = function (assetId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var callEditCallback = true;
            var commandFunction = _this.onEditCommands[command];
            if (commandFunction != null && commandFunction.apply(void 0, args) === false)
                callEditCallback = false;
            if (callEditCallback && _this.externalSubscriber.onAssetEdited != null) {
                (_a = _this.externalSubscriber).onAssetEdited.apply(_a, [assetId, command].concat(args));
            }
            var _a;
        };
        this.onEditCommands = {
            upload: function () {
                var texture = _this.tileSetAsset.pub.texture;
                _this.prepareTexture(texture, function () {
                    _this.tileSetRenderer.setTileSet(new TileSet_1.default(_this.tileSetAsset.pub));
                    var width = texture.image.width / _this.tileSetAsset.pub.grid.width;
                    var height = texture.image.height / _this.tileSetAsset.pub.grid.height;
                    _this.tileSetRenderer.gridRenderer.resize(width, height);
                    _this.tileSetRenderer.gridRenderer.setRatio({ x: 1, y: _this.tileSetAsset.pub.grid.width / _this.tileSetAsset.pub.grid.height });
                    if (_this.externalSubscriber.onAssetEdited != null) {
                        _this.externalSubscriber.onAssetEdited(_this.tileSetAsset.id, "upload");
                    }
                });
                return false;
            },
            setProperty: function (key, value) {
                switch (key) {
                    case "grid.width":
                    case "grid.height":
                        _this.tileSetRenderer.refreshScaleRatio();
                        var width = _this.tileSetAsset.pub.texture.image.width / _this.tileSetAsset.pub.grid.width;
                        var height = _this.tileSetAsset.pub.texture.image.height / _this.tileSetAsset.pub.grid.height;
                        _this.tileSetRenderer.gridRenderer.resize(width, height);
                        _this.tileSetRenderer.gridRenderer.setRatio({ x: 1, y: _this.tileSetAsset.pub.grid.width / _this.tileSetAsset.pub.grid.height });
                        break;
                }
            }
        };
        this.onTileSetAssetTrashed = function (assetId) {
            _this.tileSetRenderer.setTileSet(null);
            if (_this.externalSubscriber.onAssetTrashed != null)
                _this.externalSubscriber.onAssetTrashed(assetId);
        };
        this.client = client;
        this.tileSetRenderer = tileSetRenderer;
        this.tileSetAssetId = config.tileSetAssetId;
        if (this.externalSubscriber == null)
            this.externalSubscriber = {};
        this.tileSetSubscriber = {
            onAssetReceived: this.onTileSetAssetReceived,
            onAssetEdited: this.onTileSetAssetEdited,
            onAssetTrashed: this.onTileSetAssetTrashed
        };
        if (this.tileSetAssetId != null)
            this.client.subAsset(this.tileSetAssetId, "tileSet", this.tileSetSubscriber);
    }
    TileSetRendererUpdater.prototype.destroy = function () {
        if (this.tileSetAssetId != null) {
            this.client.unsubAsset(this.tileSetAssetId, this.tileSetSubscriber);
        }
    };
    TileSetRendererUpdater.prototype.changeTileSetId = function (tileSetId) {
        if (this.tileSetAssetId != null)
            this.client.unsubAsset(this.tileSetAssetId, this.tileSetSubscriber);
        this.tileSetAssetId = tileSetId;
        this.tileSetAsset = null;
        this.tileSetRenderer.setTileSet(null);
        this.tileSetRenderer.gridRenderer.resize(1, 1);
        if (this.tileSetAssetId != null)
            this.client.subAsset(this.tileSetAssetId, "tileSet", this.tileSetSubscriber);
    };
    TileSetRendererUpdater.prototype.prepareTexture = function (texture, callback) {
        if (texture == null) {
            callback();
            return;
        }
        if (texture.image.complete)
            callback();
        else
            texture.image.addEventListener("load", callback);
    };
    return TileSetRendererUpdater;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TileSetRendererUpdater;
