"use strict";
var TileMapSettingsEditor = (function () {
    function TileMapSettingsEditor(container, projectClient) {
        var _this = this;
        this.fields = {};
        this.onResourceReceived = function (resourceId, resource) {
            _this.resource = resource;
            for (var setting in resource.pub) {
                if (setting === "formatVersion")
                    continue;
                if (setting === "grid") {
                    _this.fields["grid.width"].value = resource.pub.grid.width.toString();
                    _this.fields["grid.height"].value = resource.pub.grid.height.toString();
                }
                else
                    _this.fields[setting].value = resource.pub[setting];
            }
        };
        this.onResourceEdited = function (resourceId, command, propertyName, value) {
            _this.fields[propertyName].value = value;
        };
        this.projectClient = projectClient;
        var tbody = SupClient.table.createTable(container).tbody;
        this.pixelsPerUnitRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TileMap.pixelsPerUnit"));
        this.fields["pixelsPerUnit"] = SupClient.table.appendNumberField(this.pixelsPerUnitRow.valueCell, "");
        this.defaultWidthRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TileMap.defaultWidth"));
        this.fields["width"] = SupClient.table.appendNumberField(this.defaultWidthRow.valueCell, "");
        this.defaultHeightRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TileMap.defaultHeight"));
        this.fields["height"] = SupClient.table.appendNumberField(this.defaultHeightRow.valueCell, "");
        this.depthOffsetRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TileMap.depthOffset"));
        this.fields["layerDepthOffset"] = SupClient.table.appendNumberField(this.depthOffsetRow.valueCell, "");
        this.gridSizeRow = SupClient.table.appendRow(tbody, SupClient.i18n.t("settingsEditors:TileMap.tileSetGridSize"));
        var gridFields = SupClient.table.appendNumberFields(this.gridSizeRow.valueCell, ["", ""]);
        this.fields["grid.width"] = gridFields[0];
        this.fields["grid.height"] = gridFields[1];
        var fieldNames = Object.keys(this.fields);
        fieldNames.forEach(function (fieldName) {
            var field = _this.fields[fieldName];
            field.addEventListener("change", function (event) {
                _this.projectClient.editResource("tileMapSettings", "setProperty", fieldName, parseInt(event.target.value, 10));
            });
        });
        this.projectClient.subResource("tileMapSettings", this);
    }
    return TileMapSettingsEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TileMapSettingsEditor;
