"use strict";
var BehaviorEditor = (function () {
    function BehaviorEditor(tbody, config, projectClient, editConfig) {
        var _this = this;
        this.onResourceReceived = function (resourceId, resource) {
            _this.behaviorPropertiesResource = resource;
            _this._buildBehaviorPropertiesUI();
        };
        this.onResourceEdited = function (resourceId, command) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            if (command === "setScriptBehaviors" || command === "clearScriptBehaviors")
                _this._buildBehaviorPropertiesUI();
        };
        this.onEntriesReceived = function () { };
        this.onEntryAdded = function () { };
        this.onEntryTrashed = function () { };
        this.onEntryMoved = function (id) {
            if (_this.behaviorPropertiesResource != null &&
                _this.behaviorPropertiesResource.behaviorNamesByScriptId[id] != null) {
                _this._buildBehaviorPropertiesUI();
            }
        };
        this.onSetEntryProperty = function (id, key) {
            if (key === "name" &&
                _this.behaviorPropertiesResource != null &&
                _this.behaviorPropertiesResource.behaviorNamesByScriptId[id] != null) {
                _this._buildBehaviorPropertiesUI();
            }
        };
        this.onChangeBehaviorName = function (event) { _this.editConfig("setProperty", "behaviorName", event.target.value); };
        this.onOpenBehavior = function () {
            var behavior = _this.behaviorPropertiesResource.pub.behaviors[_this.config.behaviorName];
            if (behavior != null)
                SupClient.openEntry(behavior.scriptId, { line: behavior.line != null ? behavior.line : 0, ch: 0 });
        };
        // private onChangePropertySet = (event: any) => {}
        this.onChangePropertyValue = function (event) {
            _this.applyPropertyValueChange(event.target);
        };
        this.onDropPropertyValue = function (event) {
            // When the drop event is called, the value won't have been updated yet since it can be cancelled
            // so we'll call .applyPropertyValueChange after the event has been applied
            setTimeout(function () {
                _this.applyPropertyValueChange(event.target);
            }, 0);
        };
        this.tbody = tbody;
        this.config = config;
        this.projectClient = projectClient;
        this.editConfig = editConfig;
        // Using a <select> rather than <input> + <datalist> because of bugs in Chrome and Electron
        // See https://trello.com/c/jNNRLgdb/651 and https://github.com/atom/electron/issues/360
        var behaviorNameRow = SupClient.table.appendRow(this.tbody, SupClient.i18n.t("componentEditors:Behavior.class"));
        var behaviorDiv = document.createElement("div");
        behaviorDiv.classList.add("inputs");
        behaviorNameRow.valueCell.appendChild(behaviorDiv);
        this.behaviorNameField = SupClient.table.appendSelectBox(behaviorDiv, { "": SupClient.i18n.t("common:none") });
        this.behaviorNameField.addEventListener("change", this.onChangeBehaviorName);
        this.openBehaviorButton = document.createElement("button");
        this.openBehaviorButton.disabled = true;
        this.openBehaviorButton.textContent = SupClient.i18n.t("common:actions.open");
        behaviorDiv.appendChild(this.openBehaviorButton);
        this.openBehaviorButton.addEventListener("click", this.onOpenBehavior);
        SupClient.table.appendHeader(this.tbody, SupClient.i18n.t("componentEditors:Behavior.customizableProperties"));
        this.propertySettingsByName = {};
        this.projectClient.subEntries(this);
        this.projectClient.subResource("behaviorProperties", this);
    }
    BehaviorEditor.prototype.destroy = function () {
        this.projectClient.unsubResource("behaviorProperties", this);
        this.projectClient.unsubEntries(this);
    };
    BehaviorEditor.prototype._buildBehaviorPropertiesUI = function () {
        // Setup behavior list
        while (this.behaviorNameField.childElementCount > 1)
            this.behaviorNameField.removeChild(this.behaviorNameField.lastElementChild);
        var entries = [];
        for (var scriptId in this.behaviorPropertiesResource.behaviorNamesByScriptId) {
            var behaviorNames = this.behaviorPropertiesResource.behaviorNamesByScriptId[scriptId];
            if (behaviorNames.length > 1) {
                entries.push([this.projectClient.entries.getPathFromId(scriptId)].concat(behaviorNames));
            }
            else if (behaviorNames.length === 1) {
                entries.push(behaviorNames[0]);
            }
        }
        entries.sort(function (a, b) {
            if (Array.isArray(a))
                a = a[0];
            if (Array.isArray(b))
                b = b[0];
            return a.localeCompare(b);
        });
        for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
            var entry = entries_1[_i];
            if (Array.isArray(entry)) {
                var group = SupClient.table.appendSelectOptionGroup(this.behaviorNameField, entry[0]);
                for (var _a = 0, _b = entry.slice(1); _a < _b.length; _a++) {
                    var behaviorName_1 = _b[_a];
                    SupClient.table.appendSelectOption(group, behaviorName_1, behaviorName_1);
                }
            }
            else {
                SupClient.table.appendSelectOption(this.behaviorNameField, entry, entry);
            }
        }
        if (this.config.behaviorName.length > 0 && this.behaviorPropertiesResource.pub.behaviors[this.config.behaviorName] == null) {
            SupClient.table.appendSelectOption(this.behaviorNameField, this.config.behaviorName, "(Missing) " + this.config.behaviorName);
            this.openBehaviorButton.disabled = true;
        }
        else
            this.openBehaviorButton.disabled = false;
        this.behaviorNameField.value = this.config.behaviorName;
        // Clear old property settings
        for (var name_1 in this.propertySettingsByName) {
            var propertySetting = this.propertySettingsByName[name_1];
            propertySetting.row.parentElement.removeChild(propertySetting.row);
        }
        this.propertySettingsByName = {};
        // Setup new property settings
        var behaviorName = this.config.behaviorName;
        var listedProperties = [];
        while (behaviorName != null) {
            var behavior = this.behaviorPropertiesResource.pub.behaviors[behaviorName];
            if (behavior == null)
                break;
            for (var _c = 0, _d = behavior.properties; _c < _d.length; _c++) {
                var property = _d[_c];
                if (listedProperties.indexOf(property.name) !== -1)
                    continue;
                listedProperties.push(property.name);
                this._createPropertySetting(property);
            }
            behaviorName = behavior.parentBehavior;
        }
        // TODO: Display and allow cleaning up left-over property values
    };
    BehaviorEditor.prototype._createPropertySetting = function (property) {
        var _this = this;
        var propertySetting = SupClient.table.appendRow(this.tbody, property.name, { checkbox: true, title: property.name + " (" + property.type + ")" });
        this.propertySettingsByName[property.name] = propertySetting;
        this._createPropertyField(property.name);
        propertySetting.checkbox.checked = this.config.propertyValues[property.name] != null;
        propertySetting.checkbox.addEventListener("change", function (event) {
            if (!event.target.checked) {
                _this.editConfig("clearBehaviorPropertyValue", property.name);
                return;
            }
            // defaultValue = property.value someday
            var defaultValue;
            switch (property.type) {
                case "boolean":
                    defaultValue = false;
                    break;
                case "number":
                    defaultValue = 0;
                    break;
                case "string":
                    defaultValue = "";
                    break;
                case "Sup.Math.Vector2":
                    defaultValue = { x: 0, y: 0 };
                    break;
                case "Sup.Math.Vector3":
                    defaultValue = { x: 0, y: 0, z: 0 };
                    break;
                // TODO: Support more types
                default:
                    defaultValue = null;
                    break;
            }
            _this.editConfig("setBehaviorPropertyValue", property.name, property.type, defaultValue);
        });
    };
    BehaviorEditor.prototype._createPropertyField = function (propertyName) {
        var behaviorName = this.config.behaviorName;
        var property;
        while (behaviorName != null) {
            var behavior = this.behaviorPropertiesResource.pub.behaviors[behaviorName];
            property = this.behaviorPropertiesResource.propertiesByNameByBehavior[behaviorName][propertyName];
            if (property != null)
                break;
            behaviorName = behavior.parentBehavior;
        }
        var propertySetting = this.propertySettingsByName[propertyName];
        // TODO: We probably want to collect and display default values?
        // defaultPropertyValue = behaviorProperty?.value
        var propertyValue = null;
        var uiType = property.type;
        var propertyValueInfo = this.config.propertyValues[property.name];
        if (propertyValueInfo != null) {
            propertyValue = propertyValueInfo.value;
            if (propertyValueInfo.type !== property.type)
                uiType = "incompatibleType";
        }
        var propertyFields;
        switch (uiType) {
            case "incompatibleType":
                {
                    var propertyField = propertySetting.valueCell.querySelector("input[type=text]");
                    if (propertyField == null) {
                        propertySetting.valueCell.innerHTML = "";
                        propertyField = SupClient.table.appendTextField(propertySetting.valueCell, "");
                    }
                    propertyField.value = "(Incompatible type: " + propertyValueInfo.type + ")";
                    propertyField.disabled = true;
                    propertyFields = [propertyField];
                }
                break;
            case "boolean":
                {
                    var propertyField = propertySetting.valueCell.querySelector("input[type=checkbox]");
                    if (propertyField == null) {
                        propertySetting.valueCell.innerHTML = "";
                        propertyField = SupClient.table.appendBooleanField(propertySetting.valueCell, false);
                        propertyField.addEventListener("change", this.onChangePropertyValue);
                    }
                    propertyField.checked = propertyValue;
                    propertyField.disabled = propertyValueInfo == null;
                    propertyFields = [propertyField];
                }
                break;
            case "number":
                {
                    var propertyField = propertySetting.valueCell.querySelector("input[type=number]");
                    if (propertyField == null) {
                        propertySetting.valueCell.innerHTML = "";
                        propertyField = SupClient.table.appendNumberField(propertySetting.valueCell, 0);
                        propertyField.addEventListener("change", this.onChangePropertyValue);
                        propertyField.addEventListener("drop", this.onDropPropertyValue);
                    }
                    propertyField.value = propertyValue;
                    propertyField.disabled = propertyValueInfo == null;
                    propertyFields = [propertyField];
                }
                break;
            case "string":
                {
                    var propertyField = propertySetting.valueCell.querySelector("input[type=text]");
                    if (propertyField == null) {
                        propertySetting.valueCell.innerHTML = "";
                        propertyField = SupClient.table.appendTextField(propertySetting.valueCell, "");
                        propertyField.addEventListener("change", this.onChangePropertyValue);
                        propertyField.addEventListener("drop", this.onDropPropertyValue);
                    }
                    propertyField.value = propertyValue;
                    propertyField.disabled = propertyValueInfo == null;
                    propertyFields = [propertyField];
                }
                break;
            case "Sup.Math.Vector2":
            case "Sup.Math.Vector3":
                {
                    var vectorContainer = propertySetting.valueCell.querySelector(".inputs");
                    if (vectorContainer == null) {
                        propertySetting.valueCell.innerHTML = "";
                        var defaultValues = uiType === "Sup.Math.Vector3" ? [0, 0, 0] : [0, 0];
                        propertyFields = SupClient.table.appendNumberFields(propertySetting.valueCell, defaultValues);
                        for (var _i = 0, propertyFields_1 = propertyFields; _i < propertyFields_1.length; _i++) {
                            var field = propertyFields_1[_i];
                            field.addEventListener("change", this.onChangePropertyValue);
                            field.addEventListener("drop", this.onDropPropertyValue);
                        }
                    }
                    else {
                        propertyFields = Array.prototype.slice.call(vectorContainer.querySelectorAll("input"));
                    }
                    propertyFields[0].value = (propertyValue != null) ? propertyValue.x : "";
                    propertyFields[1].value = (propertyValue != null) ? propertyValue.y : "";
                    if (uiType === "Sup.Math.Vector3")
                        propertyFields[2].value = (propertyValue != null) ? propertyValue.z : "";
                    for (var _a = 0, propertyFields_2 = propertyFields; _a < propertyFields_2.length; _a++) {
                        var field = propertyFields_2[_a];
                        field.disabled = propertyValueInfo == null;
                    }
                }
                break;
            // TODO: Support more types
            default: {
                propertySetting.valueCell.innerHTML = "";
                console.error("Unsupported property type: " + property.type);
                return;
            }
        }
        for (var _b = 0, propertyFields_3 = propertyFields; _b < propertyFields_3.length; _b++) {
            var field = propertyFields_3[_b];
            field.dataset["behaviorPropertyName"] = property.name;
            field.dataset["behaviorPropertyType"] = property.type;
        }
    };
    BehaviorEditor.prototype.config_setProperty = function (path, value) {
        switch (path) {
            case "behaviorName": {
                this.behaviorNameField.value = value;
                this._buildBehaviorPropertiesUI();
                break;
            }
        }
    };
    BehaviorEditor.prototype.config_setBehaviorPropertyValue = function (name, type, value) {
        this.propertySettingsByName[name].checkbox.checked = true;
        this._createPropertyField(name);
    };
    BehaviorEditor.prototype.config_clearBehaviorPropertyValue = function (name) {
        this.propertySettingsByName[name].checkbox.checked = false;
        this._createPropertyField(name);
    };
    BehaviorEditor.prototype.applyPropertyValueChange = function (target) {
        var propertyName = target.dataset["behaviorPropertyName"];
        var propertyType = target.dataset["behaviorPropertyType"];
        var propertyValue;
        switch (propertyType) {
            case "boolean":
                propertyValue = target.checked;
                break;
            case "number":
                propertyValue = parseFloat(target.value);
                break;
            case "string":
                propertyValue = target.value;
                break;
            case "Sup.Math.Vector2":
            case "Sup.Math.Vector3":
                {
                    var parent_1 = target.parentElement;
                    propertyValue = {
                        x: parseFloat(parent_1.children[0].value),
                        y: parseFloat(parent_1.children[1].value)
                    };
                    if (propertyType === "Sup.Math.Vector3")
                        propertyValue.z = parseFloat(parent_1.children[2].value);
                }
                break;
            default:
                console.error("Unsupported property type: " + propertyType);
                break;
        }
        this.editConfig("setBehaviorPropertyValue", propertyName, propertyType, propertyValue, function (err) {
            if (err != null) {
                new SupClient.Dialogs.InfoDialog(err);
                return;
            }
        });
    };
    ;
    return BehaviorEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = BehaviorEditor;
