"use strict";
var P2BodyEditor = (function () {
    function P2BodyEditor(tbody, config, projectClient, editConfig) {
        var _this = this;
        this.fields = {};
        this.tbody = tbody;
        this.projectClient = projectClient;
        this.editConfig = editConfig;
        var massRow = SupClient.table.appendRow(this.tbody, SupClient.i18n.t("componentEditors:P2Body.mass"));
        this.fields["mass"] = SupClient.table.appendNumberField(massRow.valueCell, config.mass, { min: 0 });
        this.fields["mass"].addEventListener("change", function (event) {
            _this.editConfig("setProperty", "mass", parseFloat(event.target.value));
        });
        var fixedRotationRow = SupClient.table.appendRow(this.tbody, SupClient.i18n.t("componentEditors:P2Body.fixedRotation"));
        this.fields["fixedRotation"] = SupClient.table.appendBooleanField(fixedRotationRow.valueCell, config.fixedRotation);
        this.fields["fixedRotation"].addEventListener("click", function (event) {
            _this.editConfig("setProperty", "fixedRotation", event.target.checked);
        });
        var offsetRow = SupClient.table.appendRow(this.tbody, SupClient.i18n.t("componentEditors:P2Body.offset"));
        var offsetFields = SupClient.table.appendNumberFields(offsetRow.valueCell, [config.offsetX, config.offsetY]);
        this.fields["offsetX"] = offsetFields[0];
        this.fields["offsetX"].addEventListener("change", function (event) {
            _this.editConfig("setProperty", "offsetX", parseFloat(event.target.value));
        });
        this.fields["offsetY"] = offsetFields[1];
        this.fields["offsetY"].addEventListener("change", function (event) {
            _this.editConfig("setProperty", "offsetY", parseFloat(event.target.value));
        });
        var shapeRow = SupClient.table.appendRow(this.tbody, SupClient.i18n.t("componentEditors:P2Body.shape"));
        var shapeOptions = {
            "box": SupClient.i18n.t("componentEditors:P2Body.shapeOptions.box"),
            "circle": SupClient.i18n.t("componentEditors:P2Body.shapeOptions.circle")
        };
        this.fields["shape"] = SupClient.table.appendSelectBox(shapeRow.valueCell, shapeOptions, config.shape);
        this.fields["shape"].addEventListener("change", function (event) {
            _this.editConfig("setProperty", "shape", event.target.value);
        });
        // Box
        this.sizeRow = SupClient.table.appendRow(this.tbody, SupClient.i18n.t("componentEditors:P2Body.size"));
        var sizeFields = SupClient.table.appendNumberFields(this.sizeRow.valueCell, [config.width, config.height], 0);
        this.fields["width"] = sizeFields[0];
        this.fields["width"].addEventListener("change", function (event) {
            _this.editConfig("setProperty", "width", parseFloat(event.target.value));
        });
        this.fields["height"] = sizeFields[1];
        this.fields["height"].addEventListener("change", function (event) {
            _this.editConfig("setProperty", "height", parseFloat(event.target.value));
        });
        // Circle
        this.radiusRow = SupClient.table.appendRow(this.tbody, SupClient.i18n.t("componentEditors:P2Body.radius"));
        this.fields["radius"] = SupClient.table.appendNumberField(this.radiusRow.valueCell, config.radius, { min: 0 });
        this.fields["radius"].addEventListener("change", function (event) {
            _this.editConfig("setProperty", "radius", parseFloat(event.target.value));
        });
        this.updateShapeInput(config.shape);
    }
    P2BodyEditor.prototype.updateShapeInput = function (shape) {
        switch (shape) {
            case "box":
                {
                    this.sizeRow.row.hidden = false;
                    this.radiusRow.row.hidden = true;
                }
                break;
            case "circle":
                {
                    this.sizeRow.row.hidden = true;
                    this.radiusRow.row.hidden = false;
                }
                break;
        }
    };
    P2BodyEditor.prototype.destroy = function () { };
    P2BodyEditor.prototype.config_setProperty = function (path, value) {
        if (path === "fixedRotation")
            this.fields["fixedRotation"].checked = value;
        else
            this.fields[path].value = value;
        if (path === "shape")
            this.updateShapeInput(value);
    };
    return P2BodyEditor;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = P2BodyEditor;
