"use strict";
var fs = require("fs");
var async = require("async");
var mkdirp = require("mkdirp");
SupCore.system.serverBuild = function (server, buildPath, callback) {
    var exportedProject = { name: server.data.manifest.pub.name, assets: server.data.entries.getForStorage() };
    fs.mkdirSync(buildPath + "/assets");
    var assetIdsToExport = [];
    server.data.entries.walk(function (entry, parent) {
        if (entry.type != null)
            assetIdsToExport.push(entry.id);
    });
    async.each(assetIdsToExport, function (assetId, cb) {
        server.data.assets.acquire(assetId, null, function (err, asset) {
            var folderPath = buildPath + "/assets/" + server.data.entries.getStoragePathFromId(assetId);
            mkdirp(folderPath, function (err) {
                asset.save(folderPath, function (err) {
                    server.data.assets.release(assetId, null);
                    cb();
                });
            });
        });
    }, function (err) {
        if (err != null) {
            callback("Could not export all assets");
            return;
        }
        fs.mkdirSync(buildPath + "/resources");
        async.each(Object.keys(server.system.data.resourceClasses), function (resourceId, cb) {
            server.data.resources.acquire(resourceId, null, function (err, resource) {
                var folderPath = buildPath + "/resources/" + resourceId;
                fs.mkdir(folderPath, function (err) {
                    resource.save(folderPath, function (err) {
                        server.data.resources.release(resourceId, null);
                        cb();
                    });
                });
            });
        }, function (err) {
            if (err != null) {
                callback("Could not export all resources");
                return;
            }
            var json = JSON.stringify(exportedProject, null, 2);
            fs.writeFile(buildPath + "/project.json", json, { encoding: "utf8" }, function (err) {
                if (err != null) {
                    callback("Could not save project.json");
                    return;
                }
                callback(null);
            });
        });
    });
};
